searchNodes=[{"doc":"Define reusable function components with HEEx templates. A function component is any function that receives an assigns map as an argument and returns a rendered struct built with the ~H sigil : defmodule MyComponent do use Phoenix.Component def greet ( assigns ) do ~H&quot;&quot;&quot; &lt;p&gt;Hello, &lt;%= @name %&gt;!&lt;/p&gt; &quot;&quot;&quot; end end When invoked within a ~H sigil or HEEx template file: &lt; MyComponent.greet name = &quot;Jane&quot; /&gt; The following HTML is rendered: &lt; p &gt; Hello, Jane! &lt;/ p &gt; If the function component is defined locally, or its module is imported, then the caller can invoke the function directly without specifying the module: &lt; .greet name = &quot;Jane&quot; /&gt; For dynamic values, you can interpolate Elixir expressions into a function component: &lt; .greet name = { @user . name } /&gt; Function components can also accept blocks of HEEx content (more on this later): &lt; .card &gt; &lt; p &gt; This is the body of my card! &lt;/ p &gt; &lt;/ .card &gt; Note how the name attribute automatically becomes the @name assign inside function components. This can be further leveraged by using two higher-level abstractions for us: attributes and slots. Attributes Phoenix.Component provides the attr/3 macro to declare what attributes the proceeding function component expects to receive when invoked: attr :name , :string , required : true def greet ( assigns ) do ~H&quot;&quot;&quot; &lt;p&gt;Hello, &lt;%= @name %&gt;!&lt;/p&gt; &quot;&quot;&quot; end By calling attr/3 , it is now clear that greet/1 requires a string attribute called name present in its assigns map to properly render. Failing to do so will result in a compilation warning: &lt; MyComponent.greet /&gt; &lt;!-- warning: missing required attribute &quot;name&quot; for component MyAppWeb.MyComponent.greet/1 lib/app_web/my_component.ex:15 --&gt; Attributes can provide default values that are automatically merged into the assigns map: attr :name , :string , default : &quot;Bob&quot; Now you can invoke the function component without providing a value for name : &lt; .greet /&gt; Rendering the following HTML: &lt; p &gt; Hello, Bob! &lt;/ p &gt; Accessing an attribute which is required and does not have a default value will fail. You must explicitly declare default: nil or assign a value programmatically with the assign_new/3 function. Multiple attributes can be declared for the same function component: attr :name , :string , required : true attr :age , :integer , required : true def celebrate ( assigns ) do ~H&quot;&quot;&quot; &lt;p&gt; Happy birthday &lt;%= @name %&gt;! You are &lt;%= @age %&gt; years old. &lt;/p&gt; &quot;&quot;&quot; end Allowing the caller to pass multiple values: &lt; .celebrate name = { &quot;Genevieve&quot; } age = { 34 } /&gt; Rendering the following HTML: &lt; p &gt; Happy birthday Genevieve! You are 34 years old. &lt;/ p &gt; Multiple function components can be defined in the same module, with different attributes. In the following example, &lt;Components.greet/&gt; requires a name , but does not require a title , and &lt;Component.heading&gt; requires a title , but does not require a name . defmodule Components do use Phoenix.Component attr :title , :string , required : true def heading ( assigns ) do ~H&quot;&quot;&quot; &lt;h1&gt;&lt;%= @title %&gt;&lt;/h1&gt; &quot;&quot;&quot; end attr :name , :string , required : true def greet ( assigns ) do ~H&quot;&quot;&quot; &lt;p&gt;Hello &lt;%= @name %&gt;&lt;/p&gt; &quot;&quot;&quot; end end With the attr/3 macro you have the core ingredients to create reusable function components. But what if you need your function components to support dynamic attributes, such as common HTML attributes to mix into a component's container? Global attributes Global attributes are a set of attributes that a function component can accept when it declares an attribute of type :global . By default, the set of attributes accepted are those attributes common to all standard HTML tags. See Global attributes for a complete list of attributes. Once a global attribute is declared, any number of attributes in the set can be passed by the caller without having to modify the function component itself. Below is an example of a function component that accepts a dynamic number of global attributes: attr :message , :string , required : true attr :rest , :global def notification ( assigns ) do ~H&quot;&quot;&quot; &lt;span {@rest}&gt;&lt;%= @message %&gt;&lt;/span&gt; &quot;&quot;&quot; end The caller can pass multiple global attributes (such as phx-* bindings or the class attribute): &lt; .notification message = &quot;You&#39;ve got mail!&quot; class = &quot;bg-green-200&quot; phx-click = &quot;close&quot; /&gt; Rendering the following HTML: &lt; span class = &quot;bg-green-200&quot; phx-click = &quot;close&quot; &gt; You&#39;ve got mail! &lt;/ span &gt; Note that the function component did not have to explicitly declare a class or phx-click attribute in order to render. Global attributes can define defaults which are merged with attributes provided by the caller. For example, you may declare a default class if the caller does not provide one: attr :rest , :global , default : %{ class : &quot;bg-blue-200&quot; } Now you can call the function component without a class attribute: &lt; .notification message = &quot;You&#39;ve got mail!&quot; phx-click = &quot;close&quot; /&gt; Rendering the following HTML: &lt; span class = &quot;bg-blue-200&quot; phx-click = &quot;close&quot; &gt; You&#39;ve got mail! &lt;/ span &gt; Note that the global attribute cannot be provided directly and doing so will emit a warning. In other words, this is invalid: &lt; .notification message = &quot;You&#39;ve got mail!&quot; rest = { %{ &quot;phx-click&quot; =&gt; &quot;close&quot; } } /&gt; Included globals You may also specify which attributes are included in addition to the known globals with the :include option. For example to support the form attribute on a button component: # &lt;.button form=&quot;my-form&quot;/&gt; attr :rest , :global , include : ~w(form) slot :inner_block def button ( assigns ) do ~H&quot;&quot;&quot; &lt;button {@rest}&gt;&lt;%= render_slot(@inner_block) %&gt;&lt;/button&gt; &quot;&quot;&quot; end The :include option is useful to apply global additions on a case-by-case basis, but sometimes you want to extend existing components with new global attributes, such as Alpine.js' x- prefixes, which we'll outline next. Custom global attribute prefixes You can extend the set of global attributes by providing a list of attribute prefixes to use Phoenix.Component . Like the default attributes common to all HTML elements, any number of attributes that start with a global prefix will be accepted by function components invoked by the current module. By default, the following prefixes are supported: phx- , aria- , and data- . For example, to support the x- prefix used by Alpine.js , you can pass the :global_prefixes option to use Phoenix.Component : use Phoenix.Component , global_prefixes : ~w(x-) Now all function components invoked by this module will accept any number of attributes prefixed with x- , in addition to the default global prefixes. You can learn more about attributes by reading the documentation for attr/3 . Slots In addition to attributes, function components can accept blocks of HEEx content, referred to as slots. Slots enable further customization of the rendered HTML, as the caller can pass the function component HEEx content they want the component to render. Phoenix.Component provides the slot/3 macro used to declare slots for function components: slot :inner_block , required : true def button ( assigns ) do ~H&quot;&quot;&quot; &lt;button&gt; &lt;%= render_slot(@inner_block) %&gt; &lt;/button&gt; &quot;&quot;&quot; end The expression render_slot(@inner_block) renders the HEEx content. You can invoke this function component like so: &lt; .button &gt; This renders &lt; strong &gt; inside &lt;/ strong &gt; the button! &lt;/ .button &gt; Which renders the following HTML: &lt; button &gt; This renders &lt; strong &gt; inside &lt;/ strong &gt; the button! &lt;/ button &gt; Like the attr/3 macro, using the slot/3 macro will provide compile-time validations. For example, invoking button/1 without a slot of HEEx content will result in a compilation warning being emitted: &lt; .button /&gt; &lt;!-- warning: missing required slot &quot;inner_block&quot; for component MyAppWeb.MyComponent.button/1 lib/app_web/my_component.ex:15 --&gt; The default slot The example above uses the default slot, accessible as an assign named @inner_block , to render HEEx content via the render_slot/2 function. If the values rendered in the slot need to be dynamic, you can pass a second value back to the HEEx content by calling render_slot/2 : slot :inner_block , required : true attr :entries , :list , default : [ ] def unordered_list ( assigns ) do ~H&quot;&quot;&quot; &lt;ul&gt; &lt;%= for entry &lt;- @entries do %&gt; &lt;li&gt;&lt;%= render_slot(@inner_block, entry) %&gt;&lt;/li&gt; &lt;% end %&gt; &lt;/ul&gt; &quot;&quot;&quot; end When invoking the function component, you can use the special attribute :let to take the value that the function component passes back and bind it to a variable: &lt; .unordered_list :let = { fruit } entries = { ~w(apples bananas cherries) } &gt; I like &lt;%= fruit %&gt; ! &lt;/ .unordered_list &gt; Rendering the following HTML: &lt; ul &gt; &lt; li &gt; I like apples! &lt;/ li &gt; &lt; li &gt; I like bananas! &lt;/ li &gt; &lt; li &gt; I like cherries! &lt;/ li &gt; &lt;/ ul &gt; Now the separation of concerns is maintained: the caller can specify multiple values in a list attribute without having to specify the HEEx content that surrounds and separates them. Named slots In addition to the default slot, function components can accept multiple, named slots of HEEx content. For example, imagine you want to create a modal that has a header, body, and footer: slot :header slot :inner_block , required : true slot :footer , required : true def modal ( assigns ) do ~H&quot;&quot;&quot; &lt;div class=&quot;modal&quot;&gt; &lt;div class=&quot;modal-header&quot;&gt; &lt;%= render_slot(@header) || &quot;Modal&quot; %&gt; &lt;/div&gt; &lt;div class=&quot;modal-body&quot;&gt; &lt;%= render_slot(@inner_block) %&gt; &lt;/div&gt; &lt;div class=&quot;modal-footer&quot;&gt; &lt;%= render_slot(@footer) %&gt; &lt;/div&gt; &lt;/div&gt; &quot;&quot;&quot; end You can invoke this function component using the named slot HEEx syntax: &lt; .modal &gt; This is the body, everything not in a named slot is rendered in the default slot. &lt; :footer &gt; This is the bottom of the modal. &lt;/ :footer &gt; &lt;/ .modal &gt; Rendering the following HTML: &lt; div class = &quot;modal&quot; &gt; &lt; div class = &quot;modal-header&quot; &gt; Modal. &lt;/ div &gt; &lt; div class = &quot;modal-body&quot; &gt; This is the body, everything not in a named slot is rendered in the default slot. &lt;/ div &gt; &lt; div class = &quot;modal-footer&quot; &gt; This is the bottom of the modal. &lt;/ div &gt; &lt;/ div &gt; As shown in the example above, render_slot/1 returns nil when an optional slot is declared and none is given. This can be used to attach default behaviour. Slot attributes Unlike the default slot, it is possible to pass a named slot multiple pieces of HEEx content. Named slots can also accept attributes, defined by passing a block to the slot/3 macro. If multiple pieces of content are passed, render_slot/2 will merge and render all the values. Below is a table component illustrating multiple named slots with attributes: slot :column , doc : &quot;Columns with column labels&quot; do attr :label , :string , required : true , doc : &quot;Column label&quot; end attr :rows , :list , default : [ ] def table ( assigns ) do ~H&quot;&quot;&quot; &lt;table&gt; &lt;tr&gt; &lt;%= for col &lt;- @column do %&gt; &lt;th&gt;&lt;%= col.label %&gt;&lt;/th&gt; &lt;% end %&gt; &lt;/tr&gt; &lt;%= for row &lt;- @rows do %&gt; &lt;tr&gt; &lt;%= for col &lt;- @column do %&gt; &lt;td&gt;&lt;%= render_slot(col, row) %&gt;&lt;/td&gt; &lt;% end %&gt; &lt;/tr&gt; &lt;% end %&gt; &lt;/table&gt; &quot;&quot;&quot; end You can invoke this function component like so: &lt; .table rows = { [ %{ name : &quot;Jane&quot; , age : &quot;34&quot; } , %{ name : &quot;Bob&quot; , age : &quot;51&quot; } ] } &gt; &lt; :column :let = { user } label = &quot;Name&quot; &gt; &lt;%= user . name %&gt; &lt;/ :column &gt; &lt; :column :let = { user } label = &quot;Age&quot; &gt; &lt;%= user . age %&gt; &lt;/ :column &gt; &lt;/ .table &gt; Rendering the following HTML: &lt; table &gt; &lt; tr &gt; &lt; th &gt; Name &lt;/ th &gt; &lt; th &gt; Age &lt;/ th &gt; &lt;/ tr &gt; &lt; tr &gt; &lt; td &gt; Jane &lt;/ td &gt; &lt; td &gt; 34 &lt;/ td &gt; &lt;/ tr &gt; &lt; tr &gt; &lt; td &gt; Bob &lt;/ td &gt; &lt; td &gt; 51 &lt;/ td &gt; &lt;/ tr &gt; &lt;/ table &gt; You can learn more about slots and the slot/3 macro in its documentation . Embedding external template files The embed_templates/1 macro can be used to embed .html.heex files as function components. The directory path is based on the current module ( __DIR__ ), and a wildcard pattern may be used to select all files within a directory tree. For example, imagine a directory listing: ├ ─ ─ components . ex ├ ─ ─ cards │ ├ ─ ─ pricing_card . html . heex │ └ ─ ─ features_card . html . heex Then you can embed the page templates in your components.ex module and call them like any other function component: defmodule MyAppWeb.Components do use Phoenix.Component embed_templates &quot;cards/*&quot; def landing_hero ( assigns ) do ~H&quot;&quot;&quot; &lt;.pricing_card /&gt; &lt;.features_card /&gt; &quot;&quot;&quot; end end See embed_templates/1 for more information, including declarative assigns support for embedded templates.","ref":"Phoenix.Component.html","title":"Phoenix.Component","type":"module"},{"doc":"Adds key-value pairs to assigns. The first argument is either a LiveView socket or an assigns map from function components. A keyword list or a map of assigns must be given as argument to be merged into existing assigns. Examples iex&gt; assign ( socket , name : &quot;Elixir&quot; , logo : &quot;💧&quot; ) iex&gt; assign ( socket , %{ name : &quot;Elixir&quot; } )","ref":"Phoenix.Component.html#assign/2","title":"Phoenix.Component.assign/2","type":"function"},{"doc":"Adds a key - value pair to socket_or_assigns . The first argument is either a LiveView socket or an assigns map from function components. Examples iex&gt; assign ( socket , :name , &quot;Elixir&quot; )","ref":"Phoenix.Component.html#assign/3","title":"Phoenix.Component.assign/3","type":"function"},{"doc":"Assigns the given key with value from fun into socket_or_assigns if one does not yet exist. The first argument is either a LiveView socket or an assigns map from function components. This function is useful for lazily assigning values and sharing assigns. We will cover both use cases next. Lazy assigns Imagine you have a function component that accepts a color: &lt; .my_component color = &quot;red&quot; /&gt; The color is also optional, so you can skip it: &lt; .my_component /&gt; In such cases, the implementation can use assign_new to lazily assign a color if none is given. Let's make it so it picks a random one when none is given: def my_component ( assigns ) do assigns = assign_new ( assigns , :bg_color , fn -&gt; Enum . random ( ~w(bg-red-200 bg-green-200 bg-blue-200) ) end ) ~H&quot;&quot;&quot; &lt;div class={@bg_color}&gt; Example &lt;/div&gt; &quot;&quot;&quot; end Sharing assigns It is possible to share assigns between the Plug pipeline and LiveView on disconnected render and between LiveViews when connected. When disconnected When a user first accesses an application using LiveView, the LiveView is first rendered in its disconnected state, as part of a regular HTML response. By using assign_new in the mount callback of your LiveView, you can instruct LiveView to re-use any assigns already set in conn during disconnected state. Imagine you have a Plug that does: # A plug def authenticate ( conn , _opts ) do if user_id = get_session ( conn , :user_id ) do assign ( conn , :current_user , Accounts . get_user! ( user_id ) ) else send_resp ( conn , :forbidden ) end end You can re-use the :current_user assign in your LiveView during the initial render: def mount ( _params , %{ &quot;user_id&quot; =&gt; user_id } , socket ) do { :ok , assign_new ( socket , :current_user , fn -&gt; Accounts . get_user! ( user_id ) end ) } end In such case conn.assigns.current_user will be used if present. If there is no such :current_user assign or the LiveView was mounted as part of the live navigation, where no Plug pipelines are invoked, then the anonymous function is invoked to execute the query instead. When connected LiveView is also able to share assigns via assign_new within nested LiveView. If the parent LiveView defines a :current_user assign and the child LiveView also uses assign_new/3 to fetch the :current_user in its mount/3 callback, as above, the assign will be fetched from the parent LiveView, once again avoiding additional database queries. Note that fun also provides access to the previously assigned values: assigns = assigns |&gt; assign_new ( :foo , fn -&gt; &quot;foo&quot; end ) |&gt; assign_new ( :bar , fn %{ foo : foo } -&gt; foo &lt;&gt; &quot;bar&quot; end )","ref":"Phoenix.Component.html#assign_new/3","title":"Phoenix.Component.assign_new/3","type":"function"},{"doc":"Filters the assigns as a list of keywords for use in dynamic tag attributes. Useful for transforming caller assigns into dynamic attributes while stripping reserved keys from the result. Examples Imagine the following my_link component which allows a caller to pass a new_window assign, along with any other attributes they would like to add to the element, such as class, data attributes, etc: &lt; .my_link to = &quot;/&quot; id = { @id } new_window = { true } class = &quot;my-class&quot; &gt; Home &lt;/ .my_link &gt; We could support the dynamic attributes with the following component: def my_link ( assigns ) do target = if assigns [ :new_window ] , do : &quot;_blank&quot; , else : false extra = assigns_to_attributes ( assigns , [ :new_window , :to ] ) assigns = assigns |&gt; assign ( :target , target ) |&gt; assign ( :extra , extra ) ~H&quot;&quot;&quot; &lt;a href={@to} target={@target} {@extra}&gt; &lt;%= render_slot(@inner_block) %&gt; &lt;/a&gt; &quot;&quot;&quot; end The above would result in the following rendered HTML: &lt; a href = &quot;/&quot; target = &quot;_blank&quot; id = &quot;1&quot; class = &quot;my-class&quot; &gt; Home &lt;/ a &gt; The second argument (optional) to assigns_to_attributes is a list of keys to exclude. It typically includes reserved keys by the component itself, which either do not belong in the markup, or are already handled explicitly by the component.","ref":"Phoenix.Component.html#assigns_to_attributes/2","title":"Phoenix.Component.assigns_to_attributes/2","type":"function"},{"doc":"Declares attributes for a HEEx function components. Arguments name - an atom defining the name of the attribute. Note that attributes cannot define the same name as any other attributes or slots declared for the same component. type - an atom defining the type of the attribute. opts - a keyword list of options. Defaults to [] . Types An attribute is declared by its name, type, and options. The following types are supported: Name Description :any any term :string any binary string :atom any atom (including true , false , and nil ) :boolean any boolean :integer any integer :float any float :list any list of any arbitrary types :map any map of any arbitrary types :global any common HTML attributes, plus those defined by :global_prefixes A struct module any module that defines a struct with defstruct/1 Options :required - marks an attribute as required. If a caller does not pass the given attribute, a compile warning is issued. :default - the default value for the attribute if not provided. If this option is not set and the attribute is not given, accessing the attribute will fail unless a value is explicitly set with assign_new/3 . :examples - a non-exhaustive list of values accepted by the attribute, used for documentation purposes. :values - an exhaustive list of values accepted by the attributes. If a caller passes a literal not contained in this list, a compile warning is issued. :doc - documentation for the attribute. Compile-Time Validations LiveView performs some validation of attributes via the :phoenix_live_view compiler. When attributes are defined, LiveView will warn at compilation time on the caller if: A required attribute of a component is missing. An unknown attribute is given. You specify a literal attribute (such as value=&quot;string&quot; or value , but not value={expr} ) and the type does not match. The following types currently support literal validation: :string , :atom , :boolean , :integer , :float , :map and :list . You specify a literal attribute and it is not a member of the :values list. LiveView does not perform any validation at runtime. This means the type information is mostly used for documentation and reflection purposes. On the side of the LiveView component itself, defining attributes provides the following quality of life improvements: The default value of all attributes will be added to the assigns map upfront. Attribute documentation is generated for the component. Required struct types are annotated and emit compilation warnings. For example, if you specify attr :user, User, required: true and then you write @user.non_valid_field in your template, a warning will be emitted. Calls made to the component are tracked for reflection and validation purposes. Documentation Generation Public function components that define attributes will have their attribute types and docs injected into the function's documentation, depending on the value of the @doc module attribute: if @doc is a string, the attribute docs are injected into that string. The optional placeholder [INSERT LVATTRDOCS] can be used to specify where in the string the docs are injected. Otherwise, the docs are appended to the end of the @doc string. if @doc is unspecified, the attribute docs are used as the default @doc string. if @doc is false , the attribute docs are omitted entirely. The injected attribute docs are formatted as a markdown list: name ( :type ) (required) - attr docs. Defaults to :default . By default, all attributes will have their types and docs injected into the function @doc string. To hide a specific attribute, you can set the value of :doc to false . Example attr :name , :string , required : true attr :age , :integer , required : true def celebrate ( assigns ) do ~H&quot;&quot;&quot; &lt;p&gt; Happy birthday &lt;%= @name %&gt;! You are &lt;%= @age %&gt; years old. &lt;/p&gt; &quot;&quot;&quot; end","ref":"Phoenix.Component.html#attr/3","title":"Phoenix.Component.attr/3","type":"macro"},{"doc":"Checks if the given key changed in socket_or_assigns . The first argument is either a LiveView socket or an assigns map from function components. Examples iex&gt; changed? ( socket , :count )","ref":"Phoenix.Component.html#changed?/2","title":"Phoenix.Component.changed?/2","type":"function"},{"doc":"Generates a dynamically named HTML tag. Raises an ArgumentError if the tag name is found to be unsafe HTML. Attributes name ( :string ) (required) - The name of the tag, such as div . Global attributes are accepted. Slots inner_block Examples &lt; .dynamic_tag name = &quot;input&quot; type = &quot;text&quot; /&gt; &lt; input type = &quot;text&quot; /&gt; &lt; .dynamic_tag name = &quot;p&quot; &gt; content &lt;/ .dynamic_tag &gt; &lt; p &gt; content &lt;/ p &gt;","ref":"Phoenix.Component.html#dynamic_tag/1","title":"Phoenix.Component.dynamic_tag/1","type":"function"},{"doc":"Embeds external template files into the module as function components. Options :root - The root directory to embed files. Defaults to the current module's directory ( __DIR__ ) :suffix - The string value to append to embedded function names. By default, function names will be the name of the template file excluding the format and engine. A wildcard pattern may be used to select all files within a directory tree. For example, imagine a directory listing: ├ ─ ─ components . ex ├ ─ ─ pages │ ├ ─ ─ about_page . html . heex │ └ ─ ─ welcome_page . html . heex Then to embed the page templates in your components.ex module: defmodule MyAppWeb.Components do use Phoenix.Component embed_templates &quot;pages/*&quot; end Now, your module will have an about_page/1 and welcome_page/1 function component defined. Embedded templates also support declarative assigns via bodyless function definitions, for example: defmodule MyAppWeb.Components do use Phoenix.Component embed_templates &quot;pages/*&quot; attr :name , :string , required : true def welcome_page ( assigns ) slot :header def about_page ( assigns ) end Multiple invocations of embed_templates is also supported, which can be useful if you have more than one template format. For example: defmodule MyAppWeb.Emails do use Phoenix.Component embed_templates &quot;emails/*.html&quot; , suffix : &quot;_html&quot; embed_templates &quot;emails/*.text&quot; , suffix : &quot;_text&quot; end Note: this function is the same as Phoenix.Template.embed_templates/2 . It is also provided here for convenience and documentation purposes. Therefore, if you want to embed templates for other formats, which are not related to Phoenix.Component , prefer to import Phoenix.Template, only: [embed_templates: 1] than this module.","ref":"Phoenix.Component.html#embed_templates/2","title":"Phoenix.Component.embed_templates/2","type":"macro"},{"doc":"Wraps tab focus around a container for accessibility. This is an essential accessibility feature for interfaces such as modals, dialogs, and menus. Attributes id ( :string ) (required) - The DOM identifier of the container tag. Global attributes are accepted. Slots inner_block (required) - The content rendered inside of the container tag. Examples Simply render your inner content within this component and focus will be wrapped around the container as the user tabs through the containers content: &lt; .focus_wrap id = &quot;my-modal&quot; class = &quot;bg-white&quot; &gt; &lt; div id = &quot;modal-content&quot; &gt; Are you sure? &lt; button phx-click = &quot;cancel&quot; &gt; Cancel &lt;/ button &gt; &lt; button phx-click = &quot;confirm&quot; &gt; OK &lt;/ button &gt; &lt;/ div &gt; &lt;/ .focus_wrap &gt;","ref":"Phoenix.Component.html#focus_wrap/1","title":"Phoenix.Component.focus_wrap/1","type":"function"},{"doc":"Renders a form. This function receives a form struct, generally created with to_form/2 , and generates the relevant form tags. It can be used either inside LiveView or outside. Examples: inside LiveView Inside LiveViews, the for={...} attribute is generally a form struct created with the to_form/1 function. to_form/1 expects either a map or an Ecto.Changeset as thesource of data. For example, you may use the parameters received in a Phoenix.LiveView.handle_event/3 callback to create an Ecto changeset and then use to_form/1 to convert it to a form. Then, in your templates, you pass the @form as argument to :for : &lt; .form for = { @form } phx-change = &quot;change_name&quot; &gt; &lt; .input field = { @form [ :email ] } /&gt; &lt;/ .form &gt; The .input component is generally defined as part of your own application and adds all styling necessary: def input(assigns) do ~H&quot;&quot;&quot; &lt; input type = &quot;text&quot; name = { @field . name } id = { @field . id } value = { @field . value } class = &quot;...&quot; /&gt; &quot;&quot;&quot; end A form accepts multiple options. For example, if you are doing file uploads and you want to capture submissions, you might write instead: &lt; .form for = { @form } multipart phx-change = &quot;change_user&quot; phx-submit = &quot;save_user&quot; &gt; ... &lt; input type = &quot;submit&quot; value = &quot;Save&quot; /&gt; &lt;/ .form &gt; Notice how both examples use phx-change . The LiveView must implement the phx-change event and store the input values as they arrive on change. This is important because, if an unrelated change happens on the page, LiveView should re-render the inputs with their updated values. Without phx-change , the inputs would otherwise be cleared. Alternatively, you can use phx-update=&quot;ignore&quot; on the form to discard any updates. Using the for attribute The for attribute can also be a map or an Ecto.Changeset. In such cases, a form will be created on the fly, and you can capture it using :let : &lt; .form :let = { form } for = { @changeset } phx-change = &quot;change_user&quot; &gt; However, such approach is discouraged in LiveView for two reasons: LiveView can better optimize your code if you access the form fields using @form[:field] rather than through the let-variable form Ecto changesets are meant to be single use. By never storing the changeset in the assign, you will be less tempted to use it across operations A note on :errors Even if changeset.errors is non-empty, errors will not be displayed in a form if the changeset :action is nil or :ignore . This is useful for things like validation hints on form fields, e.g. an empty changeset for a new form. That changeset isn't valid, but we don't want to show errors until an actual user action has been performed. For example, if the user submits and a Repo.insert/1 is called and fails on changeset validation, the action will be set to :insert to show that an insert was attempted, and the presence of that action will cause errors to be displayed. The same is true for Repo.update/delete. If you want to show errors manually you can also set the action yourself, either directly on the Ecto.Changeset struct field or by using Ecto.Changeset.apply_action/2 . Since the action can be arbitrary, you can set it to :validate or anything else to avoid giving the impression that a database operation has actually been attempted. Example: outside LiveView (regular HTTP requests) The form component can still be used to submit forms outside of LiveView. In such cases, the action attribute MUST be given. Without said attribute, the form method and csrf token are discarded. &lt; .form :let = { f } for = { @changeset } action = { Routes . comment_path ( :create , @comment ) } &gt; &lt; .input field = { f [ :body ] } /&gt; &lt;/ .form &gt; In the example above, we use passed a changeset to for and captured the value using :let={f} . This approach is ok outside of LiveViews, as there are no change tracking optimizations to consider. CSRF protection CSRF protection is a mechanism to ensure that the user who rendered the form is the one actually submitting it. This module generates a CSRF token by default. Your application should check this token on the server to avoid attackers from making requests on your server on behalf of other users. Phoenix by default checks this token. When posting a form with a host in its address, such as &quot;//host.com/path&quot; instead of only &quot;/path&quot;, Phoenix will include the host signature in the token and validate the token only if the accessed host is the same as the host in the token. This is to avoid tokens from leaking to third party applications. If this behaviour is problematic, you can generate a non-host specific token with Plug.CSRFProtection.get_csrf_token/0 and pass it to the form generator via the :csrf_token option. Attributes for ( :any ) (required) - An existing form or the form source data. action ( :string ) - The action to submit the form on. This attribute must be given if you intend to submit the form to a URL without LiveView. Defaults to nil . as ( :atom ) - The server side parameter in which all params for this form will be collected. For example, setting as: :user_params means the parameters will be nested &quot;user_params&quot; in your handle_event or conn.params.user_params for regular HTTP requests. Defaults to nil . multipart ( :boolean ) - Sets enctype to multipart/form-data . Required when uploading files. Defaults to false . method ( :string ) - The HTTP method. It is only used if an :action is given. If the method is not get nor post , an input tag with name _method is generated alongside the form tag. csrf_token ( :any ) - A token to authenticate the validity of requests. One is automatically generated when an action is given and the method is not get . When set to false , no token is generated. Global attributes are accepted. Slots inner_block (required) - The content rendered inside of the form tag.","ref":"Phoenix.Component.html#form/1","title":"Phoenix.Component.form/1","type":"function"},{"doc":"Renders nested form inputs for associations or embeds. Attributes field ( Phoenix.HTML.FormField ) (required) - A %Phoenix.HTML.Form{}/field name tuple, for example: {@form[:email]}. id ( :string ) - The id to be used in the form, defaults to the concatenation of the given field to the parent form id. as ( :atom ) - The name to be used in the form, defaults to the concatenation of the given field to the parent form name. default ( :any ) - The value to use if none is available. prepend ( :list ) - The values to prepend when rendering. This only applies if the field value is a list and no parameters were sent through the form. append ( :list ) - The values to append when rendering. This only applies if the field value is a list and no parameters were sent through the form. skip_hidden ( :boolean ) - Skip the automatic rendering of hidden fields to allow for more tight control over the generated markup. Defaults to false . Slots inner_block (required) - The content rendered for each nested form. Examples &lt; .form :let = { f } phx-change = &quot;change_name&quot; &gt; &lt; .inputs_for :let = { f_nested } field = { f [ :nested ] } &gt; &lt; .input type = &quot;text&quot; field = { f_nested [ :name ] } /&gt; &lt;/ .inputs_for &gt; &lt;/ .form &gt;","ref":"Phoenix.Component.html#inputs_for/1","title":"Phoenix.Component.inputs_for/1","type":"function"},{"doc":"Intersperses separator slot between an enumerable. Useful when you need to add a separator between items such as when rendering breadcrumbs for navigation. Provides each item to the inner block. Examples &lt; .intersperse :let = { item } enum = { [ &quot;home&quot; , &quot;profile&quot; , &quot;settings&quot; ] } &gt; &lt; :separator &gt; &lt; span class = &quot;sep&quot; &gt; | &lt;/ span &gt; &lt;/ :separator &gt; &lt;%= item %&gt; &lt;/ .intersperse &gt; Renders the following markup: home &lt; span class = &quot;sep&quot; &gt; | &lt; / span &gt; profile &lt; span class = &quot;sep&quot; &gt; | &lt; / span &gt; settings Attributes enum ( :any ) (required) - the enumerable to intersperse with separators. Slots inner_block (required) - the inner_block to render for each item. separator (required) - the slot for the separator.","ref":"Phoenix.Component.html#intersperse/1","title":"Phoenix.Component.intersperse/1","type":"function"},{"doc":"Generates a link for live and href navigation. Attributes navigate ( :string ) - Navigates from a LiveView to a new LiveView. The browser page is kept, but a new LiveView process is mounted and its content on the page is reloaded. It is only possible to navigate between LiveViews declared under the same router Phoenix.LiveView.Router.live_session/3 . Otherwise, a full browser redirect is used. patch ( :string ) - Patches the current LiveView. The handle_params callback of the current LiveView will be invoked and the minimum content will be sent over the wire, as any other LiveView diff. href ( :any ) - Uses traditional browser navigation to the new location. This means the whole page is reloaded on the browser. replace ( :boolean ) - When using :patch or :navigate , should the browser's history be replaced with pushState ? Defaults to false . method ( :string ) - The HTTP method to use with the link. This is intended for usage outside of LiveView and therefore only works with the href={...} attribute. It has no effect on patch and navigate instructions. In case the method is not get , the link is generated inside the form which sets the proper information. In order to submit the form, JavaScript must be enabled in the browser. Defaults to &quot;get&quot; . csrf_token ( :any ) - A boolean or custom token to use for links with an HTTP method other than get . Defaults to true . Global attributes are accepted. Slots inner_block (required) - The content rendered inside of the a tag. Examples &lt; .link href = &quot;/&quot; &gt; Regular anchor link &lt;/ .link &gt; &lt; .link navigate = { Routes . page_path ( @socket , :index ) } class = &quot;underline&quot; &gt; home &lt;/ .link &gt; &lt; .link navigate = { Routes . live_path ( @socket , MyLive , dir : :asc ) } replace = { false } &gt; Sort By Price &lt;/ .link &gt; &lt; .link patch = { Routes . page_path ( @socket , :index , :details ) } &gt; view details &lt;/ .link &gt; &lt; .link href = { URI . parse ( &quot;https://elixir-lang.org&quot; ) } &gt; hello &lt;/ .link &gt; &lt; .link href = &quot;/the_world&quot; method = &quot;delete&quot; data-confirm = &quot;Really?&quot; &gt; delete &lt;/ .link &gt; JavaScript dependency In order to support links where :method is not &quot;get&quot; or use the above data attributes, Phoenix.HTML relies on JavaScript. You can load priv/static/phoenix_html.js into your build tool. Data attributes Data attributes are added as a keyword list passed to the data key. The following data attributes are supported: data-confirm - shows a confirmation prompt before generating and submitting the form when :method is not &quot;get&quot; . Overriding the default confirm behaviour phoenix_html.js does trigger a custom event phoenix.link.click on the clicked DOM element when a click happened. This allows you to intercept the event on its way bubbling up to window and do your own custom logic to enhance or replace how the data-confirm attribute is handled. You could for example replace the browsers confirm() behavior with a custom javascript implementation: // listen on document.body, so it's executed before the default of // phoenix_html, which is listening on the window object document.body.addEventListener('phoenix.link.click', function (e) { // Prevent default implementation e.stopPropagation(); // Introduce alternative implementation var message = e.target.getAttribute(&quot;data-confirm&quot;); if(!message){ return true; } vex.dialog.confirm({ message: message, callback: function (value) { if (value == false) { e.preventDefault(); } } }) }, false); Or you could attach your own custom behavior. window.addEventListener('phoenix.link.click', function (e) { // Introduce custom behaviour var message = e.target.getAttribute(&quot;data-prompt&quot;); var answer = e.target.getAttribute(&quot;data-prompt-answer&quot;); if(message &amp;&amp; answer &amp;&amp; (answer != window.prompt(message))) { e.preventDefault(); } }, false); The latter could also be bound to any click event, but this way you can be sure your custom code is only executed when the code of phoenix_html.js is run. CSRF Protection By default, CSRF tokens are generated through Plug.CSRFProtection .","ref":"Phoenix.Component.html#link/1","title":"Phoenix.Component.link/1","type":"function"},{"doc":"A function component for rendering Phoenix.LiveComponent within a parent LiveView. While LiveView s can be nested, each LiveView starts its own process. A LiveComponent provides similar functionality to LiveView , except they run in the same process as the LiveView , with its own encapsulated state. That's why they are called stateful components. Attributes id ( :string ) (required) - A unique identifier for the LiveComponent. Note the id won't necessarily be used as the DOM id . That is up to the component to decide. module ( :atom ) (required) - The LiveComponent module to render. Any additional attributes provided will be passed to the LiveComponent as a map of assigns. See Phoenix.LiveComponent for more information. Examples &lt; .live_component module = { MyApp.WeatherComponent } id = &quot;thermostat&quot; city = &quot;Kraków&quot; /&gt;","ref":"Phoenix.Component.html#live_component/1","title":"Phoenix.Component.live_component/1","type":"function"},{"doc":"Builds a file input tag for a LiveView upload. Attributes :upload - The %Phoenix.LiveView.UploadConfig{} struct. Arbitrary attributes may be passed to be applied to the file input tag. Drag and Drop Drag and drop is supported by annotating the droppable container with a phx-drop-target attribute pointing to the DOM id of the file input. By default, the file input id is the upload ref , so the following markup is all that is required for drag and drop support: &lt; div class = &quot;container&quot; phx-drop-target = { @uploads . avatar . ref } &gt; &lt;!-- ... --&gt; &lt; .live_file_input upload = { @uploads . avatar } /&gt; &lt;/ div &gt; Examples &lt; .live_file_input upload = { @uploads . avatar } /&gt;","ref":"Phoenix.Component.html#live_file_input/1","title":"Phoenix.Component.live_file_input/1","type":"function"},{"doc":"Returns the flash message from the LiveView flash assign. Examples &lt; p class = &quot;alert alert-info&quot; &gt; &lt;%= live_flash ( @flash , :info ) %&gt; &lt;/ p &gt; &lt; p class = &quot;alert alert-danger&quot; &gt; &lt;%= live_flash ( @flash , :error ) %&gt; &lt;/ p &gt;","ref":"Phoenix.Component.html#live_flash/2","title":"Phoenix.Component.live_flash/2","type":"function"},{"doc":"Generates an image preview on the client for a selected file. Examples &lt;%= for entry &lt;- @uploads . avatar . entries do %&gt; &lt; .live_img_preview entry = { entry } width = &quot;75&quot; /&gt; &lt;% end %&gt;","ref":"Phoenix.Component.html#live_img_preview/1","title":"Phoenix.Component.live_img_preview/1","type":"function"},{"doc":"Renders a LiveView within a template. This is useful in two situations: When rendering a child LiveView inside a LiveView. When rendering a LiveView inside a regular (non-live) controller/view. Options :session - a map of binary keys with extra session data to be serialized and sent to the client. All session data currently in the connection is automatically available in LiveViews. You can use this option to provide extra data. Remember all session data is serialized and sent to the client, so you should always keep the data in the session to a minimum. For example, instead of storing a User struct, you should store the &quot;user_id&quot; and load the User when the LiveView mounts. :container - an optional tuple for the HTML tag and DOM attributes to be used for the LiveView container. For example: {:li, style: &quot;color: blue;&quot;} . By default it uses the module definition container. See the &quot;Containers&quot; section below for more information. :id - both the DOM ID and the ID to uniquely identify a LiveView. An :id is automatically generated when rendering root LiveViews but it is a required option when rendering a child LiveView. :sticky - an optional flag to maintain the LiveView across live redirects, even if it is nested within another LiveView. If you are rendering the sticky view within your live layout, make sure that the sticky view itself does not use the same layout. You can do so by returning {:ok, socket, layout: false} from mount. Examples When rendering from a controller/view, you can call: &lt;%= live_render ( @conn , MyApp.ThermostatLive ) %&gt; Or: &lt;%= live_render ( @conn , MyApp.ThermostatLive , session : %{ &quot;home_id&quot; =&gt; @home . id } ) %&gt; Within another LiveView, you must pass the :id option: &lt;%= live_render ( @socket , MyApp.ThermostatLive , id : &quot;thermostat&quot; ) %&gt; Containers When a LiveView is rendered, its contents are wrapped in a container. By default, the container is a div tag with a handful of LiveView specific attributes. The container can be customized in different ways: You can change the default container on use Phoenix.LiveView : use Phoenix.LiveView , container : { :tr , id : &quot;foo-bar&quot; } You can override the container tag and pass extra attributes when calling live_render (as well as on your live call in your router): live_render socket , MyLiveView , container : { :tr , class : &quot;highlight&quot; } If you don't want the container to affect layout, you can use the CSS property display: contents or a class that applies it, like Tailwind's .contents .","ref":"Phoenix.Component.html#live_render/3","title":"Phoenix.Component.live_render/3","type":"function"},{"doc":"Renders a title with automatic prefix/suffix on @page_title updates. Attributes prefix ( :string ) - A prefix added before the content of inner_block . Defaults to nil . suffix ( :string ) - A suffix added after the content of inner_block . Defaults to nil . Slots inner_block (required) - Content rendered inside the title tag. Examples &lt; .live_title prefix = &quot;MyApp – &quot; &gt; &lt;%= assigns [ :page_title ] || &quot;Welcome&quot; %&gt; &lt;/ .live_title &gt; &lt; .live_title suffix = &quot;- MyApp&quot; &gt; &lt;%= assigns [ :page_title ] || &quot;Welcome&quot; %&gt; &lt;/ .live_title &gt;","ref":"Phoenix.Component.html#live_title/1","title":"Phoenix.Component.live_title/1","type":"function"},{"doc":"Renders a slot entry with the given optional argument . &lt;%= render_slot ( @inner_block , @form ) %&gt; If the slot has no entries, nil is returned. If multiple slot entries are defined for the same slot, render_slot/2 will automatically render all entries, merging their contents. In case you want to use the entries' attributes, you need to iterate over the list to access each slot individually. For example, imagine a table component: &lt; .table rows = { @users } &gt; &lt; :col :let = { user } label = &quot;Name&quot; &gt; &lt;%= user . name %&gt; &lt;/ :col &gt; &lt; :col :let = { user } label = &quot;Address&quot; &gt; &lt;%= user . address %&gt; &lt;/ :col &gt; &lt;/ .table &gt; At the top level, we pass the rows as an assign and we define a :col slot for each column we want in the table. Each column also has a label , which we are going to use in the table header. Inside the component, you can render the table with headers, rows, and columns: def table ( assigns ) do ~H&quot;&quot;&quot; &lt;table&gt; &lt;tr&gt; &lt;%= for col &lt;- @col do %&gt; &lt;th&gt;&lt;%= col.label %&gt;&lt;/th&gt; &lt;% end %&gt; &lt;/tr&gt; &lt;%= for row &lt;- @rows do %&gt; &lt;tr&gt; &lt;%= for col &lt;- @col do %&gt; &lt;td&gt;&lt;%= render_slot(col, row) %&gt;&lt;/td&gt; &lt;% end %&gt; &lt;/tr&gt; &lt;% end %&gt; &lt;/table&gt; &quot;&quot;&quot; end","ref":"Phoenix.Component.html#render_slot/2","title":"Phoenix.Component.render_slot/2","type":"macro"},{"doc":"The ~H sigil for writing HEEx templates inside source files. HEEx is a HTML-aware and component-friendly extension of Elixir Embedded language ( EEx ) that provides: Built-in handling of HTML attributes An HTML-like notation for injecting function components Compile-time validation of the structure of the template The ability to minimize the amount of data sent over the wire Out-of-the-box code formatting via mix format Example ~H&quot;&quot;&quot; &lt;div title=&quot;My div&quot; class={@class}&gt; &lt;p&gt;Hello &lt;%= @name %&gt;&lt;/p&gt; &lt;MyApp.Weather.city name=&quot;Kraków&quot;/&gt; &lt;/div&gt; &quot;&quot;&quot; Syntax HEEx is built on top of Embedded Elixir ( EEx ). In this section, we are going to cover the basic constructs in HEEx templates as well as its syntax extensions. Interpolation Both HEEx and EEx templates use &lt;%= ... %&gt; for interpolating code inside the body of HTML tags: &lt; p &gt; Hello, &lt;%= @name %&gt; &lt;/ p &gt; Similarly, conditionals and other block Elixir constructs are supported: &lt;%= if @show_greeting? do %&gt; &lt; p &gt; Hello, &lt;%= @name %&gt; &lt;/ p &gt; &lt;% end %&gt; Note we don't include the equal sign = in the closing &lt;% end %&gt; tag (because the closing tag does not output anything). There is one important difference between HEEx and Elixir's builtin EEx . HEEx uses a specific annotation for interpolating HTML tags and attributes. Let's check it out. HEEx extension: Defining attributes Since HEEx must parse and validate the HTML structure, code interpolation using &lt;%= ... %&gt; and &lt;% ... %&gt; are restricted to the body (inner content) of the HTML/component nodes and it cannot be applied within tags. For instance, the following syntax is invalid: &lt; div class = &quot; &lt;%= @class %&gt; &quot; &gt; ... &lt;/ div &gt; Instead do: &lt; div class = { @class } &gt; ... &lt;/ div &gt; You can put any Elixir expression between { ... } . For example, if you want to set classes, where some are static and others are dynamic, you can using string interpolation: &lt; div class = { &quot;btn btn- \#{ @type } &quot; } &gt; ... &lt;/ div &gt; The following attribute values have special meaning: true - if a value is true , the attribute is rendered with no value at all. For example, &lt;input required={true}&gt; is the same as &lt;input required&gt; ; false or nil - if a value is false or nil , the attribute is not rendered; list (only for the class attribute) - each element of the list is processed as a different class. nil and false elements are discarded. For multiple dynamic attributes, you can use the same notation but without assigning the expression to any specific attribute. &lt; div { @dynamic_attrs } &gt; ... &lt;/ div &gt; The expression inside {...} must be either a keyword list or a map containing the key-value pairs representing the dynamic attributes. You can pair this notation assigns_to_attributes/2 to strip out any internal LiveView attributes and user-defined assigns from being expanded into the HTML tag: &lt; div { assigns_to_attributes ( assigns , [ :visible ] ) } &gt; ... &lt;/ div &gt; The above would add all caller attributes into the HTML, but strip out LiveView assigns like slots, as well as user-defined assigns like :visible that are not meant to be added to the HTML itself. This approach is useful to allow a component to accept arbitrary HTML attributes like class, ARIA attributes, etc. HEEx extension: Defining function components Function components are stateless components implemented as pure functions with the help of the Phoenix.Component module. They can be either local (same module) or remote (external module). HEEx allows invoking these function components directly in the template using an HTML-like notation. For example, a remote function: &lt; MyApp.Weather.city name = &quot;Kraków&quot; /&gt; A local function can be invoked with a leading dot: &lt; .city name = &quot;Kraków&quot; /&gt; where the component could be defined as follows: defmodule MyApp.Weather do use Phoenix.Component def city ( assigns ) do ~H&quot;&quot;&quot; The chosen city is: &lt;%= @name %&gt;. &quot;&quot;&quot; end def country ( assigns ) do ~H&quot;&quot;&quot; The chosen country is: &lt;%= @name %&gt;. &quot;&quot;&quot; end end It is typically best to group related functions into a single module, as opposed to having many modules with a single render/1 function. Function components support other important features, such as slots. You can learn more about components in Phoenix.Component . HEEx extension: special attributes Apart from normal HTML attributes, HEEx also supports some special attributes such as :let and :for . :let This is used by components and slots that want to yield a value back to the caller. For an example, see how form/1 works: &lt; .form :let = { f } for = { @form } phx-change = &quot;validate&quot; phx-submit = &quot;save&quot; &gt; &lt; .input field = { f [ :username ] } type = &quot;text&quot; /&gt; ... &lt;/ .form &gt; Notice how the variable f , defined by .form is used by your input component. The Phoenix.Component module has detailed documentation on how to use and implement such functionality. :if and :for It is a syntax sugar for &lt;%= if .. do %&gt; and &lt;%= for .. do %&gt; that can be used in regular HTML, function components, and slots. For example in an HTML tag: &lt; table id = &quot;admin-table&quot; :if = { @admin? } &gt; &lt; tr :for = { user &lt;- @users } &gt; &lt; td &gt; &lt;%= user . name %&gt; &lt;/ td &gt; &lt;/ tr &gt; &lt; table &gt; The snippet above will only render the table if @admin? is true, and generate a tr per user as you would expect from the collection. :for can be used similarly in function components: &lt; .error :for = { msg &lt;- @errors } message = { msg } /&gt; Which is equivalent to writing: &lt;%= for msg &lt;- @errors do %&gt; &lt; .error message = { msg } /&gt; &lt;% end %&gt; And :for in slots behaves the same way: &lt; .table id = &quot;my-table&quot; rows = { @users } &gt; &lt; :col :for = { header &lt;- @headers } :let = { user } &gt; &lt; td &gt; &lt;%= user [ :header ] %&gt; &lt;/ td &gt; &lt;/ :col &gt; &lt; table &gt; You can also combine :for and :if for tags, components, and slot to act as a filter: &lt; .error :for = { msg &lt;- @errors } :if = { msg != nil } message = { msg } /&gt; Code formatting You can automatically format HEEx templates (.heex) and ~H sigils using Phoenix.LiveView.HTMLFormatter . Please check that module for more information.","ref":"Phoenix.Component.html#sigil_H/2","title":"Phoenix.Component.sigil_H/2","type":"macro"},{"doc":"Declares a slot. See slot/3 for more information.","ref":"Phoenix.Component.html#slot/2","title":"Phoenix.Component.slot/2","type":"macro"},{"doc":"Declares a function component slot. Arguments name - an atom defining the name of the slot. Note that slots cannot define the same name as any other slots or attributes declared for the same component. opts - a keyword list of options. Defaults to [] . block - a code block containing calls to attr/3 . Defaults to nil . Options :required - marks a slot as required. If a caller does not pass a value for a required slot, a compilation warning is emitted. Otherwise, an omitted slot will default to [] . :doc - documentation for the slot. Any slot attributes declared will have their documentation listed alongside the slot. Slot Attributes A named slot may declare attributes by passing a block with calls to attr/3 . Unlike attributes, slot attributes cannot accept the :default option. Passing one will result in a compile warning being issued. The Default Slot The default slot can be declared by passing :inner_block as the name of the slot. Note that the :inner_block slot declaration cannot accept a block. Passing one will result in a compilation error. Compile-Time Validations LiveView performs some validation of slots via the :phoenix_live_view compiler. When slots are defined, LiveView will warn at compilation time on the caller if: A required slot of a component is missing. An unknown slot is given. An unknown slot attribute is given. On the side of the function component itself, defining attributes provides the following quality of life improvements: Slot documentation is generated for the component. Calls made to the component are tracked for reflection and validation purposes. Documentation Generation Public function components that define slots will have their docs injected into the function's documentation, depending on the value of the @doc module attribute: if @doc is a string, the slot docs are injected into that string. The optional placeholder [INSERT LVATTRDOCS] can be used to specify where in the string the docs are injected. Otherwise, the docs are appended to the end of the @doc string. if @doc is unspecified, the slot docs are used as the default @doc string. if @doc is false , the slot docs are omitted entirely. The injected slot docs are formatted as a markdown list: name (required) - slot docs. Accepts attributes: name ( :type ) (required) - attr docs. Defaults to :default . By default, all slots will have their docs injected into the function @doc string. To hide a specific slot, you can set the value of :doc to false . Example slot :header slot :inner_block , required : true slot :footer def modal ( assigns ) do ~H&quot;&quot;&quot; &lt;div class=&quot;modal&quot;&gt; &lt;div class=&quot;modal-header&quot;&gt; &lt;%= render_slot(@header) || &quot;Modal&quot; %&gt; &lt;/div&gt; &lt;div class=&quot;modal-body&quot;&gt; &lt;%= render_slot(@inner_block) %&gt; &lt;/div&gt; &lt;div class=&quot;modal-footer&quot;&gt; &lt;%= render_slot(@footer) || submit_button() %&gt; &lt;/div&gt; &lt;/div&gt; &quot;&quot;&quot; end As shown in the example above, render_slot/1 returns nil when an optional slot is declared and none is given. This can be used to attach default behaviour.","ref":"Phoenix.Component.html#slot/3","title":"Phoenix.Component.slot/3","type":"macro"},{"doc":"Converts a given data structure to a Phoenix.HTML.Form according to Phoenix.HTML.FormData . This is commonly used to convert a map or an Ecto changeset into a form to be given to the form/1 component. Creating a form from params If you want to create a form based on handle_event parameters, you could do: def handle_event ( &quot;submitted&quot; , params , socket ) do { :noreply , assign ( socket , form : to_form ( params ) ) } end However, most typically, we specify a name to nest the parameters: def handle_event ( &quot;submitted&quot; , %{ &quot;user&quot; =&gt; user_params } , socket ) do { :noreply , assign ( socket , form : to_form ( user_params , as : :user ) ) } end Creating a form from changesets When using changesets, the name :as is automatically retrieved from the schema. For example, if you have a user schema: defmodule MyApp.Users.User do use Ecto.Schema schema &quot;...&quot; do ... end end And then you create a changeset which you pass to to_form : % MyApp.Blog.Post { } |&gt; Ecto.Changeset . change ( ) |&gt; to_form ( ) Phoenix will take care of getting all of the relevant information from the changeset, including errors, data, and names for you. In this case, the parameters will be available under %{&quot;post&quot; =&gt; post_params} . Options :as - the name prefix to be used in form inputs :id - the id prefix to be used in form inputs The underlying data may accept additional options when converted to forms. For example, a map accepts :errors to list errors, but such option is not accepted by changesets. If an existing Phoenix.HTML.Form struct is given, the options below will override its existing values if given. Then the remaining options are merged with the existing form options.","ref":"Phoenix.Component.html#to_form/2","title":"Phoenix.Component.to_form/2","type":"function"},{"doc":"Updates an existing key with fun in the given socket_or_assigns . The first argument is either a LiveView socket or an assigns map from function components. The update function receives the current key's value and returns the updated value. Raises if the key does not exist. The update function may also be of arity 2, in which case it receives the current key's value as the first argument and the current assigns as the second argument. Raises if the key does not exist. Examples iex&gt; update ( socket , :count , fn count -&gt; count + 1 end ) iex&gt; update ( socket , :count , &amp; ( &amp;1 + 1 ) ) iex&gt; update ( socket , :max_users_this_session , fn current_max , %{ users : users } -&gt; max ( current_max , length ( users ) ) end )","ref":"Phoenix.Component.html#update/3","title":"Phoenix.Component.update/3","type":"function"},{"doc":"Returns the entry errors for an upload. The following error may be returned: :too_many_files - The number of selected files exceeds the :max_entries constraint Examples def error_to_string ( :too_many_files ) , do : &quot;You have selected too many files&quot; &lt;%= for err &lt;- upload_errors ( @uploads . avatar ) do %&gt; &lt; div class = &quot;alert alert-danger&quot; &gt; &lt;%= error_to_string ( err ) %&gt; &lt;/ div &gt; &lt;% end %&gt;","ref":"Phoenix.Component.html#upload_errors/1","title":"Phoenix.Component.upload_errors/1","type":"function"},{"doc":"Returns the entry errors for an upload. The following errors may be returned: :too_large - The entry exceeds the :max_file_size constraint :not_accepted - The entry does not match the :accept MIME types Examples def error_to_string ( :too_large ) , do : &quot;Too large&quot; def error_to_string ( :not_accepted ) , do : &quot;You have selected an unacceptable file type&quot; &lt;%= for entry &lt;- @uploads . avatar . entries do %&gt; &lt;%= for err &lt;- upload_errors ( @uploads . avatar , entry ) do %&gt; &lt; div class = &quot;alert alert-danger&quot; &gt; &lt;%= error_to_string ( err ) %&gt; &lt;/ div &gt; &lt;% end %&gt; &lt;% end %&gt;","ref":"Phoenix.Component.html#upload_errors/2","title":"Phoenix.Component.upload_errors/2","type":"function"},{"doc":"LiveComponents are a mechanism to compartmentalize state, markup, and events in LiveView. LiveComponents are defined by using Phoenix.LiveComponent and are used by calling Phoenix.Component.live_component/1 in a parent LiveView. They run inside the LiveView process but have their own state and life-cycle. For this reason, they are also often called &quot;stateful components&quot;. This is a contrast to Phoenix.Component , also known as &quot;function components&quot;, which are stateless. The smallest LiveComponent only needs to define a render/1 function: defmodule HeroComponent do # If you generated an app with mix phx.new --live, # the line below would be: use MyAppWeb, :live_component use Phoenix.LiveComponent def render ( assigns ) do ~H&quot;&quot;&quot; &lt;div class=&quot;hero&quot;&gt;&lt;%= @content %&gt;&lt;/div&gt; &quot;&quot;&quot; end end A LiveComponent is rendered as: &lt; . live_component module = { HeroComponent } id = &quot;hero&quot; content = { @content } / &gt; You must always pass the module and id attributes. The id will be available as an assign and it must be used to uniquely identify the component. All other attributes will be available as assigns inside the LiveComponent. Life-cycle Stateful components are identified by the component module and their ID. Therefore, two stateful components with the same module and ID are treated as the same component. We often tie the component ID to some application based ID: &lt; . live_component module = { UserComponent } id = { @user . id } user = { @user } / &gt; When live_component/1 is called, mount/1 is called once, when the component is first added to the page. mount/1 receives the socket as argument. Then update/2 is invoked with all of the assigns given to live_component/1 . If update/2 is not defined all assigns are simply merged into the socket. After the component is updated, render/1 is called with all assigns. On first render, we get: mount ( socket ) -&gt; update ( assigns , socket ) -&gt; render ( assigns ) On further rendering: update ( assigns , socket ) -&gt; render ( assigns ) The given id is not automatically used as the DOM ID. If you want to set a DOM ID, it is your responsibility to do so when rendering: defmodule UserComponent do use Phoenix.LiveComponent def render ( assigns ) do ~H&quot;&quot;&quot; &lt;div id={&quot;user-\\\#{@id}&quot;} class=&quot;user&quot;&gt; &lt;%= @user.name %&gt; &lt;/div&gt; &quot;&quot;&quot; end end Targeting Component Events Stateful components can also implement the handle_event/3 callback that works exactly the same as in LiveView. For a client event to reach a component, the tag must be annotated with a phx-target . If you want to send the event to yourself, you can simply use the @myself assign, which is an internal unique reference to the component instance: &lt; a href = &quot;#&quot; phx - click = &quot;say_hello&quot; phx - target = { @myself } &gt; Say hello! &lt; / a &gt; Note that @myself is not set for stateless components, as they cannot receive events. If you want to target another component, you can also pass an ID or a class selector to any element inside the targeted component. For example, if there is a UserComponent with the DOM ID of &quot;user-13&quot; , using a query selector, we can send an event to it with: &lt; a href = &quot;#&quot; phx - click = &quot;say_hello&quot; phx - target = &quot;#user-13&quot; &gt; Say hello! &lt; / a &gt; In both cases, handle_event/3 will be called with the &quot;say_hello&quot; event. When handle_event/3 is called for a component, only the diff of the component is sent to the client, making them extremely efficient. Any valid query selector for phx-target is supported, provided that the matched nodes are children of a LiveView or LiveComponent, for example to send the close event to multiple components: &lt; a href = &quot;#&quot; phx - click = &quot;close&quot; phx - target = &quot;#modal, #sidebar&quot; &gt; Dismiss &lt; / a &gt; Preloading and update Stateful components also support an optional preload/1 callback. The preload/1 callback is useful when multiple components of the same type are rendered on the page and you want to preload or augment their data in batches. Once a LiveView renders a LiveComponent, the optional preload/1 and update/2 callbacks are called before render/1 . So on first render, the following callbacks will be invoked: preload ( list_of_assigns ) -&gt; mount ( socket ) -&gt; update ( assigns , socket ) -&gt; render ( assigns ) On subsequent renders, these callbacks will be invoked: preload ( list_of_assigns ) -&gt; update ( assigns , socket ) -&gt; render ( assigns ) To provide a more complete understanding of why both callbacks are necessary, let's see an example. Imagine you are implementing a component and the component needs to load some state from the database. For example: &lt; . live_component module = { UserComponent } id = { user_id } / &gt; A possible implementation would be to load the user on the update/2 callback: def update ( assigns , socket ) do user = Repo . get! ( User , assigns . id ) { :ok , assign ( socket , :user , user ) } end However, the issue with said approach is that, if you are rendering multiple user components in the same page, you have a N+1 query problem. The preload/1 callback helps address this problem as it is invoked with a list of assigns for all components of the same type. For example, instead of implementing update/2 as above, one could implement: def preload ( list_of_assigns ) do list_of_ids = Enum . map ( list_of_assigns , &amp; &amp;1 . id ) users = from ( u in User , where : u . id in ^ list_of_ids , select : { u . id , u } ) |&gt; Repo . all ( ) |&gt; Map . new ( ) Enum . map ( list_of_assigns , fn assigns -&gt; Map . put ( assigns , :user , users [ assigns . id ] ) end ) end Now only a single query to the database will be made. In fact, the preloading algorithm is a breadth-first tree traversal, which means that even for nested components, the amount of queries are kept to a minimum. Finally, note that preload/1 must return an updated list_of_assigns , keeping the assigns in the same order as they were given. Slots LiveComponent can also receive slots, in the same way as a Phoenix.Component : &lt; . live_component module = { MyComponent } id = { @data . id } &gt; &lt; div &gt; Inner content here &lt; / div &gt; &lt; / . live_component &gt; If the LiveComponent defines an update/2 , be sure that the socket it returns includes the :inner_block assign it received. See the docs for Phoenix.Component for more information. Live patches and live redirects A template rendered inside a component can use &lt;.link patch={...}&gt; and &lt;.link navigate={...}&gt; . Patches are always handled by the parent LiveView , as components do not provide handle_params . Managing state Now that we have learned how to define and use components, as well as how to use preload/1 as a data loading optimization, it is important to talk about how to manage state in components. Generally speaking, you want to avoid both the parent LiveView and the LiveComponent working on two different copies of the state. Instead, you should assume only one of them to be the source of truth. Let's discuss the two different approaches in detail. Imagine a scenario where a LiveView represents a board with each card in it as a separate stateful LiveComponent. Each card has a form to allow update of the card title directly in the component, as follows: defmodule CardComponent do use Phoenix.LiveComponent def render ( assigns ) do ~H&quot;&quot;&quot; &lt;form phx-submit=&quot;...&quot; phx-target={@myself}&gt; &lt;input name=&quot;title&quot;&gt;&lt;%= @card.title %&gt;&lt;/input&gt; ... &lt;/form&gt; &quot;&quot;&quot; end ... end We will see how to organize the data flow to keep either the board LiveView or the card LiveComponents as the source of truth. LiveView as the source of truth If the board LiveView is the source of truth, it will be responsible for fetching all of the cards in a board. Then it will call live_component/1 for each card, passing the card struct as argument to CardComponent : &lt; % = for card &lt;- @cards do % &gt; &lt; . live_component module = { CardComponent } card = { card } id = { card . id } board_id = { @id } / &gt; &lt; % end % &gt; Now, when the user submits the form, CardComponent.handle_event/3 will be triggered. However, if the update succeeds, you must not change the card struct inside the component. If you do so, the card struct in the component will get out of sync with the LiveView. Since the LiveView is the source of truth, you should instead tell the LiveView that the card was updated. Luckily, because the component and the view run in the same process, sending a message from the LiveComponent to the parent LiveView is as simple as sending a message to self() : defmodule CardComponent do ... def handle_event ( &quot;update_title&quot; , %{ &quot;title&quot; =&gt; title } , socket ) do send self ( ) , { :updated_card , %{ socket . assigns . card | title : title } } { :noreply , socket } end end The LiveView then receives this event using Phoenix.LiveView.handle_info/2 : defmodule BoardView do ... def handle_info ( { :updated_card , card } , socket ) do # update the list of cards in the socket { :noreply , updated_socket } end end Because the list of cards in the parent socket was updated, the parent LiveView will be re-rendered, sending the updated card to the component. So in the end, the component does get the updated card, but always driven from the parent. Alternatively, instead of having the component send a message directly to the parent view, the component could broadcast the update using Phoenix.PubSub . Such as: defmodule CardComponent do ... def handle_event ( &quot;update_title&quot; , %{ &quot;title&quot; =&gt; title } , socket ) do message = { :updated_card , %{ socket . assigns . card | title : title } } Phoenix.PubSub . broadcast ( MyApp.PubSub , board_topic ( socket ) , message ) { :noreply , socket } end defp board_topic ( socket ) do &quot;board:&quot; &lt;&gt; socket . assigns . board_id end end As long as the parent LiveView subscribes to the board:&lt;ID&gt; topic, it will receive updates. The advantage of using PubSub is that we get distributed updates out of the box. Now, if any user connected to the board changes a card, all other users will see the change. LiveComponent as the source of truth If each card LiveComponent is the source of truth, then the board LiveView must no longer fetch the card structs from the database. Instead, the board LiveView must only fetch the card ids, then render each component only by passing an ID: &lt; % = for card_id &lt;- @card_ids do % &gt; &lt; . live_component module = { CardComponent } id = { card_id } board_id = { @id } / &gt; &lt; % end % &gt; Now, each CardComponent will load its own card. Of course, doing so per card could be expensive and lead to N queries, where N is the number of cards, so we can use the preload/1 callback to make it efficient. Once the card components are started, they can each manage their own card, without concerning themselves with the parent LiveView. However, note that components do not have a Phoenix.LiveView.handle_info/2 callback. Therefore, if you want to track distributed changes on a card, you must have the parent LiveView receive those events and redirect them to the appropriate card. For example, assuming card updates are sent to the &quot;board:ID&quot; topic, and that the board LiveView is subscribed to said topic, one could do: def handle_info ( { :updated_card , card } , socket ) do send_update CardComponent , id : card . id , board_id : socket . assigns . id { :noreply , socket } end With Phoenix.LiveView.send_update/3 , the CardComponent given by id will be invoked, triggering both preload and update callbacks, which will load the most up to date data from the database. Cost of stateful components The internal infrastructure LiveView uses to keep track of stateful components is very lightweight. However, be aware that in order to provide change tracking and to send diffs over the wire, all of the components' assigns are kept in memory - exactly as it is done in LiveViews themselves. Therefore it is your responsibility to keep only the assigns necessary in each component. For example, avoid passing all of LiveView's assigns when rendering a component: &lt; . live_component module = { MyComponent } { assigns } / &gt; Instead pass only the keys that you need: &lt; . live_component module = { MyComponent } user = { @user } org = { @org } / &gt; Luckily, because LiveViews and LiveComponents are in the same process, they share the data structure representations in memory. For example, in the code above, the view and the component will share the same copies of the @user and @org assigns. You should also avoid using stateful components to provide abstract DOM components. As a guideline, a good LiveComponent encapsulates application concerns and not DOM functionality. For example, if you have a page that shows products for sale, you can encapsulate the rendering of each of those products in a component. This component may have many buttons and events within it. On the opposite side, do not write a component that is simply encapsulating generic DOM components. For instance, do not do this: defmodule MyButton do use Phoenix.LiveComponent def render ( assigns ) do ~H&quot;&quot;&quot; &lt;button class=&quot;css-framework-class&quot; phx-click=&quot;click&quot;&gt; &lt;%= @text %&gt; &lt;/button&gt; &quot;&quot;&quot; end def handle_event ( &quot;click&quot; , _ , socket ) do _ = socket . assigns . on_click . ( ) { :noreply , socket } end end Instead, it is much simpler to create a function component: def my_button ( %{ text : _ , click : _ } = assigns ) do ~H&quot;&quot;&quot; &lt;button class=&quot;css-framework-class&quot; phx-click={@click}&gt; &lt;%= @text %&gt; &lt;/button&gt; &quot;&quot;&quot; end If you keep components mostly as an application concern with only the necessary assigns, it is unlikely you will run into issues related to stateful components. Limitations Live Components require a single HTML tag at the root Live Components require a single HTML tag at the root. It is not possible to have components that render only text or multiple tags. SVG support Given components compartmentalize markup on the server, they are also rendered in isolation on the client, which provides great performance benefits on the client too. However, when rendering components on the client, the client needs to choose the mime type of the component contents, which defaults to HTML. This is the best default but in some cases it may lead to unexpected results. For example, if you are rendering SVG, the SVG will be interpreted as HTML. This may work just fine for most components but you may run into corner cases. For example, the &lt;image&gt; SVG tag may be rewritten to the &lt;img&gt; tag, since &lt;image&gt; is an obsolete HTML tag. Luckily, there is a simple solution to this problem. Since SVG allows &lt;svg&gt; tags to be nested, you can wrap the component content into an &lt;svg&gt; tag. This will ensure that it is correctly interpreted by the browser.","ref":"Phoenix.LiveComponent.html","title":"Phoenix.LiveComponent","type":"behaviour"},{"doc":"Uses LiveComponent in the current module. use Phoenix.LiveComponent Options :global_prefixes - the global prefixes to use for components. See Global Attributes in Phoenix.Component for more information.","ref":"Phoenix.LiveComponent.html#__using__/1","title":"Phoenix.LiveComponent.__using__/1","type":"macro"},{"doc":"","ref":"Phoenix.LiveComponent.html#c:handle_event/3","title":"Phoenix.LiveComponent.handle_event/3","type":"callback"},{"doc":"","ref":"Phoenix.LiveComponent.html#c:mount/1","title":"Phoenix.LiveComponent.mount/1","type":"callback"},{"doc":"","ref":"Phoenix.LiveComponent.html#c:preload/1","title":"Phoenix.LiveComponent.preload/1","type":"callback"},{"doc":"","ref":"Phoenix.LiveComponent.html#c:render/1","title":"Phoenix.LiveComponent.render/1","type":"callback"},{"doc":"","ref":"Phoenix.LiveComponent.html#c:update/2","title":"Phoenix.LiveComponent.update/2","type":"callback"},{"doc":"LiveView provides rich, real-time user experiences with server-rendered HTML. The LiveView programming model is declarative: instead of saying &quot;once event X happens, change Y on the page&quot;, events in LiveView are regular messages which may cause changes to its state. Once the state changes, LiveView will re-render the relevant parts of its HTML template and push it to the browser, which updates itself in the most efficient manner. This means developers write LiveView templates as any other server-rendered HTML and LiveView does the hard work of tracking changes and sending the relevant diffs to the browser. A LiveView is just a process that receives events as messages and updates its state. The state itself is nothing more than functional and immutable Elixir data structures. The events are either internal application messages (usually emitted by Phoenix.PubSub ) or sent by the client/browser. LiveView is first rendered statically as part of regular HTTP requests, which provides quick times for &quot;First Meaningful Paint&quot;, in addition to helping search and indexing engines. Then a persistent connection is established between client and server. This allows LiveView applications to react faster to user events as there is less work to be done and less data to be sent compared to stateless requests that have to authenticate, decode, load, and encode data on every request. The flipside is that LiveView uses more memory on the server compared to stateless requests. Life-cycle A LiveView begins as a regular HTTP request and HTML response, and then upgrades to a stateful view on client connect, guaranteeing a regular HTML page even if JavaScript is disabled. Any time a stateful view changes or updates its socket assigns, it is automatically re-rendered and the updates are pushed to the client. Socket assigns are stateful values kept on the server side in Phoenix.LiveView.Socket . This is different from the common stateless HTTP pattern of sending the connection state to the client in the form of a token or cookie and rebuilding the state on the server to service every request. You begin by rendering a LiveView typically from your router. When LiveView is first rendered, the mount/3 callback is invoked with the current params, the current session and the LiveView socket. As in a regular request, params contains public data that can be modified by the user. The session always contains private data set by the application itself. The mount/3 callback wires up socket assigns necessary for rendering the view. After mounting, handle_params/3 is invoked so uri and query params are handled. Finally, render/1 is invoked and the HTML is sent as a regular HTML response to the client. After rendering the static page, LiveView connects from the client to the server where stateful views are spawned to push rendered updates to the browser, and receive client events via phx- bindings. Just like the first rendering, mount/3 , is invoked with params, session, and socket state. However in the connected client case, a LiveView process is spawned on the server, runs handle_params/3 again and then pushes the result of render/1 to the client and continues on for the duration of the connection. If at any point during the stateful life-cycle a crash is encountered, or the client connection drops, the client gracefully reconnects to the server, calling mount/3 and handle_params/3 again. LiveView also allows attaching hooks to specific life-cycle stages with attach_hook/4 . Example Before writing your first example, make sure that Phoenix LiveView is properly installed. All applications generated with Phoenix v1.6 and later come with LiveView installed and configured. For previously existing projects, please follow the steps in the installation guide before continuing. A LiveView is a simple module that requires two callbacks: mount/3 and render/1 : defmodule MyAppWeb.ThermostatLive do # In Phoenix v1.6+ apps, the line below should be: use MyAppWeb, :live_view use Phoenix.LiveView def render ( assigns ) do ~H&quot;&quot;&quot; Current temperature: &lt;%= @temperature %&gt; &quot;&quot;&quot; end def mount ( _params , %{ &quot;current_user_id&quot; =&gt; user_id } , socket ) do temperature = Thermostat . get_user_reading ( user_id ) { :ok , assign ( socket , :temperature , temperature ) } end end The render/1 callback receives the socket.assigns and is responsible for returning rendered content. We use the ~H sigil to define a HEEx template, which stands for HTML+EEx. They are an extension of Elixir's builtin EEx templates, with support for HTML validation, syntax-based components, smart change tracking, and more. You can learn more about the template syntax in Phoenix.Component.sigil_H/2 (note Phoenix.Component is automatically imported when you use Phoenix.LiveView ). Next, decide where you want to use your LiveView. You can serve the LiveView directly from your router (recommended): defmodule MyAppWeb.Router do use Phoenix.Router import Phoenix.LiveView.Router scope &quot;/&quot; , MyAppWeb do live &quot;/thermostat&quot; , ThermostatLive end end Note: the above assumes there is plug :put_root_layout call in your router that configures the LiveView layout. This call is automatically included in Phoenix v1.6 apps and described in the installation guide . Alternatively, you can live_render from any template. In your view: import Phoenix.Component Then in your template: &lt; h1 &gt; Temperature Control &lt; / h1 &gt; &lt; % = live_render ( @conn , MyAppWeb.ThermostatLive ) % &gt; Once the LiveView is rendered, a regular HTML response is sent. In your app.js file, you should find the following: import { Socket } from &quot;phoenix&quot; import { LiveSocket } from &quot;phoenix_live_view&quot; let csrfToken = document . querySelector ( &quot;meta[name=&#39;csrf-token&#39;]&quot; ) . getAttribute ( &quot;content&quot; ) let liveSocket = new LiveSocket ( &quot;/live&quot; , Socket , { params : { _csrf_token : csrfToken } } ) liveSocket . connect ( ) After the client connects, mount/3 will be invoked inside a spawned LiveView process. At this point, you can use connected?/1 to conditionally perform stateful work, such as subscribing to pubsub topics, sending messages, etc. For example, you can periodically update a LiveView with a timer: defmodule DemoWeb.ThermostatLive do use Phoenix.LiveView ... def mount ( _params , %{ &quot;current_user_id&quot; =&gt; user_id } , socket ) do if connected? ( socket ) , do : Process . send_after ( self ( ) , :update , 30000 ) case Thermostat . get_user_reading ( user_id ) do { :ok , temperature } -&gt; { :ok , assign ( socket , temperature : temperature , user_id : user_id ) } { :error , _reason } -&gt; { :ok , redirect ( socket , to : &quot;/error&quot; ) } end end def handle_info ( :update , socket ) do Process . send_after ( self ( ) , :update , 30000 ) { :ok , temperature } = Thermostat . get_reading ( socket . assigns . user_id ) { :noreply , assign ( socket , :temperature , temperature ) } end end We used connected?(socket) on mount to send our view a message every 30s if the socket is in a connected state. We receive the :update message in the handle_info/2 callback, just like in an Elixir GenServer , and update our socket assigns. Whenever a socket's assigns change, render/1 is automatically invoked, and the updates are sent to the client. Colocating templates In the examples above, we have placed the template directly inside the LiveView: defmodule MyAppWeb.ThermostatLive do use Phoenix.LiveView def render ( assigns ) do ~H&quot;&quot;&quot; Current temperature: &lt;%= @temperature %&gt; &quot;&quot;&quot; end For larger templates, you can place them in a file in the same directory and same name as the LiveView. For example, if the file above is placed at lib/my_app_web/live/thermostat_live.ex , you can also remove the render/1 definition above and instead put the template code at lib/my_app_web/live/thermostat_live.html.heex . In all cases, each assign in the template will be accessible as @assign . You can learn more about assigns and HEEx templates in their own guide . Bindings Phoenix supports DOM element bindings for client-server interaction. For example, to react to a click on a button, you would render the element: &lt; button phx - click = &quot;inc_temperature&quot; &gt; + &lt; / button &gt; Then on the server, all LiveView bindings are handled with the handle_event/3 callback, for example: def handle_event ( &quot;inc_temperature&quot; , _value , socket ) do { :ok , new_temp } = Thermostat . inc_temperature ( socket . assigns . id ) { :noreply , assign ( socket , :temperature , new_temp ) } end To update UI state, for example, to open and close dropdowns, switch tabs, etc, LiveView also supports JS commands ( Phoenix.LiveView.JS ), which execute directly on the client without reaching the server. To learn more, see our bindings page for a complete list of all LiveView bindings as well as our JavaScript interoperability guide . Compartmentalize state, markup, and events in LiveView LiveView supports two extension mechanisms: function components, provided by HEEx templates, and stateful components. Function components are any function that receives an assigns map, similar to render(assigns) in our LiveView, and returns a ~H template. For example: def weather_greeting ( assigns ) do ~H&quot;&quot;&quot; &lt;div title=&quot;My div&quot; class={@class}&gt; &lt;p&gt;Hello &lt;%= @name %&gt;&lt;/p&gt; &lt;MyApp.Weather.city name=&quot;Kraków&quot;/&gt; &lt;/div&gt; &quot;&quot;&quot; end You can learn more about function components in the Phoenix.Component module. At the end of the day, they are useful mechanism to reuse markup in your LiveViews. However, sometimes you need to compartmentalize or reuse more than markup. Perhaps you want to move part of the state or part of the events in your LiveView to a separate module. For these cases, LiveView provides Phoenix.LiveComponent , which are rendered using live_component/1 : &lt; . live_component module = { UserComponent } id = { user . id } user = { user } / &gt; Components have their own mount/3 and handle_event/3 callbacks, as well as their own state with change tracking support. Components are also lightweight as they &quot;run&quot; in the same process as the parent LiveView . However, this means an error in a component would cause the whole view to fail to render. See Phoenix.LiveComponent for a complete rundown on components. Finally, if you want complete isolation between parts of a LiveView, you can always render a LiveView inside another LiveView by calling live_render/3 . This child LiveView runs in a separate process than the parent, with its own callbacks. If a child LiveView crashes, it won't affect the parent. If the parent crashes, all children are terminated. When rendering a child LiveView, the :id option is required to uniquely identify the child. A child LiveView will only ever be rendered and mounted a single time, provided its ID remains unchanged. To force a child to re-mount with new session data, a new ID must be provided. Given that a LiveView runs on its own process, it is an excellent tool for creating completely isolated UI elements, but it is a slightly expensive abstraction if all you want is to compartmentalize markup or events (or both). To sum it up: use Phoenix.Component to compartmentalize/reuse markup use Phoenix.LiveComponent to compartmentalize state, markup, and events use nested Phoenix.LiveView to compartmentalize state, markup, events, and error isolation Endpoint configuration LiveView accepts the following configuration in your endpoint under the :live_view key: :signing_salt (required) - the salt used to sign data sent to the client :hibernate_after (optional) - the idle time in milliseconds allowed in the LiveView before compressing its own memory and state. Defaults to 15000ms (15 seconds) Guides LiveView has many guides to help you on your journey. Server-side These guides focus on server-side functionality: Assigns and HEEx templates Error and exception handling Live Layouts Live Navigation Security considerations of the LiveView model Telemetry Uploads Using Gettext for internationalization Client-side These guides focus on LiveView bindings and client-side integration: Bindings Form bindings DOM patching and temporary assigns JavaScript interoperability Uploads (External)","ref":"Phoenix.LiveView.html","title":"Phoenix.LiveView","type":"behaviour"},{"doc":"Uses LiveView in the current module to mark it a LiveView. use Phoenix.LiveView , namespace : MyAppWeb , container : { :tr , class : &quot;colorized&quot; } , layout : { MyAppWeb.LayoutView , :app } , log : :info Options :namespace - configures the namespace the LiveView is in :container - configures the container the LiveView will be wrapped in :layout - configures the layout the LiveView will be rendered in :log - configures the log level for the LiveView :global_prefixes - the global prefixes to use for components. See Global Attributes in Phoenix.Component for more information.","ref":"Phoenix.LiveView.html#__using__/1","title":"Phoenix.LiveView.__using__/1","type":"macro"},{"doc":"Allows an upload for the provided name. Options :accept - Required. A list of unique file type specifiers or the atom :any to allow any kind of file. For example, [&quot;.jpeg&quot;] , :any , etc. :max_entries - The maximum number of selected files to allow per file input. Defaults to 1. :max_file_size - The maximum file size in bytes to allow to be uploaded. Defaults 8MB. For example, 12_000_000 . :chunk_size - The chunk size in bytes to send when uploading. Defaults 64_000 . :chunk_timeout - The time in milliseconds to wait before closing the upload channel when a new chunk has not been received. Defaults 10_000 . :external - The 2-arity function for generating metadata for external client uploaders. See the Uploads section for example usage. :progress - The optional 3-arity function for receiving progress events :auto_upload - Instructs the client to upload the file automatically on file selection instead of waiting for form submits. Default false. Raises when a previously allowed upload under the same name is still active. Examples allow_upload ( socket , :avatar , accept : ~w(.jpg .jpeg) , max_entries : 2 ) allow_upload ( socket , :avatar , accept : :any ) For consuming files automatically as they are uploaded, you can pair auto_upload: true with a custom progress function to consume the entries as they are completed. For example: allow_upload ( socket , :avatar , accept : :any , progress : &amp; handle_progress / 3 , auto_upload : true ) defp handle_progress ( :avatar , entry , socket ) do if entry . done? do uploaded_file = consume_uploaded_entry ( socket , entry , fn %{ } = meta -&gt; { :ok , ... } end ) { :noreply , put_flash ( socket , :info , &quot;file \#{ uploaded_file . name } uploaded&quot; ) } else { :noreply , socket } end end","ref":"Phoenix.LiveView.html#allow_upload/3","title":"Phoenix.LiveView.allow_upload/3","type":"function"},{"doc":"Attaches the given fun by name for the lifecycle stage into socket . Note: This function is for server-side lifecycle callbacks. For client-side hooks, see the JS Interop guide . Hooks provide a mechanism to tap into key stages of the LiveView lifecycle in order to bind/update assigns, intercept events, patches, and regular messages when necessary, and to inject common functionality. Hooks may be attached to any of the following lifecycle stages: :mount (via on_mount/1 ), :handle_params , :handle_event , :handle_info , and :after_render . Return Values Lifecycle hooks take place immediately before a given lifecycle callback is invoked on the LiveView. With the exception of :after_render , a hook may return {:halt, socket} to halt the reduction, otherwise it must return {:cont, socket} so the operation may continue until all hooks have been invoked for the current stage. For :after_render hooks, the socket itself must be returned. Any updates to the socket assigns will not trigger a new render or diff calculation to the client. Halting the lifecycle Note that halting from a hook will halt the entire lifecycle stage . This means that when a hook returns {:halt, socket} then the LiveView callback will not be invoked. This has some implications. Implications for plugin authors When defining a plugin that matches on specific callbacks, you must define a catch-all clause, as your hook will be invoked even for events you may not be interested on. Implications for end-users Allowing a hook to halt the invocation of the callback means that you can attach hooks to intercept specific events before detaching themselves, while allowing other events to continue normally. Replying to events Hooks attached to the :handle_event stage are able to reply to client events by returning {:halt, reply, socket} . This is useful especially for JavaScript interoperability because a client hook can push an event and receive a reply. Examples Attaching and detaching a hook: def mount ( _params , _session , socket ) do socket = attach_hook ( socket , :my_hook , :handle_event , fn &quot;very-special-event&quot; , _params , socket -&gt; # Handle the very special event and then detach the hook { :halt , detach_hook ( socket , :my_hook , :handle_event ) } _event , _params , socket -&gt; { :cont , socket } end ) { :ok , socket } end Replying to a client event: # JavaScript: # let Hooks = {} # Hooks.ClientHook = { # mounted() { # this.pushEvent(&quot;ClientHook:mounted&quot;, {hello: &quot;world&quot;}, (reply) =&gt; { # console.log(&quot;received reply:&quot;, reply) # }) # } # } # let liveSocket = new LiveSocket(&quot;/live&quot;, Socket, {hooks: Hooks, ...}) def render ( assigns ) do ~H&quot;&quot;&quot; &lt;div id=&quot;my-client-hook&quot; phx-hook=&quot;ClientHook&quot;&gt;&lt;/div&gt; &quot;&quot;&quot; end def mount ( _params , _session , socket ) do socket = attach_hook ( socket , :reply_on_client_hook_mounted , :handle_event , fn &quot;ClientHook:mounted&quot; , params , socket -&gt; { :halt , params , socket } _ , _ , socket -&gt; { :cont , socket } end ) { :ok , socket } end","ref":"Phoenix.LiveView.html#attach_hook/4","title":"Phoenix.LiveView.attach_hook/4","type":"function"},{"doc":"Cancels an upload for the given entry. Examples &lt; % = for entry &lt;- @uploads . avatar . entries do % &gt; ... &lt; button phx - click = &quot;cancel-upload&quot; phx - value - ref = &quot;&lt;%= entry.ref %&gt;&quot; &gt; cancel &lt; / button &gt; &lt; % end % &gt; def handle_event ( &quot;cancel-upload&quot; , %{ &quot;ref&quot; =&gt; ref } , socket ) do { :noreply , cancel_upload ( socket , :avatar , ref ) } end","ref":"Phoenix.LiveView.html#cancel_upload/3","title":"Phoenix.LiveView.cancel_upload/3","type":"function"},{"doc":"Clears the flash. Examples iex&gt; clear_flash ( socket )","ref":"Phoenix.LiveView.html#clear_flash/1","title":"Phoenix.LiveView.clear_flash/1","type":"function"},{"doc":"Clears a key from the flash. Examples iex&gt; clear_flash ( socket , :info )","ref":"Phoenix.LiveView.html#clear_flash/2","title":"Phoenix.LiveView.clear_flash/2","type":"function"},{"doc":"Returns true if the socket is connected. Useful for checking the connectivity status when mounting the view. For example, on initial page render, the view is mounted statically, rendered, and the HTML is sent to the client. Once the client connects to the server, a LiveView is then spawned and mounted statefully within a process. Use connected?/1 to conditionally perform stateful work, such as subscribing to pubsub topics, sending messages, etc. Examples defmodule DemoWeb.ClockLive do use Phoenix.LiveView ... def mount ( _params , _session , socket ) do if connected? ( socket ) , do : :timer . send_interval ( 1000 , self ( ) , :tick ) { :ok , assign ( socket , date : :calendar . local_time ( ) ) } end def handle_info ( :tick , socket ) do { :noreply , assign ( socket , date : :calendar . local_time ( ) ) } end end","ref":"Phoenix.LiveView.html#connected?/1","title":"Phoenix.LiveView.connected?/1","type":"function"},{"doc":"Consumes the uploaded entries. Raises when there are still entries in progress. Typically called when submitting a form to handle the uploaded entries alongside the form data. For form submissions, it is guaranteed that all entries have completed before the submit event is invoked. Once entries are consumed, they are removed from the upload. The function passed to consume may return a tagged tuple of the form {:ok, my_result} to collect results about the consumed entries, or {:postpone, my_result} to collect results, but postpone the file consumption to be performed later. Examples def handle_event ( &quot;save&quot; , _params , socket ) do uploaded_files = consume_uploaded_entries ( socket , :avatar , fn %{ path : path } , _entry -&gt; dest = Path . join ( &quot;priv/static/uploads&quot; , Path . basename ( path ) ) File . cp! ( path , dest ) { :ok , Routes . static_path ( socket , &quot;/uploads/ \#{ Path . basename ( dest ) } &quot; ) } end ) { :noreply , update ( socket , :uploaded_files , &amp; ( &amp;1 ++ uploaded_files ) ) } end","ref":"Phoenix.LiveView.html#consume_uploaded_entries/3","title":"Phoenix.LiveView.consume_uploaded_entries/3","type":"function"},{"doc":"Consumes an individual uploaded entry. Raises when the entry is still in progress. Typically called when submitting a form to handle the uploaded entries alongside the form data. Once entries are consumed, they are removed from the upload. This is a lower-level feature than consume_uploaded_entries/3 and useful for scenarios where you want to consume entries as they are individually completed. Like consume_uploaded_entries/3 , the function passed to consume may return a tagged tuple of the form {:ok, my_result} to collect results about the consumed entries, or {:postpone, my_result} to collect results, but postpone the file consumption to be performed later. Examples def handle_event ( &quot;save&quot; , _params , socket ) do case uploaded_entries ( socket , :avatar ) do { [ _ | _ ] = entries , [ ] } -&gt; uploaded_files = for entry &lt;- entries do consume_uploaded_entry ( socket , entry , fn %{ path : path } -&gt; dest = Path . join ( &quot;priv/static/uploads&quot; , Path . basename ( path ) ) File . cp! ( path , dest ) { :ok , Routes . static_path ( socket , &quot;/uploads/ \#{ Path . basename ( dest ) } &quot; ) } end ) end { :noreply , update ( socket , :uploaded_files , &amp; ( &amp;1 ++ uploaded_files ) ) } _ -&gt; { :noreply , socket } end end","ref":"Phoenix.LiveView.html#consume_uploaded_entry/3","title":"Phoenix.LiveView.consume_uploaded_entry/3","type":"function"},{"doc":"Detaches a hook with the given name from the lifecycle stage . Note: This function is for server-side lifecycle callbacks. For client-side hooks, see the JS Interop guide . If no hook is found, this function is a no-op. Examples def handle_event ( _ , socket ) do { :noreply , detach_hook ( socket , :hook_that_was_attached , :handle_event ) } end","ref":"Phoenix.LiveView.html#detach_hook/3","title":"Phoenix.LiveView.detach_hook/3","type":"function"},{"doc":"Revokes a previously allowed upload from allow_upload/3 . Examples disallow_upload ( socket , :avatar )","ref":"Phoenix.LiveView.html#disallow_upload/2","title":"Phoenix.LiveView.disallow_upload/2","type":"function"},{"doc":"","ref":"Phoenix.LiveView.html#get_connect_info/1","title":"Phoenix.LiveView.get_connect_info/1","type":"function"},{"doc":"Accesses a given connect info key from the socket. The following keys are supported: :peer_data , :trace_context_headers , :x_headers , :uri , and :user_agent . The connect information is available only during mount. During disconnected render, all keys are available. On connected render, only the keys explicitly declared in your socket are available. See Phoenix.Endpoint.socket/3 for a complete description of the keys. Examples The first step is to declare the connect_info you want to receive. Typically, it includes at least the session, but you must include all other keys you want to access on connected mount, such as :peer_data : socket &quot;/live&quot; , Phoenix.LiveView.Socket , websocket : [ connect_info : [ :peer_data , session : @session_options ] ] Those values can now be accessed on the connected mount as get_connect_info/2 : def mount ( _params , _session , socket ) do peer_data = get_connect_info ( socket , :peer_data ) { :ok , assign ( socket , ip : peer_data . address ) } end If the key is not available, usually because it was not specified in connect_info , it returns nil.","ref":"Phoenix.LiveView.html#get_connect_info/2","title":"Phoenix.LiveView.get_connect_info/2","type":"function"},{"doc":"Accesses the connect params sent by the client for use on connected mount. Connect params are only sent when the client connects to the server and only remain available during mount. nil is returned when called in a disconnected state and a RuntimeError is raised if called after mount. Reserved params The following params have special meaning in LiveView: &quot;_csrf_token&quot; - the CSRF Token which must be explicitly set by the user when connecting &quot;_mounts&quot; - the number of times the current LiveView is mounted. It is 0 on first mount, then increases on each reconnect. It resets when navigating away from the current LiveView or on errors &quot;_track_static&quot; - set automatically with a list of all href/src from tags with the phx-track-static annotation in them. If there are no such tags, nothing is sent &quot;_live_referer&quot; - sent by the client as the referer URL when a live navigation has occurred from push_navigate or client link navigate. Examples def mount ( _params , _session , socket ) do { :ok , assign ( socket , width : get_connect_params ( socket ) [ &quot;width&quot; ] || @width ) } end","ref":"Phoenix.LiveView.html#get_connect_params/1","title":"Phoenix.LiveView.get_connect_params/1","type":"function"},{"doc":"Invoked to handle calls from other Elixir processes. See GenServer.call/3 and GenServer.handle_call/3 for more information.","ref":"Phoenix.LiveView.html#c:handle_call/3","title":"Phoenix.LiveView.handle_call/3","type":"callback"},{"doc":"Invoked to handle casts from other Elixir processes. See GenServer.cast/2 and GenServer.handle_cast/2 for more information. It must always return {:noreply, socket} , where :noreply means no additional information is sent to the process which cast the message.","ref":"Phoenix.LiveView.html#c:handle_cast/2","title":"Phoenix.LiveView.handle_cast/2","type":"callback"},{"doc":"Invoked to handle events sent by the client. It receives the event name, the event payload as a map, and the socket. It must return {:noreply, socket} , where :noreply means no additional information is sent to the client, or {:reply, map(), socket} , where the given map() is encoded and sent as a reply to the client.","ref":"Phoenix.LiveView.html#c:handle_event/3","title":"Phoenix.LiveView.handle_event/3","type":"callback"},{"doc":"Invoked to handle messages from other Elixir processes. See Kernel.send/2 and GenServer.handle_info/2 for more information. It must always return {:noreply, socket} , where :noreply means no additional information is sent to the process which sent the message.","ref":"Phoenix.LiveView.html#c:handle_info/2","title":"Phoenix.LiveView.handle_info/2","type":"callback"},{"doc":"Invoked after mount and whenever there is a live patch event. It receives the current params , including parameters from the router, the current uri from the client and the socket . It is invoked after mount or whenever there is a live navigation event caused by push_patch/2 or &lt;.link patch={...}&gt; . It must always return {:noreply, socket} , where :noreply means no additional information is sent to the client.","ref":"Phoenix.LiveView.html#c:handle_params/3","title":"Phoenix.LiveView.handle_params/3","type":"callback"},{"doc":"The LiveView entry-point. For each LiveView in the root of a template, mount/3 is invoked twice: once to do the initial page load and again to establish the live socket. It expects three arguments: params - a map of string keys which contain public information that can be set by the user. The map contains the query params as well as any router path parameter. If the LiveView was not mounted at the router, this argument is the atom :not_mounted_at_router session - the connection session socket - the LiveView socket It must return either {:ok, socket} or {:ok, socket, options} , where options is one of: :temporary_assigns - a keyword list of assigns that are temporary and must be reset to their value after every render. Note that once the value is reset, it won't be re-rendered again until it is explicitly assigned :layout - the optional layout to be used by the LiveView","ref":"Phoenix.LiveView.html#c:mount/3","title":"Phoenix.LiveView.mount/3","type":"callback"},{"doc":"Declares a module callback to be invoked on the LiveView's mount. The function within the given module, which must be named on_mount , will be invoked before both disconnected and connected mounts. The hook has the option to either halt or continue the mounting process as usual. If you wish to redirect the LiveView, you must halt, otherwise an error will be raised. Tip: if you need to define multiple on_mount callbacks, avoid defining multiple modules. Instead, pass a tuple and use pattern matching to handle different cases: def on_mount ( :admin , _params , _session , socket ) do { :cont , socket } end def on_mount ( :user , _params , _session , socket ) do { :cont , socket } end And then invoke it as: on_mount { MyAppWeb.SomeHook , :admin } on_mount { MyAppWeb.SomeHook , :user } Registering on_mount hooks can be useful to perform authentication as well as add custom behaviour to other callbacks via attach_hook/4 . Examples The following is an example of attaching a hook via Phoenix.LiveView.Router.live_session/3 : # lib/my_app_web/live/init_assigns.ex defmodule MyAppWeb.InitAssigns do @moduledoc &quot;&quot;&quot; Ensures common `assigns` are applied to all LiveViews attaching this hook. &quot;&quot;&quot; import Phoenix.LiveView import Phoenix.Component def on_mount ( :default , _params , _session , socket ) do { :cont , assign ( socket , :page_title , &quot;DemoWeb&quot; ) } end def on_mount ( :user , params , session , socket ) do # code end def on_mount ( :admin , params , session , socket ) do # code end end # lib/my_app_web/router.ex defmodule MyAppWeb.Router do use MyAppWeb , :router # pipelines, plugs, etc. live_session :default , on_mount : MyAppWeb.InitAssigns do scope &quot;/&quot; , MyAppWeb do pipe_through :browser live &quot;/&quot; , PageLive , :index end end live_session :authenticated , on_mount : { MyAppWeb.InitAssigns , :user } do scope &quot;/&quot; , MyAppWeb do pipe_through [ :browser , :require_user ] live &quot;/profile&quot; , UserLive.Profile , :index end end live_session :admins , on_mount : { MyAppWeb.InitAssigns , :admin } do scope &quot;/admin&quot; , MyAppWeb.Admin do pipe_through [ :browser , :require_user , :require_admin ] live &quot;/&quot; , AdminLive.Index , :index end end end","ref":"Phoenix.LiveView.html#on_mount/1","title":"Phoenix.LiveView.on_mount/1","type":"macro"},{"doc":"Pushes an event to the client. Events can be handled in two ways: They can be handled on window via addEventListener . A &quot;phx:&quot; prefix will be added to the event name. They can be handled inside a hook via handleEvent . Note that events are dispatched to all active hooks on the client who are handling the given event . If you need to scope events, then this must be done by namespacing them. Hook example If you push a &quot;scores&quot; event from your LiveView: { :noreply , push_event ( socket , &quot;scores&quot; , %{ points : 100 , user : &quot;josé&quot; } ) } A hook declared via phx-hook can handle it via handleEvent : this . handleEvent ( &quot;scores&quot; , data =&gt; ... ) window example All events are also dispatched on the window . This means you can handle them by adding listeners. For example, if you want to remove an element from the page, you can do this: { :noreply , push_event ( socket , &quot;remove-el&quot; , %{ id : &quot;foo-bar&quot; } ) } And now in your app.js you can register and handle it: window . addEventListener ( &quot;phx:remove-el&quot; , e =&gt; document . getElementById ( e . detail . id ) . remove ( ) )","ref":"Phoenix.LiveView.html#push_event/3","title":"Phoenix.LiveView.push_event/3","type":"function"},{"doc":"Annotates the socket for navigation to another LiveView. The current LiveView will be shutdown and a new one will be mounted in its place, without reloading the whole page. This can also be used to remount the same LiveView, in case you want to start fresh. If you want to navigate to the same LiveView without remounting it, use push_patch/2 instead. Options :to - the required path to link to. It must always be a local path :replace - the flag to replace the current history or push a new state. Defaults false . Examples { :noreply , push_navigate ( socket , to : &quot;/&quot; ) } { :noreply , push_navigate ( socket , to : &quot;/&quot; , replace : true ) }","ref":"Phoenix.LiveView.html#push_navigate/2","title":"Phoenix.LiveView.push_navigate/2","type":"function"},{"doc":"Annotates the socket for navigation within the current LiveView. When navigating to the current LiveView, handle_params/3 is immediately invoked to handle the change of params and URL state. Then the new state is pushed to the client, without reloading the whole page while also maintaining the current scroll position. For live navigation to another LiveView, use push_navigate/2 . Options :to - the required path to link to. It must always be a local path :replace - the flag to replace the current history or push a new state. Defaults false . Examples { :noreply , push_patch ( socket , to : &quot;/&quot; ) } { :noreply , push_patch ( socket , to : &quot;/&quot; , replace : true ) }","ref":"Phoenix.LiveView.html#push_patch/2","title":"Phoenix.LiveView.push_patch/2","type":"function"},{"doc":"Annotates the socket for navigation to another LiveView. The current LiveView will be shutdown and a new one will be mounted in its place, without reloading the whole page. This can also be used to remount the same LiveView, in case you want to start fresh. If you want to navigate to the same LiveView without remounting it, use push_patch/2 instead. Options :to - the required path to link to. It must always be a local path :replace - the flag to replace the current history or push a new state. Defaults false . Examples { :noreply , push_redirect ( socket , to : &quot;/&quot; ) } { :noreply , push_redirect ( socket , to : &quot;/&quot; , replace : true ) }","ref":"Phoenix.LiveView.html#push_redirect/2","title":"Phoenix.LiveView.push_redirect/2","type":"function"},{"doc":"Adds a flash message to the socket to be displayed. Note : While you can use put_flash/3 inside a Phoenix.LiveComponent , components have their own @flash assigns. The @flash assign in a component is only copied to its parent LiveView if the component calls push_navigate/2 or push_patch/2 . Note : You must also place the Phoenix.LiveView.Router.fetch_live_flash/2 plug in your browser's pipeline in place of fetch_flash for LiveView flash messages be supported, for example: import Phoenix.LiveView.Router pipeline :browser do ... plug :fetch_live_flash end Examples iex&gt; put_flash ( socket , :info , &quot;It worked!&quot; ) iex&gt; put_flash ( socket , :error , &quot;You can&#39;t access that page&quot; )","ref":"Phoenix.LiveView.html#put_flash/3","title":"Phoenix.LiveView.put_flash/3","type":"function"},{"doc":"Annotates the socket for redirect to a destination path. Note : LiveView redirects rely on instructing client to perform a window.location update on the provided redirect location. The whole page will be reloaded and all state will be discarded. Options :to - the path to redirect to. It must always be a local path :external - an external path to redirect to. Either a string or {scheme, url} to redirect to a custom scheme","ref":"Phoenix.LiveView.html#redirect/2","title":"Phoenix.LiveView.redirect/2","type":"function"},{"doc":"Renders a template. This callback is invoked whenever LiveView detects new content must be rendered and sent to the client. If you define this function, it must return a template defined via the Phoenix.Component.sigil_H/2 . If you don't define this function, LiveView will attempt to render a template in the same directory as your LiveView. For example, if you have a LiveView named MyApp.MyCustomView inside lib/my_app/live_views/my_custom_view.ex , Phoenix will look for a template at lib/my_app/live_views/my_custom_view.html.heex .","ref":"Phoenix.LiveView.html#c:render/1","title":"Phoenix.LiveView.render/1","type":"callback"},{"doc":"Asynchronously updates a Phoenix.LiveComponent with new assigns. The :id that identifies the component must be passed as part of the assigns and it will be used to identify the live components to be updated. The pid argument is optional and it defaults to the current process, which means the update instruction will be sent to a component running on the same LiveView. If the current process is not a LiveView or you want to send updates to a live component running on another LiveView, you should explicitly pass the LiveView's pid instead. When the component receives the update, first the optional preload/1 then update/2 is invoked with the new assigns. If update/2 is not defined all assigns are simply merged into the socket. While a component may always be updated from the parent by updating some parent assigns which will re-render the child, thus invoking update/2 on the child component, send_update/3 is useful for updating a component that entirely manages its own state, as well as messaging between components mounted in the same LiveView. Examples def handle_event ( &quot;cancel-order&quot; , _ , socket ) do ... send_update ( Cart , id : &quot;cart&quot; , status : &quot;cancelled&quot; ) { :noreply , socket } end def handle_event ( &quot;cancel-order-asynchronously&quot; , _ , socket ) do ... pid = self ( ) Task . start ( fn -&gt; # Do something asynchronously send_update ( pid , Cart , id : &quot;cart&quot; , status : &quot;cancelled&quot; ) end ) { :noreply , socket } end","ref":"Phoenix.LiveView.html#send_update/3","title":"Phoenix.LiveView.send_update/3","type":"function"},{"doc":"Similar to send_update/3 but the update will be delayed according to the given time_in_milliseconds . Examples def handle_event ( &quot;cancel-order&quot; , _ , socket ) do ... send_update_after ( Cart , [ id : &quot;cart&quot; , status : &quot;cancelled&quot; ] , 3000 ) { :noreply , socket } end def handle_event ( &quot;cancel-order-asynchronously&quot; , _ , socket ) do ... pid = self ( ) Task . start ( fn -&gt; # Do something asynchronously send_update_after ( pid , Cart , [ id : &quot;cart&quot; , status : &quot;cancelled&quot; ] , 3000 ) end ) { :noreply , socket } end","ref":"Phoenix.LiveView.html#send_update_after/4","title":"Phoenix.LiveView.send_update_after/4","type":"function"},{"doc":"Returns true if the socket is connected and the tracked static assets have changed. This function is useful to detect if the client is running on an outdated version of the marked static files. It works by comparing the static paths sent by the client with the one on the server. Note: this functionality requires Phoenix v1.5.2 or later. To use this functionality, the first step is to annotate which static files you want to be tracked by LiveView, with the phx-track-static . For example: &lt; link phx - track - static rel = &quot;stylesheet&quot; href = &quot;&lt;%= Routes.static_path(@conn, &quot; / css / app . css &quot;) %&gt;&quot; / &gt; &lt; script defer phx - track - static type = &quot;text/javascript&quot; src = &quot;&lt;%= Routes.static_path(@conn, &quot; / js / app . js &quot;) %&gt;&quot; &gt; &lt; / script &gt; Now, whenever LiveView connects to the server, it will send a copy src or href attributes of all tracked statics and compare those values with the latest entries computed by mix phx.digest in the server. The tracked statics on the client will match the ones on the server the huge majority of times. However, if there is a new deployment, those values may differ. You can use this function to detect those cases and show a banner to the user, asking them to reload the page. To do so, first set the assign on mount: def mount ( params , session , socket ) do { :ok , assign ( socket , static_changed? : static_changed? ( socket ) ) } end And then in your views: &lt; % = if @static_changed? do % &gt; &lt; div id = &quot;reload-static&quot; &gt; The app has been updated . Click here to &lt; a href = &quot;#&quot; onclick = &quot;window.location.reload()&quot; &gt; reload &lt; / a &gt; . &lt; / div &gt; &lt; % end % &gt; If you prefer, you can also send a JavaScript script that immediately reloads the page. Note: only set phx-track-static on your own assets. For example, do not set it in external JavaScript files: &lt; script defer phx - track - static type = &quot;text/javascript&quot; src = &quot;https://ajax.googleapis.com/ajax/libs/jquery/3.4.1/jquery.min.js&quot; &gt; &lt; / script &gt; Because you don't actually serve the file above, LiveView will interpret the static above as missing, and this function will return true.","ref":"Phoenix.LiveView.html#static_changed?/1","title":"Phoenix.LiveView.static_changed?/1","type":"function"},{"doc":"Assigns a new stream to the socket. Streams are a mechanism for managing large collections on the client without keeping the resources on the server. name - The string or atom name of the key to place under the @streams assign. items - The enumerable of items for initial insert The following options are supported: :dom_id - The optional function to generate each stream item's DOM id. The function accepts each stream item and converts the item to a string id. By default, the :id field of a map or struct will be used if the item has such a field, and will be prefixed by the name hyphenated with the id. For example, the following definitions are equivalent: stream ( socket , :songs , songs ) stream ( socket , :songs , songs , dom_id : &amp; ( &quot; songs - \#{&amp;1.id})) Once a stream is defined, a new @streams assign is available containing the name of the defined streams. For example, in the above definition, the stream may be referenced as @streams.songs in your template. Stream items are temporary and freed from socket state as soon as they are rendered. Required DOM attributes For stream items to be trackable on the client, the following requirements must be met: The parent DOM container must include a phx-update=&quot;stream&quot; attribute, along with a unique DOM id. Each stream item must include its DOM id on the item's element. When consuming a stream in a template, the DOM id and item is passed as a tuple, allowing convenient inclusion of the DOM id for each item. For example: &lt; table &gt; &lt; tbody id = &quot;songs&quot; phx-update = &quot;stream&quot; &gt; &lt; tr :for = { { dom_id , song } &lt;- @streams . songs } id = { dom_id } &gt; &lt; td &gt; &lt;%= song . title %&gt; &lt;/ td &gt; &lt; td &gt; &lt;%= song . duration %&gt; &lt;/ td &gt; &lt;/ tr &gt; &lt;/ tbody &gt; &lt;/ table &gt; We consume the stream in a for comprehension by referencing the @streams.songs assign. We used the computed DOM id to populate the &lt;tr&gt; id, then we render the table row as usual. Now stream_insert/3 and stream_delete/3 may be issued and new rows will be inserted or deleted from the client.","ref":"Phoenix.LiveView.html#stream/4","title":"Phoenix.LiveView.stream/4","type":"function"},{"doc":"Deletes an item from the stream. The item's DOM is computed from the :dom_id provided in the stream/3 definition. Delete information for this DOM id is sent to the client and the item's element is removed from the DOM, following the same behavior of element removal, such as invoking phx-remove commands and executing client hook destroyed() callbacks. Examples def handle_event ( &quot;delete&quot; , %{ &quot;id&quot; =&gt; id } ) song = get_song! ( id ) { :noreply , stream_delete ( socket , :songs , song ) } end See stream_delete_by_dom_id/3 to remove an item without requiring the original datastructure.","ref":"Phoenix.LiveView.html#stream_delete/3","title":"Phoenix.LiveView.stream_delete/3","type":"function"},{"doc":"Deletes an item from the stream given its computed DOM id. Behaves just like stream_delete/3 , but accept the precomputed DOM id, which allows deleting from a stream without fetching or building the original stream datastructure. Examples def render ( assigns ) do ~H&quot;&quot;&quot; &lt;table&gt; &lt;tbody id=&quot;songs&quot; phx-update=&quot;stream&quot;&gt; &lt;tr :for={{dom_id, song} &lt;- @streams.songs} id={dom_id} &gt; &lt;td&gt;&lt;%= song.title %&gt;&lt;/td&gt; &lt;td&gt;&lt;button phx-click={JS.push(&quot;delete&quot;, value: %{id: dom_id})}&gt;delete&lt;/button&gt;&lt;/td&gt; &lt;/tr&gt; &lt;/tbody&gt; &lt;/table&gt; &quot;&quot;&quot; end def handle_event ( &quot;delete&quot; , %{ &quot;id&quot; =&gt; dom_id } ) { :noreply , stream_delete_by_dom_id ( socket , :songs , dom_id ) } end","ref":"Phoenix.LiveView.html#stream_delete_by_dom_id/3","title":"Phoenix.LiveView.stream_delete_by_dom_id/3","type":"function"},{"doc":"Inserts a new item or updates an existing item in the stream. By default, the item is appended to the parent DOM container. The :at option may be provided to insert or update an item to a particular index in the collection on the client. Examples Imagine you define a stream on mount with a single item: stream ( socket , :songs , [ % Song { id : 1 , title : &quot;Song 1&quot; } ] ) Then, in a callback such as handle_info or handle_event , you can append a new song: stream_insert ( socket , :songs , % Song { id : 2 , title : &quot;Song 2&quot; } ) Or prepend a new song with at: 0 : stream_insert ( socket , :songs , % Song { id : 2 , title : &quot;Song 2&quot; } , at : 0 ) Or updating an existing song, while also moving it to the top of the collection: stream_insert ( socket , :songs , % Song { id : 1 , title : &quot;Song 1 updated&quot; } , at : 0 ) Updating Items As shown, an existing item on the client can be updated by issuing a stream_insert for the existing item. When the client updates an existing item with an &quot;append&quot; operation (passing the at: -1 option), the item will remain in the same location as it was previously, and will not be moved to the end of the parent children. To both update an existing item and move it to the end of a collection, issue a stream_delete , followed by a stream_insert . For example: song = get_song! ( id ) socket |&gt; stream_delete ( :songs , song ) |&gt; stream_insert ( :songs , song , at : - 1 ) See stream_delete/3 for more information on deleting items.","ref":"Phoenix.LiveView.html#stream_insert/4","title":"Phoenix.LiveView.stream_insert/4","type":"function"},{"doc":"Invoked when the LiveView is terminating. In case of errors, this callback is only invoked if the LiveView is trapping exits. See GenServer.terminate/2 for more info.","ref":"Phoenix.LiveView.html#c:terminate/2","title":"Phoenix.LiveView.terminate/2","type":"callback"},{"doc":"Returns the transport pid of the socket. Raises ArgumentError if the socket is not connected. Examples iex&gt; transport_pid ( socket ) # PID &lt; 0 . 107 . 0 &gt;","ref":"Phoenix.LiveView.html#transport_pid/1","title":"Phoenix.LiveView.transport_pid/1","type":"function"},{"doc":"Returns the completed and in progress entries for the upload. Examples case uploaded_entries ( socket , :photos ) do { [ _ | _ ] = completed , [ ] } -&gt; # all entries are completed { [ ] , [ _ | _ ] = in_progress } -&gt; # all entries are still in progress end","ref":"Phoenix.LiveView.html#uploaded_entries/2","title":"Phoenix.LiveView.uploaded_entries/2","type":"function"},{"doc":"","ref":"Phoenix.LiveView.html#t:unsigned_params/0","title":"Phoenix.LiveView.unsigned_params/0","type":"type"},{"doc":"Helpers for rendering LiveViews from a controller.","ref":"Phoenix.LiveView.Controller.html","title":"Phoenix.LiveView.Controller","type":"module"},{"doc":"Renders a live view from a Plug request and sends an HTML response from within a controller. It also automatically sets the @live_module assign with the value of the LiveView to be rendered. Options See Phoenix.Component.live_render/3 for all supported options. Examples defmodule ThermostatController do use MyAppWeb , :controller # &quot;use MyAppWeb, :controller&quot; should import Phoenix.LiveView.Controller. # If it does not, you can either import it there or uncomment the line below: # import Phoenix.LiveView.Controller def show ( conn , %{ &quot;id&quot; =&gt; thermostat_id } ) do live_render ( conn , ThermostatLive , session : %{ &quot;thermostat_id&quot; =&gt; thermostat_id , &quot;current_user_id&quot; =&gt; get_session ( conn , :user_id ) } ) end end","ref":"Phoenix.LiveView.Controller.html#live_render/3","title":"Phoenix.LiveView.Controller.live_render/3","type":"function"},{"doc":"Provides commands for executing JavaScript utility operations on the client. JS commands support a variety of utility operations for common client-side needs, such as adding or removing CSS classes, setting or removing tag attributes, showing or hiding content, and transitioning in and out with animations. While these operations can be accomplished via client-side hooks, JS commands are DOM-patch aware, so operations applied by the JS APIs will stick to elements across patches from the server. In addition to purely client-side utilities, the JS commands include a rich push API, for extending the default phx- binding pushes with options to customize targets, loading states, and additional payload values. Client Utility Commands The following utilities are included: add_class - Add classes to elements, with optional transitions remove_class - Remove classes from elements, with optional transitions set_attribute - Set an attribute on elements remove_attribute - Remove an attribute from elements show - Show elements, with optional transitions hide - Hide elements, with optional transitions toggle - Shows or hides elements based on visibility, with optional transitions transition - Apply a temporary transition to elements for animations dispatch - Dispatch a DOM event to elements For example, the following modal component can be shown or hidden on the client without a trip to the server: alias Phoenix.LiveView.JS def hide_modal ( js \\\\ % JS { } ) do js |&gt; JS . hide ( transition : &quot;fade-out&quot; , to : &quot;#modal&quot; ) |&gt; JS . hide ( transition : &quot;fade-out-scale&quot; , to : &quot;#modal-content&quot; ) end def modal ( assigns ) do ~H&quot;&quot;&quot; &lt;div id=&quot;modal&quot; class=&quot;phx-modal&quot; phx-remove={hide_modal()}&gt; &lt;div id=&quot;modal-content&quot; class=&quot;phx-modal-content&quot; phx-click-away={hide_modal()} phx-window-keydown={hide_modal()} phx-key=&quot;escape&quot; &gt; &lt;button class=&quot;phx-modal-close&quot; phx-click={hide_modal()}&gt;✖&lt;/button&gt; &lt;p&gt;&lt;%= @text %&gt;&lt;/p&gt; &lt;/div&gt; &lt;/div&gt; &quot;&quot;&quot; end Enhanced push events The push/1 command allows you to extend the built-in pushed event handling when a phx- event is pushed to the server. For example, you may wish to target a specific component, specify additional payload values to include with the event, apply loading states to external elements, etc. For example, given this basic phx-click event: &lt; div phx - click = &quot;inc&quot; &gt; + &lt; / div &gt; Imagine you need to target your current component, and apply a loading state to the parent container while the client awaits the server acknowledgement: alias Phoenix.LiveView.JS &lt; div phx - click = { JS . push ( &quot;inc&quot; , loading : &quot;.thermo&quot; , target : @myself ) } &gt; + &lt; / div &gt; Push commands also compose with all other utilities. For example, to add a class when pushing: &lt; div phx - click = { JS . push ( &quot;inc&quot; , loading : &quot;.thermo&quot; , target : @myself ) |&gt; JS . add_class ( &quot;warmer&quot; , to : &quot;.thermo&quot; ) } &gt; + &lt; / div &gt; Custom JS events with JS.dispatch/1 and window.addEventListener dispatch/1 can be used to dispatch custom JavaScript events to elements. For example, you can use JS.dispatch(&quot;click&quot;, to: &quot;#foo&quot;) , to dispatch a click event to an element. This also means you can augment your elements with custom events, by using JavaScript's window.addEventListener and invoking them with dispatch/1 . For example, imagine you want to provide a copy-to-clipboard functionality in your application. You can add a custom event for it: window . addEventListener ( &quot;my_app:clipcopy&quot; , ( event ) =&gt; { if ( &quot;clipboard&quot; in navigator ) { const text = event . target . textContent ; navigator . clipboard . writeText ( text ) ; } else { alert ( &quot;Sorry, your browser does not support clipboard copy.&quot; ) ; } } ) ; Now you can have a button like this: &lt; button phx - click = { JS . dispatch ( &quot;my_app:clipcopy&quot; , to : &quot;#element-with-text-to-copy&quot; ) } &gt; Copy content &lt; / button &gt; The combination of dispatch/1 with window.addEventListener is a powerful mechanism to increase the amount of actions you can trigger client-side from your LiveView code. You can also use window.addEventListener to listen to events pushed from the server. You can learn more in our JS interoperability guide .","ref":"Phoenix.LiveView.JS.html","title":"Phoenix.LiveView.JS","type":"module"},{"doc":"Adds classes to elements. names - The string of classes to add. Options :to - The optional DOM selector to add classes to. Defaults to the interacted element. :transition - The string of classes to apply before adding classes or a 3-tuple containing the transition class, the class to apply to start the transition, and the ending transition class, such as: {&quot;ease-out duration-300&quot;, &quot;opacity-0&quot;, &quot;opacity-100&quot;} :time - The time to apply the transition from :transition . Defaults 200 Examples &lt; div id = &quot;item&quot; &gt; My Item &lt; / div &gt; &lt; button phx - click = { JS . add_class ( &quot;highlight underline&quot; , to : &quot;#item&quot; ) } &gt; highlight! &lt; / button &gt;","ref":"Phoenix.LiveView.JS.html#add_class/1","title":"Phoenix.LiveView.JS.add_class/1","type":"function"},{"doc":"See add_class/1 .","ref":"Phoenix.LiveView.JS.html#add_class/2","title":"Phoenix.LiveView.JS.add_class/2","type":"function"},{"doc":"See add_class/1 .","ref":"Phoenix.LiveView.JS.html#add_class/3","title":"Phoenix.LiveView.JS.add_class/3","type":"function"},{"doc":"Dispatches an event to the DOM. event - The string event name to dispatch. Note : All events dispatched are of a type CustomEvent , with the exception of &quot;click&quot; . For a &quot;click&quot; , a MouseEvent is dispatched to properly simulate a UI click. For emitted CustomEvent 's, the event detail will contain a dispatcher , which references the DOM node that dispatched the JS event to the target element. Options :to - The optional DOM selector to dispatch the event to. Defaults to the interacted element. :detail - The optional detail map to dispatch along with the client event. The details will be available in the event.detail attribute for event listeners. :bubbles – The boolean flag to bubble the event or not. Default true . Examples window . addEventListener ( &quot;click&quot; , e =&gt; console . log ( &quot;clicked!&quot; , e . detail ) ) &lt; button phx - click = { JS . dispatch ( &quot;click&quot; , to : &quot;.nav&quot; ) } &gt; Click me! &lt; / button &gt;","ref":"Phoenix.LiveView.JS.html#dispatch/2","title":"Phoenix.LiveView.JS.dispatch/2","type":"function"},{"doc":"See dispatch/2 .","ref":"Phoenix.LiveView.JS.html#dispatch/3","title":"Phoenix.LiveView.JS.dispatch/3","type":"function"},{"doc":"Sends focus to a selector. Options :to - The optional DOM selector to send focus to. Defaults to the current element. Examples JS . focus ( to : &quot;main&quot; )","ref":"Phoenix.LiveView.JS.html#focus/0","title":"Phoenix.LiveView.JS.focus/0","type":"function"},{"doc":"","ref":"Phoenix.LiveView.JS.html#focus/1","title":"Phoenix.LiveView.JS.focus/1","type":"function"},{"doc":"","ref":"Phoenix.LiveView.JS.html#focus/2","title":"Phoenix.LiveView.JS.focus/2","type":"function"},{"doc":"Sends focus to the first focusable child in selector. Options :to - The optional DOM selector to focus. Defaults to the current element. Examples JS . focus_first ( to : &quot;#modal&quot; )","ref":"Phoenix.LiveView.JS.html#focus_first/0","title":"Phoenix.LiveView.JS.focus_first/0","type":"function"},{"doc":"","ref":"Phoenix.LiveView.JS.html#focus_first/1","title":"Phoenix.LiveView.JS.focus_first/1","type":"function"},{"doc":"","ref":"Phoenix.LiveView.JS.html#focus_first/2","title":"Phoenix.LiveView.JS.focus_first/2","type":"function"},{"doc":"Hides elements. Options :to - The optional DOM selector to hide. Defaults to the interacted element. :transition - The string of classes to apply before hiding or a 3-tuple containing the transition class, the class to apply to start the transition, and the ending transition class, such as: {&quot;ease-out duration-300&quot;, &quot;opacity-0&quot;, &quot;opacity-100&quot;} :time - The time to apply the transition from :transition . Defaults 200 During the process, the following events will be dispatched to the hidden elements: When the action is triggered on the client, phx:hide-start is dispatched. After the time specified by :time , phx:hide-end is dispatched. Examples &lt; div id = &quot;item&quot; &gt; My Item &lt; / div &gt; &lt; button phx - click = { JS . hide ( to : &quot;#item&quot; ) } &gt; hide! &lt; / button &gt; &lt; button phx - click = { JS . hide ( to : &quot;#item&quot; , transition : &quot;fade-out-scale&quot; ) } &gt; hide fancy! &lt; / button &gt;","ref":"Phoenix.LiveView.JS.html#hide/1","title":"Phoenix.LiveView.JS.hide/1","type":"function"},{"doc":"See hide/1 .","ref":"Phoenix.LiveView.JS.html#hide/2","title":"Phoenix.LiveView.JS.hide/2","type":"function"},{"doc":"Sends a navigation event to the server and updates the browser's pushState history. Options :replace - Whether to replace the browser's pushState history. Defaults false. Examples JS . navigate ( &quot;/my-path&quot; )","ref":"Phoenix.LiveView.JS.html#navigate/1","title":"Phoenix.LiveView.JS.navigate/1","type":"function"},{"doc":"","ref":"Phoenix.LiveView.JS.html#navigate/2","title":"Phoenix.LiveView.JS.navigate/2","type":"function"},{"doc":"","ref":"Phoenix.LiveView.JS.html#navigate/3","title":"Phoenix.LiveView.JS.navigate/3","type":"function"},{"doc":"Sends a patch event to the server and updates the browser's pushState history. Options :replace - Whether to replace the browser's pushState history. Defaults false. Examples JS . patch ( &quot;/my-path&quot; )","ref":"Phoenix.LiveView.JS.html#patch/1","title":"Phoenix.LiveView.JS.patch/1","type":"function"},{"doc":"","ref":"Phoenix.LiveView.JS.html#patch/2","title":"Phoenix.LiveView.JS.patch/2","type":"function"},{"doc":"","ref":"Phoenix.LiveView.JS.html#patch/3","title":"Phoenix.LiveView.JS.patch/3","type":"function"},{"doc":"Focuses the last pushed element. Examples JS . pop_focus ( )","ref":"Phoenix.LiveView.JS.html#pop_focus/0","title":"Phoenix.LiveView.JS.pop_focus/0","type":"function"},{"doc":"","ref":"Phoenix.LiveView.JS.html#pop_focus/1","title":"Phoenix.LiveView.JS.pop_focus/1","type":"function"},{"doc":"Pushes an event to the server. event - The string event name to push. Options :target - The selector or component ID to push to :loading - The selector to apply the phx loading classes to :page_loading - Boolean to trigger the phx:page-loading-start and phx:page-loading-stop events for this push. Defaults to false :value - The map of values to send to the server Examples &lt; button phx - click = { JS . push ( &quot;clicked&quot; ) } &gt; click me! &lt; / button &gt; &lt; button phx - click = { JS . push ( &quot;clicked&quot; , value : %{ id : @id } ) } &gt; click me! &lt; / button &gt; &lt; button phx - click = { JS . push ( &quot;clicked&quot; , page_loading : true ) } &gt; click me! &lt; / button &gt;","ref":"Phoenix.LiveView.JS.html#push/1","title":"Phoenix.LiveView.JS.push/1","type":"function"},{"doc":"See push/1 .","ref":"Phoenix.LiveView.JS.html#push/2","title":"Phoenix.LiveView.JS.push/2","type":"function"},{"doc":"See push/1 .","ref":"Phoenix.LiveView.JS.html#push/3","title":"Phoenix.LiveView.JS.push/3","type":"function"},{"doc":"Pushes focus from the source element to be later popped. Options :to - The optional DOM selector to push focus to. Defaults to the current element. Examples JS . push_focus ( ) JS . push_focus ( to : &quot;#my-button&quot; )","ref":"Phoenix.LiveView.JS.html#push_focus/0","title":"Phoenix.LiveView.JS.push_focus/0","type":"function"},{"doc":"","ref":"Phoenix.LiveView.JS.html#push_focus/1","title":"Phoenix.LiveView.JS.push_focus/1","type":"function"},{"doc":"","ref":"Phoenix.LiveView.JS.html#push_focus/2","title":"Phoenix.LiveView.JS.push_focus/2","type":"function"},{"doc":"Removes an attribute from elements. attr - The string attribute name to remove. Options :to - The optional DOM selector to remove attributes from. Defaults to the interacted element. Examples &lt; button phx - click = { JS . remove_attribute ( &quot;aria-expanded&quot; , to : &quot;#dropdown&quot; ) } &gt; hide &lt; / button &gt;","ref":"Phoenix.LiveView.JS.html#remove_attribute/1","title":"Phoenix.LiveView.JS.remove_attribute/1","type":"function"},{"doc":"See remove_attribute/1 .","ref":"Phoenix.LiveView.JS.html#remove_attribute/2","title":"Phoenix.LiveView.JS.remove_attribute/2","type":"function"},{"doc":"See remove_attribute/1 .","ref":"Phoenix.LiveView.JS.html#remove_attribute/3","title":"Phoenix.LiveView.JS.remove_attribute/3","type":"function"},{"doc":"Removes classes from elements. names - The string of classes to remove. Options :to - The optional DOM selector to remove classes from. Defaults to the interacted element. :transition - The string of classes to apply before removing classes or a 3-tuple containing the transition class, the class to apply to start the transition, and the ending transition class, such as: {&quot;ease-out duration-300&quot;, &quot;opacity-0&quot;, &quot;opacity-100&quot;} :time - The time to apply the transition from :transition . Defaults 200 Examples &lt; div id = &quot;item&quot; &gt; My Item &lt; / div &gt; &lt; button phx - click = { JS . remove_class ( &quot;highlight underline&quot; , to : &quot;#item&quot; ) } &gt; remove highlight! &lt; / button &gt;","ref":"Phoenix.LiveView.JS.html#remove_class/1","title":"Phoenix.LiveView.JS.remove_class/1","type":"function"},{"doc":"See remove_class/1 .","ref":"Phoenix.LiveView.JS.html#remove_class/2","title":"Phoenix.LiveView.JS.remove_class/2","type":"function"},{"doc":"See remove_class/1 .","ref":"Phoenix.LiveView.JS.html#remove_class/3","title":"Phoenix.LiveView.JS.remove_class/3","type":"function"},{"doc":"Sets an attribute on elements. Accepts a tuple containing the string attribute name/value pair. Options :to - The optional DOM selector to add attributes to. Defaults to the interacted element. Examples &lt; button phx - click = { JS . set_attribute ( { &quot;aria-expanded&quot; , &quot;true&quot; } , to : &quot;#dropdown&quot; ) } &gt; show &lt; / button &gt;","ref":"Phoenix.LiveView.JS.html#set_attribute/1","title":"Phoenix.LiveView.JS.set_attribute/1","type":"function"},{"doc":"See set_attribute/1 .","ref":"Phoenix.LiveView.JS.html#set_attribute/2","title":"Phoenix.LiveView.JS.set_attribute/2","type":"function"},{"doc":"See set_attribute/1 .","ref":"Phoenix.LiveView.JS.html#set_attribute/3","title":"Phoenix.LiveView.JS.set_attribute/3","type":"function"},{"doc":"Shows elements. Options :to - The optional DOM selector to show. Defaults to the interacted element. :transition - The string of classes to apply before showing or a 3-tuple containing the transition class, the class to apply to start the transition, and the ending transition class, such as: {&quot;ease-out duration-300&quot;, &quot;opacity-0&quot;, &quot;opacity-100&quot;} :time - The time to apply the transition from :transition . Defaults 200 :display - The optional display value to set when showing. Defaults &quot;block&quot; . During the process, the following events will be dispatched to the shown elements: When the action is triggered on the client, phx:show-start is dispatched. After the time specified by :time , phx:show-end is dispatched. Examples &lt; div id = &quot;item&quot; &gt; My Item &lt; / div &gt; &lt; button phx - click = { JS . show ( to : &quot;#item&quot; ) } &gt; show! &lt; / button &gt; &lt; button phx - click = { JS . show ( to : &quot;#item&quot; , transition : &quot;fade-in-scale&quot; ) } &gt; show fancy! &lt; / button &gt;","ref":"Phoenix.LiveView.JS.html#show/1","title":"Phoenix.LiveView.JS.show/1","type":"function"},{"doc":"See show/1 .","ref":"Phoenix.LiveView.JS.html#show/2","title":"Phoenix.LiveView.JS.show/2","type":"function"},{"doc":"Toggles element visibility. Options :to - The optional DOM selector to toggle. Defaults to the interacted element. :in - The string of classes to apply when toggling in, or a 3-tuple containing the transition class, the class to apply to start the transition, and the ending transition class, such as: {&quot;ease-out duration-300&quot;, &quot;opacity-0&quot;, &quot;opacity-100&quot;} :out - The string of classes to apply when toggling out, or a 3-tuple containing the transition class, the class to apply to start the transition, and the ending transition class, such as: {&quot;ease-out duration-300&quot;, &quot;opacity-100&quot;, &quot;opacity-0&quot;} :time - The time to apply the transition :in and :out classes. Defaults 200 :display - The optional display value to set when toggling in. Defaults &quot;block&quot; . When the toggle is complete on the client, a phx:show-start or phx:hide-start , and phx:show-end or phx:hide-end event will be dispatched to the toggled elements. Examples &lt; div id = &quot;item&quot; &gt; My Item &lt; / div &gt; &lt; button phx - click = { JS . toggle ( to : &quot;#item&quot; ) } &gt; toggle item! &lt; / button &gt; &lt; button phx - click = { JS . toggle ( to : &quot;#item&quot; , in : &quot;fade-in-scale&quot; , out : &quot;fade-out-scale&quot; ) } &gt; toggle fancy! &lt; / button &gt;","ref":"Phoenix.LiveView.JS.html#toggle/1","title":"Phoenix.LiveView.JS.toggle/1","type":"function"},{"doc":"See toggle/1 .","ref":"Phoenix.LiveView.JS.html#toggle/2","title":"Phoenix.LiveView.JS.toggle/2","type":"function"},{"doc":"Transitions elements. transition - The string of classes to apply before removing classes or a 3-tuple containing the transition class, the class to apply to start the transition, and the ending transition class, such as: {&quot;ease-out duration-300&quot;, &quot;opacity-0&quot;, &quot;opacity-100&quot;} Transitions are useful for temporarily adding an animation class to element(s), such as for highlighting content changes. Options :to - The optional DOM selector to apply transitions to. Defaults to the interacted element. :time - The time to apply the transition from :transition . Defaults 200 Examples &lt; div id = &quot;item&quot; &gt; My Item &lt; / div &gt; &lt; button phx - click = { JS . transition ( &quot;shake&quot; , to : &quot;#item&quot; ) } &gt; Shake ! &lt; / button &gt;","ref":"Phoenix.LiveView.JS.html#transition/1","title":"Phoenix.LiveView.JS.transition/1","type":"function"},{"doc":"See transition/1 .","ref":"Phoenix.LiveView.JS.html#transition/2","title":"Phoenix.LiveView.JS.transition/2","type":"function"},{"doc":"See transition/1 .","ref":"Phoenix.LiveView.JS.html#transition/3","title":"Phoenix.LiveView.JS.transition/3","type":"function"},{"doc":"","ref":"Phoenix.LiveView.JS.html#t:t/0","title":"Phoenix.LiveView.JS.t/0","type":"opaque"},{"doc":"Provides LiveView routing for Phoenix routers.","ref":"Phoenix.LiveView.Router.html","title":"Phoenix.LiveView.Router","type":"module"},{"doc":"Fetches the LiveView and merges with the controller flash. Replaces the default :fetch_flash plug used by Phoenix.Router . Examples defmodule MyAppWeb.Router do use LiveGenWeb , :router import Phoenix.LiveView.Router pipeline :browser do ... plug :fetch_live_flash end ... end","ref":"Phoenix.LiveView.Router.html#fetch_live_flash/2","title":"Phoenix.LiveView.Router.fetch_live_flash/2","type":"function"},{"doc":"Defines a LiveView route. A LiveView can be routed to by using the live macro with a path and the name of the LiveView: live &quot;/thermostat&quot; , ThermostatLive By default, you can generate a route to this LiveView by using the live_path helper: live_path ( @socket , ThermostatLive ) Actions and live navigation It is common for a LiveView to have multiple states and multiple URLs. For example, you can have a single LiveView that lists all articles on your web app. For each article there is an &quot;Edit&quot; button which, when pressed, opens up a modal on the same page to edit the article. It is a best practice to use live navigation in those cases, so when you click edit, the URL changes to &quot;/articles/1/edit&quot;, even though you are still within the same LiveView. Similarly, you may also want to show a &quot;New&quot; button, which opens up the modal to create new entries, and you want this to be reflected in the URL as &quot;/articles/new&quot;. In order to make it easier to recognize the current &quot;action&quot; your LiveView is on, you can pass the action option when defining LiveViews too: live &quot;/articles&quot; , ArticleLive.Index , :index live &quot;/articles/new&quot; , ArticleLive.Index , :new live &quot;/articles/:id/edit&quot; , ArticleLive.Index , :edit When an action is given, the generated route helpers are named after the LiveView itself (in the same way as for a controller). For the example above, we will have: article_index_path ( @socket , :index ) article_index_path ( @socket , :new ) article_index_path ( @socket , :edit , 123 ) The current action will always be available inside the LiveView as the @live_action assign, that can be used to render a LiveComponent: &lt; % = if @live_action == :new do % &gt; &lt; . live_component module = { MyAppWeb.ArticleLive.FormComponent } id = &quot;form&quot; / &gt; &lt; % end % &gt; Or can be used to show or hide parts of the template: &lt; % = if @live_action == :edit do % &gt; &lt; % = render ( &quot;form.html&quot; , user : @user ) % &gt; &lt; % end % &gt; Note that @live_action will be nil if no action is given on the route definition. Options :container - an optional tuple for the HTML tag and DOM attributes to be used for the LiveView container. For example: {:li, style: &quot;color: blue;&quot;} . See Phoenix.Component.live_render/3 for more information and examples. :as - optionally configures the named helper. Defaults to :live when using a LiveView without actions or defaults to the LiveView name when using actions. :metadata - a map to optional feed metadata used on telemetry events and route info, for example: %{route_name: :foo, access: :user} . :private - an optional map of private data to put in the plug connection. for example: %{route_name: :foo, access: :user} . Examples defmodule MyApp.Router use Phoenix.Router import Phoenix.LiveView.Router scope &quot;/&quot; , MyApp do pipe_through [ :browser ] live &quot;/thermostat&quot; , ThermostatLive live &quot;/clock&quot; , ClockLive live &quot;/dashboard&quot; , DashboardLive , container : { :main , class : &quot;row&quot; } end end iex&gt; MyApp.Router.Helpers . live_path ( MyApp.Endpoint , MyApp.ThermostatLive ) &quot;/thermostat&quot;","ref":"Phoenix.LiveView.Router.html#live/4","title":"Phoenix.LiveView.Router.live/4","type":"macro"},{"doc":"Defines a live session for live redirects within a group of live routes. live_session/3 allow routes defined with live/4 to support live_redirect from the client with navigation purely over the existing websocket connection. This allows live routes defined in the router to mount a new root LiveView without additional HTTP requests to the server. Security Considerations You must always perform authentication and authorization in your LiveViews. If your application handle both regular HTTP requests and LiveViews, then you must perform authentication and authorization on both. This is important because live_redirect s do not go through the plug pipeline . live_session can be used to draw boundaries between groups of LiveViews. Redirecting between live_session s will always force a full page reload and establish a brand new LiveView connection. This is useful when LiveViews require different authentication strategies or simply when they use different root layouts (as the root layout is not updated between live redirects). Please read our guide on the security model for a detailed description and general tips on authentication, authorization, and more. Options :session - The optional extra session map or MFA tuple to be merged with the LiveView session. For example, %{&quot;admin&quot; =&gt; true} , {MyMod, :session, []} . For MFA, the function is invoked, passing the Plug.Conn struct is prepended to the arguments list. :root_layout - The optional root layout tuple for the initial HTTP render to override any existing root layout set in the router. :on_mount - The optional list of hooks to attach to the mount lifecycle of each LiveView in the session . See Phoenix.LiveView.on_mount/1 . Passing a single value is also accepted. :layout - The optional layout the LiveView will be rendered in. Examples scope &quot;/&quot; , MyAppWeb do pipe_through :browser live_session :default do live &quot;/feed&quot; , FeedLive , :index live &quot;/status&quot; , StatusLive , :index live &quot;/status/:id&quot; , StatusLive , :show end live_session :admin , on_mount : MyAppWeb.AdminLiveAuth do live &quot;/admin&quot; , AdminDashboardLive , :index live &quot;/admin/posts&quot; , AdminPostLive , :index end end In the example above, we have two live sessions. Live navigation between live views in the different sessions is not possible and will always require a full page reload. This is important in the example above because the :admin live session has authentication requirements, defined by on_mount: MyAppWeb.AdminLiveAuth , that the other LiveViews do not have. If you have both regular HTTP routes (via get, post, etc) and live routes, then you need to perform the same authentication and authorization rules in both. For example, if you were to add a get &quot;/admin/health&quot; entry point inside the :admin live session above, then you must create your own plug that performs the same authentication and authorization rules as MyAppWeb.AdminLiveAuth , and then pipe through it: live_session :admin , on_mount : MyAppWeb.AdminLiveAuth do scope &quot;/&quot; do # Regular routes pipe_through [ MyAppWeb.AdminPlugAuth ] get &quot;/admin/health&quot; , AdminHealthController , :index # Live routes live &quot;/admin&quot; , AdminDashboardLive , :index live &quot;/admin/posts&quot; , AdminPostLive , :index end end The opposite is also true, if you have regular http routes and you want to add your own live routes, the same authentication and authorization checks executed by the plugs listed in pipe_through must be ported to LiveViews and be executed via on_mount hooks.","ref":"Phoenix.LiveView.Router.html#live_session/3","title":"Phoenix.LiveView.Router.live_session/3","type":"macro"},{"doc":"Conveniences for testing function components as well as LiveViews and LiveComponents. Testing function components There are two mechanisms for testing function components. Imagine the following component: def greet ( assigns ) do ~H&quot;&quot;&quot; &lt;div&gt;Hello, &lt;%= @name %&gt;!&lt;/div&gt; &quot;&quot;&quot; end You can test it by using render_component/3 , passing the function reference to the component as first argument: import Phoenix.LiveViewTest test &quot;greets&quot; do assert render_component ( &amp; MyComponents . greet / 1 , name : &quot;Mary&quot; ) == &quot;&lt;div&gt;Hello, Mary!&lt;/div&gt;&quot; end However, for complex components, often the simplest way to test them is by using the ~H sigil itself: import Phoenix.Component import Phoenix.LiveViewTest test &quot;greets&quot; do assigns = %{ } assert rendered_to_string ( ~H&quot;&quot;&quot; &lt;MyComponents.greet name=&quot;Mary&quot; /&gt; &quot;&quot;&quot; ) == &quot;&lt;div&gt;Hello, Mary!&lt;/div&gt;&quot; end The difference is that we use rendered_to_string/1 to convert the rendered template to a string for testing. Testing LiveViews and LiveComponents In LiveComponents and LiveView tests, we interact with views via process communication in substitution of a browser. Like a browser, our test process receives messages about the rendered updates from the view which can be asserted against to test the life-cycle and behavior of LiveViews and their children. Testing LiveViews The life-cycle of a LiveView as outlined in the Phoenix.LiveView docs details how a view starts as a stateless HTML render in a disconnected socket state. Once the browser receives the HTML, it connects to the server and a new LiveView process is started, remounted in a connected socket state, and the view continues statefully. The LiveView test functions support testing both disconnected and connected mounts separately, for example: import Plug.Conn import Phoenix.ConnTest import Phoenix.LiveViewTest @endpoint MyEndpoint test &quot;disconnected and connected mount&quot; , %{ conn : conn } do conn = get ( conn , &quot;/my-path&quot; ) assert html_response ( conn , 200 ) =~ &quot;&lt;h1&gt;My Disconnected View&lt;/h1&gt;&quot; { :ok , view , html } = live ( conn ) end test &quot;redirected mount&quot; , %{ conn : conn } do assert { :error , { :redirect , %{ to : &quot;/somewhere&quot; } } } = live ( conn , &quot;my-path&quot; ) end Here, we start by using the familiar Phoenix.ConnTest function, get/2 to test the regular HTTP GET request which invokes mount with a disconnected socket. Next, live/1 is called with our sent connection to mount the view in a connected state, which starts our stateful LiveView process. In general, it's often more convenient to test the mounting of a view in a single step, provided you don't need the result of the stateless HTTP render. This is done with a single call to live/2 , which performs the get step for us: test &quot;connected mount&quot; , %{ conn : conn } do { :ok , _view , html } = live ( conn , &quot;/my-path&quot; ) assert html =~ &quot;&lt;h1&gt;My Connected View&lt;/h1&gt;&quot; end Testing Events The browser can send a variety of events to a LiveView via phx- bindings, which are sent to the handle_event/3 callback. To test events sent by the browser and assert on the rendered side effect of the event, use the render_* functions: render_click/1 - sends a phx-click event and value, returning the rendered result of the handle_event/3 callback. render_focus/2 - sends a phx-focus event and value, returning the rendered result of the handle_event/3 callback. render_blur/1 - sends a phx-blur event and value, returning the rendered result of the handle_event/3 callback. render_submit/1 - sends a form phx-submit event and value, returning the rendered result of the handle_event/3 callback. render_change/1 - sends a form phx-change event and value, returning the rendered result of the handle_event/3 callback. render_keydown/1 - sends a form phx-keydown event and value, returning the rendered result of the handle_event/3 callback. render_keyup/1 - sends a form phx-keyup event and value, returning the rendered result of the handle_event/3 callback. render_hook/3 - sends a hook event and value, returning the rendered result of the handle_event/3 callback. For example: { :ok , view , _html } = live ( conn , &quot;/thermo&quot; ) assert view |&gt; element ( &quot;button#inc&quot; ) |&gt; render_click ( ) =~ &quot;The temperature is: 31℉&quot; In the example above, we are looking for a particular element on the page and triggering its phx-click event. LiveView takes care of making sure the element has a phx-click and automatically sends its values to the server. You can also bypass the element lookup and directly trigger the LiveView event in most functions: assert render_click ( view , :inc , %{ } ) =~ &quot;The temperature is: 31℉&quot; The element style is preferred as much as possible, as it helps LiveView perform validations and ensure the events in the HTML actually matches the event names on the server. Testing regular messages LiveViews are GenServer 's under the hood, and can send and receive messages just like any other server. To test the side effects of sending or receiving messages, simply message the view and use the render function to test the result: send ( view . pid , { :set_temp , 50 } ) assert render ( view ) =~ &quot;The temperature is: 50℉&quot; Testing LiveComponents LiveComponents can be tested in two ways. One way is to use the same render_component/2 function as function components. This will mount the LiveComponent and render it once, without testing any of its events: assert render_component ( MyComponent , id : 123 , user : % User { } ) =~ &quot;some markup in component&quot; However, if you want to test how components are mounted by a LiveView and interact with DOM events, you must use the regular live/2 macro to build the LiveView with the component and then scope events by passing the view and a DOM selector in a list: { :ok , view , html } = live ( conn , &quot;/users&quot; ) html = view |&gt; element ( &quot;#user-13 a&quot; , &quot;Delete&quot; ) |&gt; render_click ( ) refute html =~ &quot;user-13&quot; refute view |&gt; element ( &quot;#user-13&quot; ) |&gt; has_element? ( ) In the example above, LiveView will lookup for an element with ID=user-13 and retrieve its phx-target . If phx-target points to a component, that will be the component used, otherwise it will fallback to the view.","ref":"Phoenix.LiveViewTest.html","title":"Phoenix.LiveViewTest","type":"module"},{"doc":"Asserts a live patch will happen within timeout milliseconds. The default timeout is 100. It returns the new path. To assert on the flash message, you can assert on the result of the rendered LiveView. Examples render_click ( view , :event_that_triggers_patch ) assert_patch view render_click ( view , :event_that_triggers_patch ) assert_patch view , 30 render_click ( view , :event_that_triggers_patch ) path = assert_patch view assert path =~ ~r/path/ \d + /","ref":"Phoenix.LiveViewTest.html#assert_patch/2","title":"Phoenix.LiveViewTest.assert_patch/2","type":"function"},{"doc":"Asserts a live patch will happen to a given path within timeout milliseconds. The default timeout is 100. It always returns :ok . To assert on the flash message, you can assert on the result of the rendered LiveView. Examples render_click ( view , :event_that_triggers_patch ) assert_patch view , &quot;/path&quot; render_click ( view , :event_that_triggers_patch ) assert_patch view , &quot;/path&quot; , 30","ref":"Phoenix.LiveViewTest.html#assert_patch/3","title":"Phoenix.LiveViewTest.assert_patch/3","type":"function"},{"doc":"Asserts a live patch was performed, and returns the new path. To assert on the flash message, you can assert on the result of the rendered LiveView. Examples render_click ( view , :event_that_triggers_redirect ) assert_patched view , &quot;/path&quot;","ref":"Phoenix.LiveViewTest.html#assert_patched/2","title":"Phoenix.LiveViewTest.assert_patched/2","type":"function"},{"doc":"Asserts an event will be pushed within timeout . Examples assert_push_event view , &quot;scores&quot; , %{ points : 100 , user : &quot;josé&quot; }","ref":"Phoenix.LiveViewTest.html#assert_push_event/4","title":"Phoenix.LiveViewTest.assert_push_event/4","type":"macro"},{"doc":"Asserts a redirect will happen within timeout milliseconds. The default timeout is 100. It returns a tuple containing the new path and the flash messages from said redirect, if any. Note the flash will contain string keys. Examples render_click ( view , :event_that_triggers_redirect ) { path , flash } = assert_redirect view assert flash [ &quot;info&quot; ] == &quot;Welcome&quot; assert path =~ ~r/path \\/ \\d +/ render_click ( view , :event_that_triggers_redirect ) assert_redirect view , 30","ref":"Phoenix.LiveViewTest.html#assert_redirect/2","title":"Phoenix.LiveViewTest.assert_redirect/2","type":"function"},{"doc":"Asserts a redirect will happen to a given path within timeout milliseconds. The default timeout is 100. It returns the flash messages from said redirect, if any. Note the flash will contain string keys. Examples render_click ( view , :event_that_triggers_redirect ) flash = assert_redirect view , &quot;/path&quot; assert flash [ &quot;info&quot; ] == &quot;Welcome&quot; render_click ( view , :event_that_triggers_redirect ) assert_redirect view , &quot;/path&quot; , 30","ref":"Phoenix.LiveViewTest.html#assert_redirect/3","title":"Phoenix.LiveViewTest.assert_redirect/3","type":"function"},{"doc":"Asserts a redirect was performed. It returns the flash messages from said redirect, if any. Note the flash will contain string keys. Examples render_click ( view , :event_that_triggers_redirect ) flash = assert_redirected view , &quot;/path&quot; assert flash [ &quot;info&quot; ] == &quot;Welcome&quot;","ref":"Phoenix.LiveViewTest.html#assert_redirected/2","title":"Phoenix.LiveViewTest.assert_redirected/2","type":"function"},{"doc":"Asserts a hook reply was returned from a handle_event callback. Examples assert_reply view , %{ result : &quot;ok&quot; , transaction_id : _ }","ref":"Phoenix.LiveViewTest.html#assert_reply/3","title":"Phoenix.LiveViewTest.assert_reply/3","type":"macro"},{"doc":"Returns an element to scope a function to. It expects the current LiveView, a query selector, and a text filter. An optional text filter may be given to filter the results by the query selector. If the text filter is a string or a regex, it will match any element that contains the string (including as a substring) or matches the regex. So a link containing the text &quot;unopened&quot; will match element(&quot;a&quot;, &quot;opened&quot;) . To prevent this, a regex could specify that &quot;opened&quot; appear without the prefix &quot;un&quot;. For example, element(&quot;a&quot;, ~r{(?&lt;!un)opened}) . But it may be clearer to add an HTML attribute to make the element easier to select. After the text filter is applied, only one element must remain, otherwise an error is raised. If no text filter is given, then the query selector itself must return a single element. assert view |&gt; element ( &quot;#term a:first-child&quot; , &quot;Increment&quot; ) |&gt; render ( ) =~ &quot;Increment&lt;/a&gt;&quot; Attribute selectors are also supported, and may be used on special cases like ids which contain periods: assert view |&gt; element ( ~s{[href=&quot;/foo&quot;][id=&quot;foo.bar.baz&quot;]} ) |&gt; render ( ) =~ &quot;Increment&lt;/a&gt;&quot;","ref":"Phoenix.LiveViewTest.html#element/3","title":"Phoenix.LiveViewTest.element/3","type":"function"},{"doc":"Builds a file input for testing uploads within a form. Given the form DOM selector, the upload name, and a list of maps of client metadata for the upload, the returned file input can be passed to render_upload/2 . Client metadata takes the following form: :last_modified - the last modified timestamp :name - the name of the file :content - the binary content of the file :size - the byte size of the content :type - the MIME type of the file Examples avatar = file_input ( lv , &quot;#my-form-id&quot; , :avatar , [ %{ last_modified : 1_594_171_879_000 , name : &quot;myfile.jpeg&quot; , content : File . read! ( &quot;myfile.jpg&quot; ) , size : 1_396_009 , type : &quot;image/jpeg&quot; } ] ) assert render_upload ( avatar , &quot;myfile.jpeg&quot; ) =~ &quot;100%&quot;","ref":"Phoenix.LiveViewTest.html#file_input/4","title":"Phoenix.LiveViewTest.file_input/4","type":"macro"},{"doc":"Gets the nested LiveView child by child_id from the parent LiveView. Examples { :ok , view , _html } = live ( conn , &quot;/thermo&quot; ) assert clock_view = find_live_child ( view , &quot;clock&quot; ) assert render_click ( clock_view , :snooze ) =~ &quot;snoozing&quot;","ref":"Phoenix.LiveViewTest.html#find_live_child/2","title":"Phoenix.LiveViewTest.find_live_child/2","type":"function"},{"doc":"Follows the redirect from a render_* action or an {:error, redirect} tuple. Imagine you have a LiveView that redirects on a render_click event. You can make sure it immediately redirects after the render_click action by calling follow_redirect/3 : live_view |&gt; render_click ( &quot;redirect&quot; ) |&gt; follow_redirect ( conn ) Or in the case of an error tuple: assert { :error , { :redirect , %{ to : &quot;/somewhere&quot; } } } = result = live ( conn , &quot;my-path&quot; ) { :ok , view , html } = follow_redirect ( result , conn ) follow_redirect/3 expects a connection as second argument. This is the connection that will be used to perform the underlying request. If the LiveView redirects with a live redirect, this macro returns {:ok, live_view, disconnected_html} with the content of the new LiveView, the same as the live/3 macro. If the LiveView redirects with a regular redirect, this macro returns {:ok, conn} with the rendered redirected page. In any other case, this macro raises. Finally, note that you can optionally assert on the path you are being redirected to by passing a third argument: live_view |&gt; render_click ( &quot;redirect&quot; ) |&gt; follow_redirect ( conn , &quot;/redirected/page&quot; )","ref":"Phoenix.LiveViewTest.html#follow_redirect/3","title":"Phoenix.LiveViewTest.follow_redirect/3","type":"macro"},{"doc":"Receives a form_element and asserts that phx-trigger-action has been set to true, following up on that request. Imagine you have a LiveView that sends an HTTP form submission. Say that it sets the phx-trigger-action to true, as a response to a submit event. You can follow the trigger action like this: form = form ( live_view , selector , %{ &quot;form&quot; =&gt; &quot;data&quot; } ) # First we submit the form. Optionally verify that phx-trigger-action # is now part of the form. assert render_submit ( form ) =~ ~r/phx-trigger-action/ # Now follow the request made by the form conn = follow_trigger_action ( form , conn ) assert conn . method == &quot;POST&quot; assert conn . params == %{ &quot;form&quot; =&gt; &quot;data&quot; }","ref":"Phoenix.LiveViewTest.html#follow_trigger_action/2","title":"Phoenix.LiveViewTest.follow_trigger_action/2","type":"macro"},{"doc":"Returns a form element to scope a function to. It expects the current LiveView, a query selector, and the form data. The query selector must return a single element. The form data will be validated directly against the form markup and make sure the data you are changing/submitting actually exists, failing otherwise. Examples assert view |&gt; form ( &quot;#term&quot; , user : %{ name : &quot;hello&quot; } ) |&gt; render_submit ( ) =~ &quot;Name updated&quot; This function is meant to mimic what the user can actually do, so you cannot set hidden input values. However, hidden values can be given when calling render_submit/2 or render_change/2 , see their docs for examples.","ref":"Phoenix.LiveViewTest.html#form/3","title":"Phoenix.LiveViewTest.form/3","type":"function"},{"doc":"Checks if the given element exists on the page. Examples assert view |&gt; element ( &quot;#some-element&quot; ) |&gt; has_element? ( )","ref":"Phoenix.LiveViewTest.html#has_element?/1","title":"Phoenix.LiveViewTest.has_element?/1","type":"function"},{"doc":"Checks if the given selector with text_filter is on view . See element/3 for more information. Examples assert has_element? ( view , &quot;#some-element&quot; )","ref":"Phoenix.LiveViewTest.html#has_element?/3","title":"Phoenix.LiveViewTest.has_element?/3","type":"function"},{"doc":"Spawns a connected LiveView process. If a path is given, then a regular get(conn, path) is done and the page is upgraded to a LiveView . If no path is given, it assumes a previously rendered %Plug.Conn{} is given, which will be converted to a LiveView immediately. Examples { :ok , view , html } = live ( conn , &quot;/path&quot; ) assert view . module == MyLive assert html =~ &quot;the count is 3&quot; assert { :error , { :redirect , %{ to : &quot;/somewhere&quot; } } } = live ( conn , &quot;/path&quot; )","ref":"Phoenix.LiveViewTest.html#live/2","title":"Phoenix.LiveViewTest.live/2","type":"macro"},{"doc":"Returns the current list of LiveView children for the parent LiveView. Children are returned in the order they appear in the rendered HTML. Examples { :ok , view , _html } = live ( conn , &quot;/thermo&quot; ) assert [ clock_view ] = live_children ( view ) assert render_click ( clock_view , :snooze ) =~ &quot;snoozing&quot;","ref":"Phoenix.LiveViewTest.html#live_children/1","title":"Phoenix.LiveViewTest.live_children/1","type":"function"},{"doc":"Spawns a connected LiveView process mounted in isolation as the sole rendered element. Useful for testing LiveViews that are not directly routable, such as those built as small components to be re-used in multiple parents. Testing routable LiveViews is still recommended whenever possible since features such as live navigation require routable LiveViews. Options :session - the session to be given to the LiveView All other options are forwarded to the LiveView for rendering. Refer to Phoenix.Component.live_render/3 for a list of supported render options. Examples { :ok , view , html } = live_isolated ( conn , MyAppWeb.ClockLive , session : %{ &quot;tz&quot; =&gt; &quot;EST&quot; } ) Use put_connect_params/2 to put connect params for a call to Phoenix.LiveView.get_connect_params/1 in Phoenix.LiveView.mount/3 : { :ok , view , html } = conn |&gt; put_connect_params ( %{ &quot;param&quot; =&gt; &quot;value&quot; } ) |&gt; live_isolated ( AppWeb.ClockLive , session : %{ &quot;tz&quot; =&gt; &quot;EST&quot; } )","ref":"Phoenix.LiveViewTest.html#live_isolated/3","title":"Phoenix.LiveViewTest.live_isolated/3","type":"macro"},{"doc":"Performs a live redirect from one LiveView to another. When redirecting between two LiveViews of the same live_session , mounts the new LiveView and shutsdown the previous one, which mimics general browser live navigation behaviour. When attempting to navigate from a LiveView of a different live_session , an error redirect condition is returned indicating a failed live_redirect from the client. Examples assert { :ok , page_live , _html } = live ( conn , &quot;/page/1&quot; ) assert { :ok , page2_live , _html } = live ( conn , &quot;/page/2&quot; ) assert { :error , { :redirect , _ } } = live_redirect ( page2_live , to : &quot;/admin&quot; )","ref":"Phoenix.LiveViewTest.html#live_redirect/2","title":"Phoenix.LiveViewTest.live_redirect/2","type":"function"},{"doc":"Open the default browser to display current HTML of view_or_element . Examples view |&gt; element ( &quot;#term a:first-child&quot; , &quot;Increment&quot; ) |&gt; open_browser ( ) assert view |&gt; form ( &quot;#term&quot; , user : %{ name : &quot;hello&quot; } ) |&gt; open_browser ( ) |&gt; render_submit ( ) =~ &quot;Name updated&quot;","ref":"Phoenix.LiveViewTest.html#open_browser/2","title":"Phoenix.LiveViewTest.open_browser/2","type":"function"},{"doc":"Returns the most recent title that was updated via a page_title assign. Examples render_click ( view , :event_that_triggers_page_title_update ) assert page_title ( view ) =~ &quot;my title&quot;","ref":"Phoenix.LiveViewTest.html#page_title/1","title":"Phoenix.LiveViewTest.page_title/1","type":"function"},{"doc":"Performs a preflight upload request. Useful for testing external uploaders to retrieve the :external entry metadata. Examples avatar = file_input ( lv , &quot;#my-form-id&quot; , :avatar , [ %{ name : ... , ... } , ... ] ) assert { :ok , %{ ref : _ref , config : %{ chunk_size : _ } } } = preflight_upload ( avatar )","ref":"Phoenix.LiveViewTest.html#preflight_upload/1","title":"Phoenix.LiveViewTest.preflight_upload/1","type":"function"},{"doc":"","ref":"Phoenix.LiveViewTest.html#put_connect_info/2","title":"Phoenix.LiveViewTest.put_connect_info/2","type":"function"},{"doc":"Puts connect params to be used on LiveView connections. See Phoenix.LiveView.get_connect_params/1 .","ref":"Phoenix.LiveViewTest.html#put_connect_params/2","title":"Phoenix.LiveViewTest.put_connect_params/2","type":"function"},{"doc":"Refutes a redirect to a given path was performed. It returns :ok if the specified redirect isn't already in the mailbox. Examples render_click ( view , :event_that_triggers_redirect_to_path ) :ok = refute_redirected view , &quot;/wrong_path&quot;","ref":"Phoenix.LiveViewTest.html#refute_redirected/2","title":"Phoenix.LiveViewTest.refute_redirected/2","type":"function"},{"doc":"Returns the HTML string of the rendered view or element. If a view is provided, the entire LiveView is rendered. If a view after calling with_target/2 or an element are given, only that particular context is returned. Examples { :ok , view , _html } = live ( conn , &quot;/thermo&quot; ) assert render ( view ) =~ ~s|&lt;button id=&quot;alarm&quot;&gt;Snooze&lt;/div&gt;| assert view |&gt; element ( &quot;#alarm&quot; ) |&gt; render ( ) == &quot;Snooze&quot;","ref":"Phoenix.LiveViewTest.html#render/1","title":"Phoenix.LiveViewTest.render/1","type":"function"},{"doc":"Sends a blur event given by element and returns the rendered result. The element is created with element/3 and must point to a single element on the page with a phx-blur attribute in it. The event name given set on phx-blur is then sent to the appropriate LiveView (or component if phx-target is set accordingly). All phx-value-* entries in the element are sent as values. Extra values can be given with the value argument. It returns the contents of the whole LiveView or an {:error, redirect} tuple. Examples { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert view |&gt; element ( &quot;#inactive&quot; ) |&gt; render_blur ( ) =~ &quot;Tap to wake&quot;","ref":"Phoenix.LiveViewTest.html#render_blur/2","title":"Phoenix.LiveViewTest.render_blur/2","type":"function"},{"doc":"Sends a blur event to the view and returns the rendered result. It returns the contents of the whole LiveView or an {:error, redirect} tuple. Examples { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert html =~ &quot;The temp is: 30℉&quot; assert render_blur ( view , :inactive ) =~ &quot;Tap to wake&quot;","ref":"Phoenix.LiveViewTest.html#render_blur/3","title":"Phoenix.LiveViewTest.render_blur/3","type":"function"},{"doc":"Sends a form change event given by element and returns the rendered result. The element is created with element/3 and must point to a single element on the page with a phx-change attribute in it. The event name given set on phx-change is then sent to the appropriate LiveView (or component if phx-target is set accordingly). All phx-value-* entries in the element are sent as values. If you need to pass any extra values or metadata, such as the &quot;_target&quot; parameter, you can do so by giving a map under the value argument. It returns the contents of the whole LiveView or an {:error, redirect} tuple. Examples { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert view |&gt; element ( &quot;form&quot; ) |&gt; render_change ( %{ deg : 123 } ) =~ &quot;123 exceeds limits&quot; # Passing metadata { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert view |&gt; element ( &quot;form&quot; ) |&gt; render_change ( %{ _target : [ &quot;deg&quot; ] , deg : 123 } ) =~ &quot;123 exceeds limits&quot; As with render_submit/2 , hidden input field values can be provided like so: refute view |&gt; form ( &quot;#term&quot; , user : %{ name : &quot;hello&quot; } ) |&gt; render_change ( %{ user : %{ &quot;hidden_field&quot; =&gt; &quot;example&quot; } } ) =~ &quot;can&#39;t be blank&quot;","ref":"Phoenix.LiveViewTest.html#render_change/2","title":"Phoenix.LiveViewTest.render_change/2","type":"function"},{"doc":"Sends a form change event to the view and returns the rendered result. It returns the contents of the whole LiveView or an {:error, redirect} tuple. Examples { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert html =~ &quot;The temp is: 30℉&quot; assert render_change ( view , :validate , %{ deg : 123 } ) =~ &quot;123 exceeds limits&quot;","ref":"Phoenix.LiveViewTest.html#render_change/3","title":"Phoenix.LiveViewTest.render_change/3","type":"function"},{"doc":"Sends a click event given by element and returns the rendered result. The element is created with element/3 and must point to a single element on the page with a phx-click attribute in it. The event name given set on phx-click is then sent to the appropriate LiveView (or component if phx-target is set accordingly). All phx-value-* entries in the element are sent as values. Extra values can be given with the value argument. If the element does not have a phx-click attribute but it is a link (the &lt;a&gt; tag), the link will be followed accordingly: if the link is a live_patch , the current view will be patched if the link is a live_redirect , this function will return {:error, {:live_redirect, %{to: url}}} , which can be followed with follow_redirect/2 if the link is a regular link, this function will return {:error, {:redirect, %{to: url}}} , which can be followed with follow_redirect/2 It returns the contents of the whole LiveView or an {:error, redirect} tuple. Examples { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert view |&gt; element ( &quot;button&quot; , &quot;Increment&quot; ) |&gt; render_click ( ) =~ &quot;The temperature is: 30℉&quot;","ref":"Phoenix.LiveViewTest.html#render_click/2","title":"Phoenix.LiveViewTest.render_click/2","type":"function"},{"doc":"Sends a click event to the view with value and returns the rendered result. It returns the contents of the whole LiveView or an {:error, redirect} tuple. Examples { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert html =~ &quot;The temperature is: 30℉&quot; assert render_click ( view , :inc ) =~ &quot;The temperature is: 31℉&quot;","ref":"Phoenix.LiveViewTest.html#render_click/3","title":"Phoenix.LiveViewTest.render_click/3","type":"function"},{"doc":"Renders a component. The first argument may either be a function component, as an anonymous function: assert render_component ( &amp; Weather . city / 1 , name : &quot;Kraków&quot; ) =~ &quot;some markup in component&quot; Or a stateful component as a module. In this case, this function will mount, update, and render the component. The :id option is a required argument: assert render_component ( MyComponent , id : 123 , user : % User { } ) =~ &quot;some markup in component&quot; If your component is using the router, you can pass it as argument: assert render_component ( MyComponent , %{ id : 123 , user : % User { } } , router : SomeRouter ) =~ &quot;some markup in component&quot;","ref":"Phoenix.LiveViewTest.html#render_component/3","title":"Phoenix.LiveViewTest.render_component/3","type":"macro"},{"doc":"Sends a focus event given by element and returns the rendered result. The element is created with element/3 and must point to a single element on the page with a phx-focus attribute in it. The event name given set on phx-focus is then sent to the appropriate LiveView (or component if phx-target is set accordingly). All phx-value-* entries in the element are sent as values. Extra values can be given with the value argument. It returns the contents of the whole LiveView or an {:error, redirect} tuple. Examples { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert view |&gt; element ( &quot;#inactive&quot; ) |&gt; render_focus ( ) =~ &quot;Tap to wake&quot;","ref":"Phoenix.LiveViewTest.html#render_focus/2","title":"Phoenix.LiveViewTest.render_focus/2","type":"function"},{"doc":"Sends a focus event to the view and returns the rendered result. It returns the contents of the whole LiveView or an {:error, redirect} tuple. Examples { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert html =~ &quot;The temp is: 30℉&quot; assert render_focus ( view , :inactive ) =~ &quot;Tap to wake&quot;","ref":"Phoenix.LiveViewTest.html#render_focus/3","title":"Phoenix.LiveViewTest.render_focus/3","type":"function"},{"doc":"Sends a hook event to the view or an element and returns the rendered result. It returns the contents of the whole LiveView or an {:error, redirect} tuple. Examples { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert html =~ &quot;The temp is: 30℉&quot; assert render_hook ( view , :refresh , %{ deg : 32 } ) =~ &quot;The temp is: 32℉&quot; If you are pushing events from a hook to a component, then you must pass an element , created with element/3 , as first argument and it must point to a single element on the page with a phx-target attribute in it: { :ok , view , _html } = live ( conn , &quot;/thermo&quot; ) assert view |&gt; element ( &quot;#thermo-component&quot; ) |&gt; render_hook ( :refresh , %{ deg : 32 } ) =~ &quot;The temp is: 32℉&quot;","ref":"Phoenix.LiveViewTest.html#render_hook/3","title":"Phoenix.LiveViewTest.render_hook/3","type":"function"},{"doc":"Sends a keydown event given by element and returns the rendered result. The element is created with element/3 and must point to a single element on the page with a phx-keydown or phx-window-keydown attribute in it. The event name given set on phx-keydown is then sent to the appropriate LiveView (or component if phx-target is set accordingly). All phx-value-* entries in the element are sent as values. Extra values can be given with the value argument. It returns the contents of the whole LiveView or an {:error, redirect} tuple. Examples { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert html =~ &quot;The temp is: 30℉&quot; assert view |&gt; element ( &quot;#inc&quot; ) |&gt; render_keydown ( ) =~ &quot;The temp is: 31℉&quot;","ref":"Phoenix.LiveViewTest.html#render_keydown/2","title":"Phoenix.LiveViewTest.render_keydown/2","type":"function"},{"doc":"Sends a keydown event to the view and returns the rendered result. It returns the contents of the whole LiveView or an {:error, redirect} tuple. Examples { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert html =~ &quot;The temp is: 30℉&quot; assert render_keydown ( view , :inc ) =~ &quot;The temp is: 31℉&quot;","ref":"Phoenix.LiveViewTest.html#render_keydown/3","title":"Phoenix.LiveViewTest.render_keydown/3","type":"function"},{"doc":"Sends a keyup event given by element and returns the rendered result. The element is created with element/3 and must point to a single element on the page with a phx-keyup or phx-window-keyup attribute in it. The event name given set on phx-keyup is then sent to the appropriate LiveView (or component if phx-target is set accordingly). All phx-value-* entries in the element are sent as values. Extra values can be given with the value argument. It returns the contents of the whole LiveView or an {:error, redirect} tuple. Examples { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert html =~ &quot;The temp is: 30℉&quot; assert view |&gt; element ( &quot;#inc&quot; ) |&gt; render_keyup ( ) =~ &quot;The temp is: 31℉&quot;","ref":"Phoenix.LiveViewTest.html#render_keyup/2","title":"Phoenix.LiveViewTest.render_keyup/2","type":"function"},{"doc":"Sends a keyup event to the view and returns the rendered result. It returns the contents of the whole LiveView or an {:error, redirect} tuple. Examples { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert html =~ &quot;The temp is: 30℉&quot; assert render_keyup ( view , :inc ) =~ &quot;The temp is: 31℉&quot;","ref":"Phoenix.LiveViewTest.html#render_keyup/3","title":"Phoenix.LiveViewTest.render_keyup/3","type":"function"},{"doc":"Simulates a live_patch to the given path and returns the rendered result.","ref":"Phoenix.LiveViewTest.html#render_patch/2","title":"Phoenix.LiveViewTest.render_patch/2","type":"function"},{"doc":"Sends a form submit event given by element and returns the rendered result. The element is created with element/3 and must point to a single element on the page with a phx-submit attribute in it. The event name given set on phx-submit is then sent to the appropriate LiveView (or component if phx-target is set accordingly). All phx-value-* entries in the element are sent as values. Extra values, including hidden input fields, can be given with the value argument. It returns the contents of the whole LiveView or an {:error, redirect} tuple. Examples { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert view |&gt; element ( &quot;form&quot; ) |&gt; render_submit ( %{ deg : 123 , avatar : upload } ) =~ &quot;123 exceeds limits&quot; To submit a form along with some with hidden input values: assert view |&gt; form ( &quot;#term&quot; , user : %{ name : &quot;hello&quot; } ) |&gt; render_submit ( %{ user : %{ &quot;hidden_field&quot; =&gt; &quot;example&quot; } } ) =~ &quot;Name updated&quot;","ref":"Phoenix.LiveViewTest.html#render_submit/2","title":"Phoenix.LiveViewTest.render_submit/2","type":"function"},{"doc":"Sends a form submit event to the view and returns the rendered result. It returns the contents of the whole LiveView or an {:error, redirect} tuple. Examples { :ok , view , html } = live ( conn , &quot;/thermo&quot; ) assert html =~ &quot;The temp is: 30℉&quot; assert render_submit ( view , :refresh , %{ deg : 32 } ) =~ &quot;The temp is: 32℉&quot;","ref":"Phoenix.LiveViewTest.html#render_submit/3","title":"Phoenix.LiveViewTest.render_submit/3","type":"function"},{"doc":"Performs an upload of a file input and renders the result. See file_input/4 for details on building a file input. Examples Given the following LiveView template: &lt; % = for entry &lt;- @uploads . avatar . entries do % &gt; &lt; % = entry . name % &gt; : &lt; % = entry . progress % &gt; % &lt; % end % &gt; Your test case can assert the uploaded content: avatar = file_input ( lv , &quot;#my-form-id&quot; , :avatar , [ %{ last_modified : 1_594_171_879_000 , name : &quot;myfile.jpeg&quot; , content : File . read! ( &quot;myfile.jpg&quot; ) , size : 1_396_009 , type : &quot;image/jpeg&quot; } ] ) assert render_upload ( avatar , &quot;myfile.jpeg&quot; ) =~ &quot;100%&quot; By default, the entire file is chunked to the server, but an optional percentage to chunk can be passed to test chunk-by-chunk uploads: assert render_upload ( avatar , &quot;myfile.jpeg&quot; , 49 ) =~ &quot;49%&quot; assert render_upload ( avatar , &quot;myfile.jpeg&quot; , 51 ) =~ &quot;100%&quot;","ref":"Phoenix.LiveViewTest.html#render_upload/3","title":"Phoenix.LiveViewTest.render_upload/3","type":"function"},{"doc":"Converts a rendered template to a string. Examples import Phoenix.Component import Phoenix.LiveViewTest test &quot;greets&quot; do assigns = %{ } assert rendered_to_string ( ~H&quot;&quot;&quot; &lt;MyComponents.greet name=&quot;Mary&quot; /&gt; &quot;&quot;&quot; ) == &quot;&lt;div&gt;Hello, Mary!&lt;/div&gt;&quot; end","ref":"Phoenix.LiveViewTest.html#rendered_to_string/1","title":"Phoenix.LiveViewTest.rendered_to_string/1","type":"function"},{"doc":"Receives a form element and submits the HTTP request through the plug pipeline. Imagine you have a LiveView that validates form data, but submits the form to a controller via the normal form action attribute. This is especially useful in scenarios where the result of a form submit needs to write to the plug session. You can follow submit the form with the %Plug.Conn{} , like this: form = form ( live_view , selector , %{ &quot;form&quot; =&gt; &quot;data&quot; } ) # Now submit the LiveView form to the plug pipeline conn = submit_form ( form , conn ) assert conn . method == &quot;POST&quot; assert conn . params == %{ &quot;form&quot; =&gt; &quot;data&quot; }","ref":"Phoenix.LiveViewTest.html#submit_form/2","title":"Phoenix.LiveViewTest.submit_form/2","type":"macro"},{"doc":"Sets the target of the view for events. This emulates phx-target directly in tests, without having to dispatch the event to a specific element. This can be useful for invoking events to one or multiple components at the same time: view |&gt; with_target ( &quot;#user-1,#user-2&quot; ) |&gt; render_click ( &quot;Hide&quot; , %{ } )","ref":"Phoenix.LiveViewTest.html#with_target/2","title":"Phoenix.LiveViewTest.with_target/2","type":"function"},{"doc":"Format HEEx templates from .heex files or ~H sigils. This is a mix format plugin . Note: The HEEx HTML Formatter requires Elixir v1.13.4 or later. Setup Add it as plugin to your .formatter.exs file and make sure to put the heex extension in the inputs option. [ plugins : [ Phoenix.LiveView.HTMLFormatter ] , inputs : [ &quot;*.{heex,ex,exs}&quot; , &quot;priv/*/seeds.exs&quot; , &quot;{config,lib,test}/**/*.{heex,ex,exs}&quot; ] , # ... ] For umbrella projects In umbrella projects you must also change two files at the umbrella root, add :phoenix_live_view to your deps in the mix.exs file and add plugins: [Phoenix.LiveView.HTMLFormatter] in the .formatter.exs file. This is because the formatter does not attempt to load the dependencies of all children applications. Editor support Most editors that support mix format integration should automatically format .heex and ~H templates. Other editors may require custom integration or even provide additional functionality. Here are some reference posts: Formatting HEEx templates in VS Code Options :line_length - The Elixir formatter defaults to a maximum line length of 98 characters, which can be overwritten with the :line_length option in your .formatter.exs file. :heex_line_length - change the line length only for the HEEx formatter. [ # ...omitted heex_line_length : 300 ] Formatting This formatter tries to be as consistent as possible with the Elixir formatter. Given HTML like this: &lt;section&gt;&lt;h1&gt; &lt;b&gt; &lt;%= @user . name %&gt; &lt;/b&gt;&lt;/h1&gt;&lt;/section&gt; It will be formatted as: &lt;section&gt; &lt;h1&gt;&lt;b&gt; &lt;%= @user . name %&gt; &lt;/b&gt;&lt;/h1&gt; &lt;/section&gt; A block element will go to the next line, while inline elements will be kept in the current line as long as they fit within the configured line length. The following links list all block and inline elements. https://developer.mozilla.org/en-US/docs/Web/HTML/Block-level_elements#elements https://developer.mozilla.org/en-US/docs/Web/HTML/Inline_elements#list_of_inline_elements It will also keep inline elements in their own lines if you intentionally write them this way: &lt;section&gt; &lt;h1&gt; &lt;b&gt; &lt;%= @user . name %&gt; &lt;/b&gt; &lt;/h1&gt; &lt;/section&gt; This formatter will place all attributes on their own lines when they do not all fit in the current line. Therefore this: &lt;section id=&quot;user-section-id&quot; class=&quot;sm:focus:block flex w-full p-3&quot; phx-click=&quot;send-event&quot;&gt; &lt;p&gt;Hi&lt;/p&gt; &lt;/section&gt; Will be formatted to: &lt;section id=&quot;user-section-id&quot; class=&quot;sm:focus:block flex w-full p-3&quot; phx-click=&quot;send-event&quot; &gt; &lt;p&gt;Hi&lt;/p&gt; &lt;/section&gt; This formatter does not format Elixir expressions with do...end . The content within it will be formatted accordingly though. Therefore, the given input: &lt;%= live_redirect ( to : &quot;/my/path&quot; , class : &quot;my class&quot; ) do %&gt; My Link &lt;% end %&gt; Will be formatted to &lt;%= live_redirect ( to : &quot;/my/path&quot; , class : &quot;my class&quot; ) do %&gt; My Link &lt;% end %&gt; Note that only the text My Link has been formatted. Intentional new lines The formatter will keep intentional new lines. However, the formatter will always keep a maximum of one line break in case you have multiple ones: &lt;p&gt; text text &lt;/p&gt; Will be formatted to: &lt;p&gt; text text &lt;/p&gt; Inline elements We don't format inline elements when there is a text without whitespace before or after the element. Otherwise it would compromise what is rendered adding an extra whitespace. This is the list of inline elements: https://developer.mozilla.org/en-US/docs/Web/HTML/Inline_elements#list_of_inline_elements Skip formatting In case you don't want part of your HTML to be automatically formatted. You can use the special phx-no-format attribute so that the formatter will skip the element block. Note that this attribute will not be rendered. Therefore: &lt;.textarea phx-no-format&gt;My content&lt;/.textarea&gt; Will be kept as is your code editor, but rendered as: &lt; textarea &gt; My content &lt;/ textarea &gt; Comments Inline comments &lt;%# comment %&gt; are deprecated and the formatter will discard them silently from templates. You must change them to the multi-line comment &lt;%!-- comment --%&gt; on Elixir v1.14+ or introduce a space between &lt;% and # , such as &lt;% # comment %&gt; .","ref":"Phoenix.LiveView.HTMLFormatter.html","title":"Phoenix.LiveView.HTMLFormatter","type":"module"},{"doc":"Instrumenter to handle logging of Phoenix.LiveView and Phoenix.LiveComponent life-cycle events. Installation The logger is installed automatically when Live View starts. By default, the log level is set to :debug . Module configuration The log level can be overridden for an individual Live View module: use Phoenix.LiveView , log : :debug To disable logging for an individual Live View module: use Phoenix.LiveView , log : false Telemetry The following Phoenix.LiveView and Phoenix.LiveComponent events are logged: [:phoenix, :live_view, :mount, :start] [:phoenix, :live_view, :mount, :stop] [:phoenix, :live_view, :handle_params, :start] [:phoenix, :live_view, :handle_params, :stop] [:phoenix, :live_view, :handle_event, :start] [:phoenix, :live_view, :handle_event, :stop] [:phoenix, :live_component, :handle_event, :start] [:phoenix, :live_component, :handle_event, :stop] See the Telemetry guide for more information. Parameter filtering If enabled, Phoenix.LiveView.Logger will filter parameters based on the configuration of Phoenix.Logger .","ref":"Phoenix.LiveView.Logger.html","title":"Phoenix.LiveView.Logger","type":"module"},{"doc":"The LiveView socket for Phoenix Endpoints. This is typically mounted directly in your endpoint. socket &quot;/live&quot; , Phoenix.LiveView.Socket , websocket : [ connect_info : [ session : @session_options ] ] To share an underlying transport connection between regular Phoenix channels and LiveView processes, use Phoenix.LiveView.Socket from your own MyAppWeb.UserSocket module. Next, declare your channel definitions and optional connect/3 , and id/1 callbacks to handle your channel specific needs, then mount your own socket in your endpoint: socket &quot;/live&quot; , MyAppWeb.UserSocket , websocket : [ connect_info : [ session : @session_options ] ]","ref":"Phoenix.LiveView.Socket.html","title":"Phoenix.LiveView.Socket","type":"module"},{"doc":"The data in a LiveView as stored in the socket.","ref":"Phoenix.LiveView.Socket.html#t:assigns/0","title":"Phoenix.LiveView.Socket.assigns/0","type":"type"},{"doc":"Struct returned when assigns is not in the socket.","ref":"Phoenix.LiveView.Socket.html#t:assigns_not_in_socket/0","title":"Phoenix.LiveView.Socket.assigns_not_in_socket/0","type":"opaque"},{"doc":"","ref":"Phoenix.LiveView.Socket.html#t:fingerprints/0","title":"Phoenix.LiveView.Socket.fingerprints/0","type":"type"},{"doc":"","ref":"Phoenix.LiveView.Socket.html#t:t/0","title":"Phoenix.LiveView.Socket.t/0","type":"type"},{"doc":"The struct returned by Phoenix.LiveViewTest.element/3 . The following public fields represent the element: selector - The query selector text_filter - The text to further filter the element See the Phoenix.LiveViewTest documentation for usage.","ref":"Phoenix.LiveViewTest.Element.html","title":"Phoenix.LiveViewTest.Element","type":"module"},{"doc":"The struct returned by Phoenix.LiveViewTest.file_input/4 . The following public fields represent the element: selector - The query selector entries - The list of selected file entries See the Phoenix.LiveViewTest documentation for usage.","ref":"Phoenix.LiveViewTest.Upload.html","title":"Phoenix.LiveViewTest.Upload","type":"module"},{"doc":"The struct for testing LiveViews. The following public fields represent the LiveView: id - The DOM id of the LiveView module - The module of the running LiveView pid - The Pid of the running LiveView endpoint - The endpoint for the LiveView target - The target to scope events to See the Phoenix.LiveViewTest documentation for usage.","ref":"Phoenix.LiveViewTest.View.html","title":"Phoenix.LiveViewTest.View","type":"module"},{"doc":"The struct representing an upload.","ref":"Phoenix.LiveView.UploadConfig.html","title":"Phoenix.LiveView.UploadConfig","type":"module"},{"doc":"","ref":"Phoenix.LiveView.UploadConfig.html#t:t/0","title":"Phoenix.LiveView.UploadConfig.t/0","type":"type"},{"doc":"The struct representing an upload entry.","ref":"Phoenix.LiveView.UploadEntry.html","title":"Phoenix.LiveView.UploadEntry","type":"module"},{"doc":"","ref":"Phoenix.LiveView.UploadEntry.html#t:t/0","title":"Phoenix.LiveView.UploadEntry.t/0","type":"type"},{"doc":"The struct representing an internal unique reference to the component instance, available as the @myself assign in stateful components. Read more about the uses of @myself in the Phoenix.LiveComponent docs.","ref":"Phoenix.LiveComponent.CID.html","title":"Phoenix.LiveComponent.CID","type":"module"},{"doc":"The struct returned by components in .heex templates. This component is never meant to be output directly into the template. It should always be handled by the diffing algorithm.","ref":"Phoenix.LiveView.Component.html","title":"Phoenix.LiveView.Component","type":"module"},{"doc":"","ref":"Phoenix.LiveView.Component.html#t:t/0","title":"Phoenix.LiveView.Component.t/0","type":"type"},{"doc":"The struct returned by for-comprehensions in .heex templates. See a description about its fields and use cases in Phoenix.LiveView.Engine docs.","ref":"Phoenix.LiveView.Comprehension.html","title":"Phoenix.LiveView.Comprehension","type":"module"},{"doc":"","ref":"Phoenix.LiveView.Comprehension.html#t:t/0","title":"Phoenix.LiveView.Comprehension.t/0","type":"type"},{"doc":"An EEx template engine that tracks changes. This is often used by Phoenix.LiveView.HTMLEngine which also adds HTML validation. In the documentation below, we will explain how it works internally. For user-facing documentation, see Phoenix.LiveView . Phoenix.LiveView.Rendered Whenever you render a live template, it returns a Phoenix.LiveView.Rendered structure. This structure has three fields: :static , :dynamic and :fingerprint . The :static field is a list of literal strings. This allows the Elixir compiler to optimize this list and avoid allocating its strings on every render. The :dynamic field contains a function that takes a boolean argument (see &quot;Tracking changes&quot; below), and returns a list of dynamic content. Each element in the list is either one of: iodata - which is the dynamic content nil - the dynamic content did not change another Phoenix.LiveView.Rendered struct, see &quot;Nesting and fingerprinting&quot; below a Phoenix.LiveView.Comprehension struct, see &quot;Comprehensions&quot; below a Phoenix.LiveView.Component struct, see &quot;Component&quot; below When you render a live template, you can convert the rendered structure to iodata by alternating the static and dynamic fields, always starting with a static entry followed by a dynamic entry. The last entry will always be static too. So the following structure: % Phoenix.LiveView.Rendered { static : [ &quot;foo&quot; , &quot;bar&quot; , &quot;baz&quot; ] , dynamic : fn track_changes? -&gt; [ &quot;left&quot; , &quot;right&quot; ] end } Results in the following content to be sent over the wire as iodata: [ &quot;foo&quot; , &quot;left&quot; , &quot;bar&quot; , &quot;right&quot; , &quot;baz&quot; ] This is also what calling Phoenix.HTML.Safe.to_iodata/1 with a Phoenix.LiveView.Rendered structure returns. Of course, the benefit of live templates is exactly that you do not need to send both static and dynamic segments every time. So let's talk about tracking changes. Tracking changes By default, a live template does not track changes. Change tracking can be enabled by including a changed map in the assigns with the key __changed__ and passing true to the dynamic parts. The map should contain the name of any changed field as key and the boolean true as value. If a field is not listed in __changed__ , then it is always considered unchanged. If a field is unchanged and live believes a dynamic expression no longer needs to be computed, its value in the dynamic list will be nil . This information can be leveraged to avoid sending data to the client. Nesting and fingerprinting Phoenix.LiveView also tracks changes across live templates. Therefore, if your view has this: &lt; % = render &quot;form.html&quot; , assigns % &gt; Phoenix will be able to track what is static and dynamic across templates, as well as what changed. A rendered nested live template will appear in the dynamic list as another Phoenix.LiveView.Rendered structure, which must be handled recursively. However, because the rendering of live templates can be dynamic in itself, it is important to distinguish which live template was rendered. For example, imagine this code: &lt; % = if something? , do : render ( &quot;one.html&quot; , assigns ) , else : render ( &quot;other.html&quot; , assigns ) % &gt; To solve this, all Phoenix.LiveView.Rendered structs also contain a fingerprint field that uniquely identifies it. If the fingerprints are equal, you have the same template, and therefore it is possible to only transmit its changes. Comprehensions Another optimization done by live templates is to track comprehensions. If your code has this: &lt; % = for point &lt;- @points do % &gt; x : &lt; % = point . x % &gt; y : &lt; % = point . y % &gt; &lt; % end % &gt; Instead of rendering all points with both static and dynamic parts, it returns a Phoenix.LiveView.Comprehension struct with the static parts, that are shared across all points, and a list of dynamics to be interpolated inside the static parts. If @points is a list with %{x: 1, y: 2} and %{x: 3, y: 4} , the above expression would return: % Phoenix.LiveView.Comprehension { static : [ &quot; \\n x: &quot; , &quot; \\n y: &quot; , &quot; \\n &quot; ] , dynamics : [ [ &quot;1&quot; , &quot;2&quot; ] , [ &quot;3&quot; , &quot;4&quot; ] ] } This allows live templates to drastically optimize the data sent by comprehensions, as the static parts are emitted only once, regardless of the number of items. The list of dynamics is always a list of iodatas or components, as we don't perform change tracking inside the comprehensions themselves. Similarly, comprehensions do not have fingerprints because they are only optimized at the root, so conditional evaluation, as the one seen in rendering, is not possible. The only possible outcome for a dynamic field that returns a comprehension is nil . Components Live also supports stateful components defined with Phoenix.LiveComponent . Since they are stateful, they are always handled lazily by the diff algorithm.","ref":"Phoenix.LiveView.Engine.html","title":"Phoenix.LiveView.Engine","type":"module"},{"doc":"The HTMLEngine that powers .heex templates and the ~H sigil. It works by adding a HTML parsing and validation layer on top of EEx engine. By default it uses Phoenix.LiveView.Engine as its &quot;subengine&quot;.","ref":"Phoenix.LiveView.HTMLEngine.html","title":"Phoenix.LiveView.HTMLEngine","type":"module"},{"doc":"Renders a component defined by the given function. This function is rarely invoked directly by users. Instead, it is used by ~H to render Phoenix.Component s. For example, the following: &lt; MyApp.Weather . city name = &quot;Kraków&quot; / &gt; Is the same as: &lt; % = component ( &amp; MyApp.Weather . city / 1 , [ name : &quot;Kraków&quot; ] , { __ENV__ . module , __ENV__ . function , __ENV__ . file , __ENV__ . line } ) % &gt;","ref":"Phoenix.LiveView.HTMLEngine.html#component/3","title":"Phoenix.LiveView.HTMLEngine.component/3","type":"function"},{"doc":"Define a inner block, generally used by slots. This macro is mostly used by HTML engines that provide a slot implementation and rarely called directly. The name must be the assign name the slot/block will be stored under. If you're using HEEx templates, you should use its higher level &lt;:slot&gt; notation instead. See Phoenix.Component for more information.","ref":"Phoenix.LiveView.HTMLEngine.html#inner_block/2","title":"Phoenix.LiveView.HTMLEngine.inner_block/2","type":"macro"},{"doc":"The struct returned by .heex templates. See a description about its fields and use cases in Phoenix.LiveView.Engine docs.","ref":"Phoenix.LiveView.Rendered.html","title":"Phoenix.LiveView.Rendered","type":"module"},{"doc":"","ref":"Phoenix.LiveView.Rendered.html#t:t/0","title":"Phoenix.LiveView.Rendered.t/0","type":"type"},{"doc":"","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"Bug Fixes Fix LiveViewTest failing to find main live view","ref":"changelog.html#0-18-14-2023-02-14","title":"Changelog - 0.18.14 (2023-02-14)","type":"extras"},{"doc":"Enhancements Improve error message when failing to use Phoenix.Component","ref":"changelog.html#0-18-13-2023-02-10","title":"Changelog - 0.18.13 (2023-02-10)","type":"extras"},{"doc":"Enhancements Introduce streams for efficiently handling large collections Allow replies from :handle_event lifecycle hooks Add &lt;.inputs_for&gt; component to Phoenix.Component Support replies on lifecycle :handle_event hooks Bug Fixes Fix change tracking when re-assigning a defaulted attribute to same default value Fix upload drag and drop failing to worka after using file select dialog Fix form recovery when form's first input is phx-change","ref":"changelog.html#0-18-12-2023-02-10","title":"Changelog - 0.18.12 (2023-02-10)","type":"extras"},{"doc":"Bug Fixes Fix socket unloading connection for forms that have defaulted prevented","ref":"changelog.html#0-18-11-2023-01-19","title":"Changelog - 0.18.11 (2023-01-19)","type":"extras"},{"doc":"Bug Fixes Fix svg tags with href incorrectly unloading socket on click Fix form submits with target=&quot;_blank&quot; incorrectly unloading socket on submit","ref":"changelog.html#0-18-10-2023-01-18","title":"Changelog - 0.18.10 (2023-01-18)","type":"extras"},{"doc":"Bug Fixes Fix regular form submits failing to be dispatched","ref":"changelog.html#0-18-9-2023-01-17","title":"Changelog - 0.18.9 (2023-01-17)","type":"extras"},{"doc":"Enhancements Restore scroll position on back when previous navigation was live patch Bug Fixes Fix live layout not being applied until connected render","ref":"changelog.html#0-18-8-2023-01-16","title":"Changelog - 0.18.8 (2023-01-16)","type":"extras"},{"doc":"Bug Fixes Fix live layout not being applied when passed to :live_session during disconnect render Fix external anchor clicks and links with hashes incorrectly unloading socket","ref":"changelog.html#0-18-7-2023-01-13","title":"Changelog - 0.18.7 (2023-01-13)","type":"extras"},{"doc":"Bug Fixes Fix external anchor click unloading on external click","ref":"changelog.html#0-18-6-2023-01-09","title":"Changelog - 0.18.6 (2023-01-09)","type":"extras"},{"doc":"Bug Fixes Fix external anchor click unloading socket","ref":"changelog.html#0-18-5-2023-01-09","title":"Changelog - 0.18.5 (2023-01-09)","type":"extras"},{"doc":"Enhancements Support string upload name to support dynamically generated allow_upload 's Bug Fixes Fix nested LiveView race condition on live patch causing nested child to skip updates in some cases Fix browser history showing incorrect title when using live navigation with @page_title Fix undefined _target param when using JS.push for form changes Fix phx-no-feedback missing from inputs added after a form submit Fix phx-disconnected events firing when navigating away or submitting external forms","ref":"changelog.html#0-18-4-2023-01-05","title":"Changelog - 0.18.4 (2023-01-05)","type":"extras"},{"doc":"Enhancements Add embed_templates to Phoenix.Component for embedding template files as function components Raise on global slot attributes Bug Fixes Fix bug on slots when passing multiple slot entries with mix if/for syntax","ref":"changelog.html#0-18-3-2022-10-26","title":"Changelog - 0.18.3 (2022-10-26)","type":"extras"},{"doc":"Bug Fixes Fix match error when defining :values before :default Allow tuples in external redirects Fix race condition on dispatching click away when enter is pressed Fix formatter breaking inline blocks when surrounded by text without whitespace Enhancements Add intersperse component for rendering a separator between an enumerable","ref":"changelog.html#0-18-2-2022-10-04","title":"Changelog - 0.18.2 (2022-10-04)","type":"extras"},{"doc":"Bug Fixes Fix phx-loading class being applied to dead views Fix &lt;.live_img_preview /&gt; causing invalid attribute errors on uploads Do not fire phx events when element is disabled Enhancements Support :include option to extend global attributes on a case-by-case basis Warn when accessing a variable binding defined outside of ~H","ref":"changelog.html#0-18-1-2022-09-28","title":"Changelog - 0.18.1 (2022-09-28)","type":"extras"},{"doc":"LiveView v0.18 includes a major new feature in the form of declarative assigns with new attr and slot APIs for specifying which attributes a function component supports, the type, and default values. Attributes and slots are compile-time verified and emit warnings (requires Elixir v1.14.0+). v0.18 includes a number of new function components which replace their EEx expression counterparts &lt;%= ... %&gt; . For example, live_redirect , live_patch , and Phoenix.HTML's link have been replaced by a unified Phoenix.Component.link/1 function component: &lt; . link href = &quot;https://myapp.com&quot; &gt; my app &lt; / . link &gt; &lt; . link navigate = { @path } &gt; remount &lt; / . link &gt; &lt; . link patch = { @path } &gt; patch &lt; / . link &gt; Those new components live in the Phoenix.Component module. Phoenix.LiveView.Helpers itself has been soft deprecated and all relevant functionality has been migrated. You must import Phoenix.Component where you previously imported Phoenix.LiveView.Helpers when upgrading. You may also need to import Phoenix.Component where you also imported Phoenix.LiveView and some of its functions have been moved to Phoenix.Component . Additionally, the special let attribute on function components have been deprecated by a :let usage. Deprecations live_redirect - deprecate in favor of new &lt;.link navigate={..}&gt; component of Phoenix.Component live_patch - deprecate in favor of new &lt;.link patch={..}&gt; component of Phoenix.Component push_redirect - deprecate in favor of new push_navigate function on Phoenix.LiveView Enhancements [Component] Add declarative assigns with compile-time verifications and warnings via attr / slot [Component] Add new attrs :let and :for , and :if with HTML tag, function component, and slot support. We still support let but the formatter will convert it to :let and soon it will be deprecated. [Component] Add dynamic_tag function component [Component] Add link function component [Component] Add focus_wrap function component to wrap focus around content like modals and dialogs for accessibility [Logger] Add new LiveView logger with telemetry instrumentation for lifecycle events [JS] Add new JS commands for focus , focus_first , push_focus , and pop_focus for accessibility [Socket] Support sharing Phoenix.LiveView.Socket with regular channels via use Phoenix.LiveView.Socket Add _live_referer connect param for handling push_navigate referal URL Add new phx-connected and phx-disconnected bindings for reacting to lifecycle changes Add dead view support for JS commands Add dead view support for hooks Bug fixes Fix external upload issue where listeners are not cleaned up when an external failure happens on the client Do not debounce phx-blur","ref":"changelog.html#0-18-0-2022-09-20","title":"Changelog - 0.18.0 (2022-09-20)","type":"extras"},{"doc":"Enhancements Add support for upcoming Phoenix 1.7 flash interface","ref":"changelog.html#0-17-12-2022-09-20","title":"Changelog - 0.17.12 (2022-09-20)","type":"extras"},{"doc":"Enhancements Add replaceTransport to LiveSocket Bug fixes Cancel debounced events from firing after a live navigation event Fix hash anchor failing to scroll to anchor element on live navigation Do not debounce phx-blur events","ref":"changelog.html#0-17-11-2022-07-11","title":"Changelog - 0.17.11 (2022-07-11)","type":"extras"},{"doc":"Bug fixes [Formatter] Preserve single quote delimiter on attrs [Formatter] Do not format inline elements surrounded by texts without whitespaces [Formatter] Keep text and eex along when there isn't a whitespace [Formatter] Fix intentional line breaks after eex expressions [Formatter] Handle self close tags as inline [Formatter] Do not format inline elements without whitespaces among them [Formatter] Do not format when attr contenteditable is present Enhancements [Formatter] Introduce special attr phx-no-format to skip formatting","ref":"changelog.html#0-17-10-2022-05-25","title":"Changelog - 0.17.10 (2022-05-25)","type":"extras"},{"doc":"Bug fixes Fix sticky LiveViews failing to be patched during live navigation Do not raise on dynamic phx-update value","ref":"changelog.html#0-17-9-2022-04-07","title":"Changelog - 0.17.9 (2022-04-07)","type":"extras"},{"doc":"Enhancements Add HEEx formatter Support phx-change on individual inputs Dispatch MouseEvent on client Add :bubbles option to JS.dispatch to control event bubbling Expose underlying liveSocket instance on hooks Enable client debug by default on localhost Bug fixes Fix hook and sticky LiveView issues caused by back-to-back live redirects from mount Fix hook destroyed callback failing to be invoked for children of phx-remove in some cases Do not failsafe reload the page on push timeout if disconnected Do not bubble navigation click events to regular phx-click's No longer generate csrf_token for forms without action, reducing the payload during phx-change/phx-submit events","ref":"changelog.html#0-17-8-2022-04-06","title":"Changelog - 0.17.8 (2022-04-06)","type":"extras"},{"doc":"Enhancements Optimize nested for comprehension diffs Bug fixes Fix error when live_redirect links are clicked when not connected in certain cases","ref":"changelog.html#0-17-7-2022-02-07","title":"Changelog - 0.17.7 (2022-02-07)","type":"extras"},{"doc":"Enhancements Add JS.set_attribute and JS.remove_attribute Add sticky: true option to live_render to maintain a nested child on across live redirects Dispatch phx:show-start , phx:show-end , phx:hide-start and phx:hide-end on JS.show|hide|toggle Add get_connect_info/2 that also works on disconnected render Add LiveSocket constructor options for configuration failsafe behavior via new maxReloads , reloadJitterMin , reloadJitterMax , failsafeJitter options Bug fixes Show form errors after submit even when no changes occur on server Fix phx-disable-with failing to disable elements outside of forms Fix phx ref tracking leaving elements in awaiting state when targeting an external LiveView Fix diff on response failing to await for active transitions in certain cases Fix phx-click-away not respecting phx-target Fix &quot;disconnect&quot; broadcast failing to failsafe refresh the page Fix JS.push with :target failing to send to correct component in certain cases Deprecations Deprecate Phoenix.LiveView.get_connect_info/1 in favor of get_connect_info/2 Deprecate Phoenix.LiveViewTest.put_connect_info/2 in favor of calling the relevant functions in Plug.Conn Deprecate returning &quot;raw&quot; values from upload callbacks on Phoenix.LiveView.consume_uploaded_entry/3 and Phoenix.LiveView.consume_uploaded_entries/3 . The callback must return either {:ok, value} or {:postpone, value} . Returning any other value will emit a warning.","ref":"changelog.html#0-17-6-2022-01-18","title":"Changelog - 0.17.6 (2022-01-18)","type":"extras"},{"doc":"Bug fixes Do not trigger phx-click-away if element is not visible Fix phx-remove failing to tear down nested live children","ref":"changelog.html#0-17-5-2021-11-02","title":"Changelog - 0.17.5 (2021-11-02)","type":"extras"},{"doc":"Bug fixes Fix variable scoping issues causing various content block or duplication rendering bugs","ref":"changelog.html#0-17-4-2021-11-01","title":"Changelog - 0.17.4 (2021-11-01)","type":"extras"},{"doc":"Enhancements Support 3-tuple for JS class transitions to support staged animations where a transition class is applied with a starting and ending class Allow JS commands to be executed on DOM nodes outside of the LiveView container Optimization Avoid duplicate statics inside comprehension. In previous versions, comprehensions were able to avoid duplication only in the content of their root. Now we recursively traverse all comprehension nodes and send the static only once for the whole comprehension. This should massively reduce the cost of sending comprehensions over the wire Bug fixes Fix HTML engine bug causing expressions to be duplicated or not rendered correctly Fix HTML engine bug causing slots to not be re-rendered when they should have Fix form recovery being sent to wrong target","ref":"changelog.html#0-17-3-2021-10-28","title":"Changelog - 0.17.3 (2021-10-28)","type":"extras"},{"doc":"Bug fixes Fix HTML engine bug causing attribute expressions to be incorrectly evaluated in certain cases Fix show/hide/toggle custom display not being restored Fix default to target for JS.show|hide|dispatch Fix form input targeting","ref":"changelog.html#0-17-2-2021-10-22","title":"Changelog - 0.17.2 (2021-10-22)","type":"extras"},{"doc":"Bug fixes Fix SVG element support for phx binding interactions","ref":"changelog.html#0-17-1-2021-10-21","title":"Changelog - 0.17.1 (2021-10-21)","type":"extras"},{"doc":"Breaking Changes on_mount changes The hook API introduced in LiveView 0.16 has been improved based on feedback. LiveView 0.17 removes the custom module-function callbacks for the Phoenix.LiveView.on_mount/1 macro and the :on_mount option for Phoenix.LiveView.Router.live_session/3 in favor of supporting a custom argument. For clarity, the module function to be invoked during the mount lifecycle stage will always be named on_mount/4 . For example, if you had invoked on_mount/1 like so: on_mount MyAppWeb.MyHook on_mount { MyAppWeb.MyHook , :assign_current_user } and defined your callbacks as: # my_hook.ex def mount ( _params , _session , _socket ) do end def assign_current_user ( _params , _session , _socket ) do end Change the callback to: # my_hook.ex def on_mount ( :default , _params , _session , _socket ) do end def on_mount ( :assign_current_user , _params , _session , _socket ) do end When given only a module name, the first argument to on_mount/4 will be the atom :default . LEEx templates in stateful LiveComponents Stateful LiveComponents (where an :id is given) must now return HEEx templates ( ~H sigil or .heex extension). LEEx templates ( ~L sigil or .leex extension) are no longer supported. This addresses bugs and allows stateful components to be rendered more efficiently client-side. phx-disconnected class has been replaced with phx-loading Due to a bug in the newly released Safari 15, the previously used .phx-disconnected class has been replaced by a new .phx-loading class. The reason for the change is phx.new included a .phx-disconnected rule in the generated app.css which triggers the Safari bug. Renaming the class avoids applying the erroneous rule for existing applications. Folks can upgrade by simply renaming their .phx-disconnected rules to .phx-loading . phx-capture-click has been deprecated in favor of phx-click-away The new phx-click-away binding replaces phx-capture-click and is much more versatile because it can detect &quot;click focus&quot; being lost on containers. Removal of previously deprecated functionality Some functionality that was previously deprecated has been removed: Implicit assigns in live_component do-blocks is no longer supported Passing a @socket to live_component will now raise if possible Enhancements Allow slots in function components: they are marked as &lt;:slot_name&gt; and can be rendered with &lt;%= render_slot @slot_name %&gt; Add JS command for executing JavaScript utility operations on the client with an extended push API Optimize string attributes: If the attribute is a string interpolation, such as &lt;div class={&quot;foo bar \#{@baz}&quot;}&gt; , only the interpolation part is marked as dynamic If the attribute can be empty, such as &quot;class&quot; and &quot;style&quot;, keep the attribute name as static Add a function component for rendering Phoenix.LiveComponent . Instead of &lt;%= live_component FormComponent, id: &quot;form&quot; %&gt; , you must now do: &lt;.live_component module={FormComponent} id=&quot;form&quot; /&gt; Bug fixes Fix LiveViews with form recovery failing to properly mount following a reconnect when preceded by a live redirect Fix stale session causing full redirect fallback when issuing a push_redirect from mount Add workaround for Safari bug causing &lt;img&gt; tags with srcset and video with autoplay to fail to render Support EEx interpolation inside HTML comments in HEEx templates Support HTML tags inside script tags (as in regular HTML) Raise if using quotes in attribute names Include the filename in error messages when it is not possible to parse interpolated attributes Make sure the test client always sends the full URL on live_patch / live_redirect . This mirrors the behaviour of the JavaScript client Do not reload flash from session on live_redirect s Fix select drop-down flashes in Chrome when the DOM is patched during focus Deprecations &lt;%= live_component MyModule, id: @user.id, user: @user %&gt; is deprecated in favor of &lt;.live_component module={MyModule} id={@user.id} user={@user} /&gt; . Notice the new API requires using HEEx templates. This change allows us to further improve LiveComponent and bring new features such as slots to them. render_block/2 in deprecated in favor of render_slot/2","ref":"changelog.html#0-17-0-2021-10-21","title":"Changelog - 0.17.0 (2021-10-21)","type":"extras"},{"doc":"Enhancements Improve HEEx error messages Relax HTML tag validation to support mixed case tags Support self closing HTML tags Remove requirement for handle_params to be defined for lifecycle hooks Bug fixes Fix pushes failing to include channel join_ref on messages","ref":"changelog.html#0-16-4-2021-09-22","title":"Changelog - 0.16.4 (2021-09-22)","type":"extras"},{"doc":"Bug fixes Fix on_mount hooks calling view mount before redirecting when the hook issues a halt redirect.","ref":"changelog.html#0-16-3-2021-09-03","title":"Changelog - 0.16.3 (2021-09-03)","type":"extras"},{"doc":"Enhancements Improve error messages on tokenization Improve error message if @inner_block is missing Bug fixes Fix phx-change form recovery event being sent to wrong component on reconnect when component order changes","ref":"changelog.html#0-16-2-2021-09-03","title":"Changelog - 0.16.2 (2021-09-03)","type":"extras"},{"doc":"Enhancements Relax phoenix_html dependency requirement Allow testing functional components by passing a function reference to Phoenix.LiveViewTest.render_component/3 Bug fixes Do not generate CSRF tokens for non-POST forms Do not add compile-time dependencies on on_mount declarations","ref":"changelog.html#0-16-1-2021-08-26","title":"Changelog - 0.16.1 (2021-08-26)","type":"extras"},{"doc":"Security Considerations Upgrading from 0.15 LiveView v0.16 optimizes live redirects by supporting navigation purely over the existing WebSocket connection. This is accomplished by the new live_session/3 feature of Phoenix.LiveView.Router . The security guide has always stressed the following: ... As we have seen, LiveView begins its life-cycle as a regular HTTP request. Then a stateful connection is established. Both the HTTP request and the stateful connection receives the client data via parameters and session. This means that any session validation must happen both in the HTTP request (plug pipeline) and the stateful connection (LiveView mount) ... These guidelines continue to be valid, but it is now essential that the stateful connection enforces authentication and session validation within the LiveView mount lifecycle because a live_redirect from the client will not go through the plug pipeline as a hard-refresh or initial HTTP render would. This means authentication, authorization, etc that may be done in the Plug.Conn pipeline must also be performed within the LiveView mount lifecycle. Live sessions allow you to support a shared security model by allowing live_redirect s to only be issued between routes defined under the same live session name. If a client attempts to live redirect to a different live session, it will be refused and a graceful client-side redirect will trigger a regular HTTP request to the attempted URL. See the Phoenix.LiveView.Router.live_session/3 docs for more information and example usage. New HTML Engine LiveView v0.16 introduces HEEx (HTML + EEx) templates and the concept of function components via Phoenix.Component . The new HEEx templates validate the markup in the template while also providing smarter change tracking as well as syntax conveniences to make it easier to build composable components. A function component is any function that receives a map of assigns and returns a ~H template: defmodule MyComponent do use Phoenix.Component def btn ( assigns ) do ~H&quot;&quot;&quot; &lt;button class=&quot;btn&quot;&gt;&lt;%= @text %&gt;&lt;/button&gt; &quot;&quot;&quot; end end This component can now be used as in your HEEx templates as: &lt; MyComponent . btn text = &quot;Save&quot; &gt; The introduction of HEEx and function components brings a series of deprecation warnings, some introduced in this release and others which will be added in the future. Note HEEx templates require Elixir v1.12+. Upgrading and deprecations The main deprecation in this release is that the ~L sigil and the .leex extension are now soft-deprecated. The docs have been updated to discourage them and using them will emit warnings in future releases. We recommend using the ~H sigil and the .heex extension for all future templates in your application. You should also plan to migrate the old templates accordingly using the recommendations below. Migrating from LEEx to HEEx is relatively straightforward. There are two main differences. First of all, HEEx does not allow interpolation inside tags. So instead of: &lt; div id = &quot;&lt;%= @id %&gt;&quot; &gt; ... &lt; / div &gt; One should use the HEEx syntax: &lt; div id = { @id } &gt; ... &lt; / div &gt; The other difference is in regards to form_for . Some templates may do the following: ~L&quot;&quot;&quot; &lt;%= f = form_for @changeset, &quot;#&quot; %&gt; &lt;%= input f, :foo %&gt; &lt;/form&gt; &quot;&quot;&quot; However, when converted to ~H , it is not valid HTML: there is a &lt;/form&gt; tag but its opening is hidden inside the Elixir code. On LiveView v0.16, there is a function component named form : ~H&quot;&quot;&quot; &lt;.form :let={f} for={@changeset}&gt; &lt;%= input f, :foo %&gt; &lt;/.form&gt; &quot;&quot;&quot; We understand migrating all templates from ~L to ~H can be a daunting task. Therefore we plan to support ~L in LiveViews for a long time. However, we can't do the same for stateful LiveComponents, as some important client-side features and optimizations will depend on the ~H sigil. Therefore our recommendation is to replace ~L by ~H first in live components , particularly stateful live components. Furthermore, stateless live_component (i.e. live components without an :id ) will be deprecated in favor of the new function components. Our plan is to support them for a reasonable period of time, but you should avoid creating new ones in your application. Breaking Changes LiveView 0.16 removes the :layout and :container options from Phoenix.LiveView.Routing.live/4 in favor of the :root_layout and :container options on Phoenix.Router.live_session/3 . For instance, if you have the following in LiveView 0.15 and prior: live &quot;/path&quot; , MyAppWeb.PageLive , layout : { MyAppWeb.LayoutView , &quot;custom_layout.html&quot; } Change it to: live_session :session_name , root_layout : { MyAppWeb.LayoutView , &quot;custom_layout.html&quot; } do live &quot;/path&quot; , MyAppWeb.PageLive end On the client, the phoenix_live_view package no longer provides a default export for LiveSocket . If you have the following in your JavaScript entrypoint (typically located at assets/js/app.js ): import LiveSocket from &quot;phoenix_live_view&quot; Change it to: import { LiveSocket } from &quot;phoenix_live_view&quot; Additionally on the client, the root LiveView element no longer exposes the LiveView module name, therefore the phx-view attribute is never set. Similarly, the viewName property of client hooks has been removed. Codebases calling a custom function component/3 should rename it or specify its module to avoid a conflict, as LiveView introduces a macro with that name and it is special cased by the underlying engine. Enhancements Introduce HEEx templates Introduce Phoenix.Component Introduce Phoenix.Router.live_session/3 for optimized live redirects Introduce on_mount and attach_hook hooks which provide a mechanism to tap into key stages of the LiveView lifecycle Add upload methods to client-side hooks [Helpers] Optimize live_img_preview rendering [Helpers] Introduce form function component which wraps Phoenix.HTML.form_for [LiveViewTest] Add with_target for scoping components directly [LiveViewTest] Add refute_redirected [LiveViewTest] Support multiple phx-target values to mirror JS client [LiveViewTest] Add follow_trigger_action [JavaScript Client] Add sessionStorage option LiveSocket constructor to support client storage overrides [JavaScript Client] Do not failsafe reload the page in the background when a tab is unable to connect if the page is not visible Bug fixes Make sure components are loaded on render_component to ensure all relevant callbacks are invoked Fix Phoenix.LiveViewTest.page_title returning nil in some cases Fix buttons being re-enabled when explicitly set to disabled on server Fix live patch failing to update URL when live patch link is patched again via handle_params within the same callback lifecycle Fix phx-no-feedback class not applied when page is live-patched Fix DOMException, querySelector, not a valid selector when performing DOM lookups on non-standard IDs Fix select dropdown flashing close/opened when assigns are updated on Chrome/macOS Fix error with multiple live_file_input in one form Fix race condition in showError causing null querySelector Fix statics not resolving correctly across recursive diffs Fix no function clause matching in Phoenix.LiveView.Diff.many_to_iodata Fix upload input not being cleared after files are uploaded via a component Fix channel crash when uploading during reconnect Fix duplicate progress events being sent for large uploads Deprecations Implicit assigns when passing a do-end block to live_component is deprecated The ~L sigil and the .leex extension are now soft-deprecated in favor of ~H and .heex Stateless live components (a live_component call without an :id ) are deprecated in favor of the new function component feature","ref":"changelog.html#0-16-0-2021-08-10","title":"Changelog - 0.16.0 (2021-08-10)","type":"extras"},{"doc":"Bug fixes Fix broken webpack build throwing missing morphdom dependency","ref":"changelog.html#0-15-7-2021-05-24","title":"Changelog - 0.15.7 (2021-05-24)","type":"extras"},{"doc":"Bug fixes Fix live patch failing to update URL when live patch link is patched again from handle_params Fix regression in LiveViewTest.render_upload/3 when using channel uploads and progress callback Fix component uploads not being cleaned up on remove Fix KeyError on LiveView reconnect when an active upload was previously in progress Enhancements Support function components via component/3 Optimize progress events to send less messages for larger file sizes Allow session and local storage client overrides Deprecations Deprecate @socket/socket argument on live_component/3 call","ref":"changelog.html#0-15-6-2021-05-24","title":"Changelog - 0.15.6 (2021-05-24)","type":"extras"},{"doc":"Enhancements Add upload_errors/1 for returning top-level upload errors Bug fixes Fix consume_uploaded_entry/3 with external uploads causing inconsistent entries state Fix push_event losing events when a single diff produces multiple events from different components Fix deep merging of component tree sharing","ref":"changelog.html#0-15-5-2021-04-20","title":"Changelog - 0.15.5 (2021-04-20)","type":"extras"},{"doc":"Bug fixes Fix nested live_render 's causing remound of child LiveView even when ID does not change Do not attempt push hook events unless connected Fix preflighted refs causing auto_upload: true to fail to submit form Replace single upload entry when max_entries is 1 instead of accumulating multiple file selections Fix static_path in open_browser failing to load stylesheets","ref":"changelog.html#0-15-4-2021-01-26","title":"Changelog - 0.15.4 (2021-01-26)","type":"extras"},{"doc":"Bug fixes Fix push_redirect back causing timeout on the client","ref":"changelog.html#0-15-3-2021-01-02","title":"Changelog - 0.15.3 (2021-01-02)","type":"extras"},{"doc":"Backwards incompatible changes Remove beforeDestroy from phx-hook callbacks Bug fixes Fix form recovery failing to send input on first connection failure Fix hooks not getting remounted after LiveView reconnect Fix hooks reconnected callback being fired with no prior disconnect","ref":"changelog.html#0-15-2-2021-01-01","title":"Changelog - 0.15.2 (2021-01-01)","type":"extras"},{"doc":"Enhancements Ensure all click events bubble for mobile Safari Run consume_uploaded_entries in LiveView caller process Bug fixes Fix hooks not getting remounted after LiveView recovery Fix bug causing reload with jitter on timeout from previously closed channel Fix component child nodes being lost when component patch goes from single root node to multiple child siblings Fix phx-capture-click triggering on mouseup during text selection Fix LiveView push_event 's not clearing up in components Fix &lt;textarea&gt; being patched by LiveView while focused","ref":"changelog.html#0-15-1-2020-12-20","title":"Changelog - 0.15.1 (2020-12-20)","type":"extras"},{"doc":"Enhancements Add live uploads support for file progress, interactive file selection, and direct to cloud support Implement Phoenix.LiveViewTest.open_browser/2 that opens up a browser with the LiveView page Backwards incompatible changes Remove @inner_content in components and introduce render_block for rendering component @inner_block Remove @live_module in socket templates in favor of @socket.view Bug fixes Make sure URLs are decoded after they are split Do not recover forms without inputs Fix race condition when components are removed and then immediately re-added before the client can notify their CIDs have been destroyed Do not render LiveView if only events/replies have been added to the socket Properly merge different components when sharing component subtrees on initial render Allow variables inside do-blocks to be tainted Fix push_redirect from mount hanging on the client and causing a fallback to full page reload when following a clicked live_redirect on the client","ref":"changelog.html#0-15-0-2020-11-20","title":"Changelog - 0.15.0 (2020-11-20)","type":"extras"},{"doc":"Bug fixes Fix compatibility with latest Plug","ref":"changelog.html#0-14-8-2020-10-30","title":"Changelog - 0.14.8 (2020-10-30)","type":"extras"},{"doc":"Bug fixes Fix redirect(socket, external: ...) when returned from an event Properly follow location hashes on live patch/redirect Fix failure in Phoenix.LiveViewTest when phx-update has non-HTML nodes as children Fix phx_trigger_action submitting the form before the DOM updates are complete","ref":"changelog.html#0-14-7-2020-09-25","title":"Changelog - 0.14.7 (2020-09-25)","type":"extras"},{"doc":"Bug fixes Fix race condition on phx-trigger-action causing reconnects before server form submit","ref":"changelog.html#0-14-6-2020-09-21","title":"Changelog - 0.14.6 (2020-09-21)","type":"extras"},{"doc":"Enhancements Optimize DOM prepend and append operations Add Phoenix.LiveView.send_update_after/3 Bug fixes Fix scroll position when using back/forward with live_redirect 's Handle recursive components when generating diffs Support hard redirects on mount Properly track nested components on deletion on Phoenix.LiveViewTest","ref":"changelog.html#0-14-5-2020-09-20","title":"Changelog - 0.14.5 (2020-09-20)","type":"extras"},{"doc":"Bug fixes Fix hidden inputs throwing selection range error","ref":"changelog.html#0-14-4-2020-07-30","title":"Changelog - 0.14.4 (2020-07-30)","type":"extras"},{"doc":"Enhancements Support render_layout with LiveEEx Bug fixes Fix focused inputs being overwritten by latent patch Fix LiveView error when &quot;_target&quot; input name contains array Fix change tracking when passing a do-block to components","ref":"changelog.html#0-14-3-2020-07-24","title":"Changelog - 0.14.3 (2020-07-24)","type":"extras"},{"doc":"Bug fixes Fix Map of assigns together with @inner_content causing no function clause matching in Keyword.put/3 error Fix LiveViewTest failing to patch children properly for append/prepend based phx-update's Fix argument error when providing :as option to a live route Fix page becoming unresponsive when the server crashes while handling a live patch Fix empty diff causing pending data-ref based updates, such as classes and phx-disable-with content to not be updated Fix bug where throttling keydown events would eat key presses Fix &lt;textarea&gt; 's failing to be disabled on form submit Fix text node DOM memory leak when using phx-update append/prepend Enhancements Allow :router to be given to render_component Display file on compile warning for ~L Log error on client when using a hook without a DOM ID Optimize phx-update append/prepend based DOM updates","ref":"changelog.html#0-14-2-2020-07-21","title":"Changelog - 0.14.2 (2020-07-21)","type":"extras"},{"doc":"Bug fixes Fix nested live_render 's failing to be torn down when removed from the DOM in certain cases Fix LEEx issue for nested conditions failing to be re-evaluated","ref":"changelog.html#0-14-1-2020-07-09","title":"Changelog - 0.14.1 (2020-07-09)","type":"extras"},{"doc":"Bug fixes Fix IE11 issue where document.activeElement creates a null reference Fix setup and teardown of root views when explicitly calling liveSocket.disconnect() followed by liveSocket.connect() Fix error_tag failing to be displayed for non-text based inputs such as selects and checkboxes as the phx-no-feedback class was always applied Fix phx-error class being applied on live_redirect Properly handle Elixir's special variables, such as __MODULE__ No longer set disconnected class during patch Track flash keys to fix back-to-back flashes from being discarded Properly handle empty component diffs in the client for cases where the component has already been removed on the server Make sure components in nested live views do not conflict Fix phx-static not being sent from the client for child views Do not fail when trying to delete a view that was already deleted Ensure beforeDestroy is called on hooks in children of a removed element Enhancements Allow the whole component static subtree to be shared when the component already exists on the client Add telemetry events to mount , handle_params , and handle_event Add push_event for pushing events and data from the server to the client Add client handleEvent hook method for receiving events pushed from the server Add ability to receive a reply to a pushEvent from the server via {:reply, map, socket} Use event listener for popstate to avoid conflicting with user-defined popstate handlers Log error on client when rendering a component with no direct DOM children Make assigns.myself a struct to catch mistakes Log if component doesn't exist on send_update , raise if module is unavailable","ref":"changelog.html#0-14-0-2020-07-07","title":"Changelog - 0.14.0 (2020-07-07)","type":"extras"},{"doc":"Bug fixes Fix duplicate debounced events from being triggered on blur with timed debounce Fix client error when live_redirect ed route results in a redirect to a non-live route on the server Fix DOM siblings being removed when a rootless component is updated Fix debounced input failing to send last change when blurred via Tab, Meta, or other non-printable keys Enhancements Add dom option to LiveSocket with onBeforeElUpdated callback for external client library support of broad DOM operations","ref":"changelog.html#0-13-3-2020-06-04","title":"Changelog - 0.13.3 (2020-06-04)","type":"extras"},{"doc":"Bug fixes Fix a bug where swapping a root template with components would cause the LiveView to crash","ref":"changelog.html#0-13-2-2020-05-27","title":"Changelog - 0.13.2 (2020-05-27)","type":"extras"},{"doc":"Bug fixes Fix forced page refresh when push_redirect from a live_redirect Enhancements Optimize component diffs to avoid sending empty diffs Optimize components to share static values [LiveViewTest] Automatically synchronize before render events","ref":"changelog.html#0-13-1-2020-05-26","title":"Changelog - 0.13.1 (2020-05-26)","type":"extras"},{"doc":"Backwards incompatible changes No longer send event metadata by default. Metadata is now opt-in and user defined at the LiveSocket level. To maintain backwards compatibility with pre-0.13 behaviour, you can provide the following metadata option: let liveSocket = new LiveSocket ( &quot;/live&quot; , Socket , { params : { _csrf_token : csrfToken } , metadata : { click : ( e , el ) =&gt; { return { altKey : e . altKey , shiftKey : e . shiftKey , ctrlKey : e . ctrlKey , metaKey : e . metaKey , x : e . x || e . clientX , y : e . y || e . clientY , pageX : e . pageX , pageY : e . pageY , screenX : e . screenX , screenY : e . screenY , offsetX : e . offsetX , offsetY : e . offsetY , detail : e . detail || 1 , } } , keydown : ( e , el ) =&gt; { return { altGraphKey : e . altGraphKey , altKey : e . altKey , code : e . code , ctrlKey : e . ctrlKey , key : e . key , keyIdentifier : e . keyIdentifier , keyLocation : e . keyLocation , location : e . location , metaKey : e . metaKey , repeat : e . repeat , shiftKey : e . shiftKey } } } } ) Bug fixes Fix error caused by Chrome sending a keydown event on native UI autocomplete without a key Fix server error when a live navigation request issues a redirect Fix double window bindings when explicit calls to LiveSocket connect/disconnect/connect Enhancements Add Phoenix.LiveView.get_connect_info/1 Add Phoenix.LiveViewTest.put_connect_info/2 and Phoenix.LiveViewTest.put_connect_params/2 Add support for tracking static asset changes on the page across cold server deploys Add support for passing a @myself target to a hook's pushEventTo target Add configurable metadata for events with new metadata LiveSocket option Add &quot;_mounts&quot; key in connect params which specifies the number of times a LiveView has mounted","ref":"changelog.html#0-13-0-2020-05-21","title":"Changelog - 0.13.0 (2020-05-21)","type":"extras"},{"doc":"Bug fixes Fix component innerHTML being discarded when a sibling DOM element appears above it, in cases where the component lacks a DOM ID Fix Firefox reconnecting briefly during hard redirects Fix phx-disable-with and other pending attributes failing to be restored when an empty patch is returned by server Ensure LiveView module is loaded before mount to prevent first application request logging errors if the very first request is to a connected LiveView","ref":"changelog.html#0-12-1-2020-04-19","title":"Changelog - 0.12.1 (2020-04-19)","type":"extras"},{"doc":"This version of LiveView comes with an overhaul of the testing module, more closely integrating your LiveView template with your LiveView events. For example, in previous versions, you could write this test: render_click ( live_view , &quot;increment_by&quot; , %{ by : 1 } ) However, there is no guarantee that there is any element on the page with a phx-click=&quot;increment&quot; attribute and phx-value-by set to 1. With LiveView 0.12.0, you can now write: live_view |&gt; element ( &quot;#term .buttons a&quot; , &quot;Increment&quot; ) |&gt; render_click ( ) The new implementation will check there is a button at #term .buttons a , with &quot;Increment&quot; as text, validate that it has a phx-click attribute and automatically submit to it with all relevant phx-value entries. This brings us closer to integration/acceptance test frameworks without any of the overhead and complexities of running a headless browser. Enhancements Add assert_patch/3 and assert_patched/2 for asserting on patches Add follow_redirect/3 to automatically follow redirects from render_* events Add phx-trigger-action form annotation to trigger an HTTP form submit on next DOM patch Bug fixes Fix phx-target @myself targeting a sibling LiveView component with the same component ID Fix phx:page-loading-stop firing before the DOM patch has been performed Fix phx-update=&quot;prepend&quot; failing to properly patch the DOM when the same ID is updated back to back Fix redirects on mount failing to copy flash Backwards incompatible changes phx-error-for has been removed in favor of phx-feedback-for . phx-feedback-for will set a phx-no-feedback class whenever feedback has to be hidden Phoenix.LiveViewTest.children/1 has been renamed to Phoenix.LiveViewTest.live_children/1 Phoenix.LiveViewTest.find_child/2 has been renamed to Phoenix.LiveViewTest.find_live_child/2 Phoenix.LiveViewTest.assert_redirect/3 no longer matches on the flash, instead it returns the flash Phoenix.LiveViewTest.assert_redirect/3 no longer matches on the patch redirects, use assert_patch/3 instead Phoenix.LiveViewTest.assert_remove/3 has been removed. If the LiveView crashes, it will cause the test to crash too Passing a path with DOM IDs to render_* test functions is deprecated. Furthermore, they now require a phx-target=&quot;&lt;%= @id %&gt;&quot; on the given DOM ID: &lt; div id = &quot;component-id&quot; phx-target = &quot;component-id&quot; &gt; ... &lt;/ div &gt; html = render_submit ( [ view , &quot;#component-id&quot; ] , event , value ) In any case, this API is deprecated and you should migrate to the new element based API.","ref":"changelog.html#0-12-0-2020-04-16","title":"Changelog - 0.12.0 (2020-04-16)","type":"extras"},{"doc":"Bug fixes Fix readonly states failing to be undone after an empty diff Fix dynamically added child failing to be joined by the client Fix teardown bug causing stale client sessions to attempt a rejoin on reconnect Fix orphaned prepend/append content across joins Track unless in LiveEEx engine Backwards incompatible changes render_event / render_click and friends now expect a DOM ID selector to be given when working with components. For example, instead of render_click([live, &quot;user-13&quot;]) , you should write render_click([live, &quot;#user-13&quot;]) , mirroring the phx-target API. Accessing the socket assigns directly @socket.assigns[...] in a template will now raise the exception Phoenix.LiveView.Socket.AssignsNotInSocket . The socket assigns are available directly inside the template as LiveEEx assigns , such as @foo and @bar . Any assign access should be done using the assigns in the template where proper change tracking takes place. Enhancements Trigger debounced events immediately on input blur Support defaults option on LiveSocket constructor to configure default phx-debounce and phx-throttle values, allowing &lt;input ... phx-debounce&gt; Add detail key to click event metadata for detecting double/triple clicks","ref":"changelog.html#0-11-1-2020-04-08","title":"Changelog - 0.11.1 (2020-04-08)","type":"extras"},{"doc":"Backwards incompatible changes Remove socket.assigns during render to avoid change tracking bugs. If you were previously relying on passing @socket to functions then referencing socket assigns, pass the explicit assign instead to your functions from the template. Removed assets/css/live_view.css . If you want to show a progress bar then in app.css , replace - @import &quot;../../../../deps/phoenix_live_view/assets/css/live_view.css&quot;; + @import &quot;../node_modules/nprogress/nprogress.css&quot;; and add nprogress to assets/package.json . Full details in the Progress animation guide Bug fixes Fix client issue with greater than two levels of LiveView nesting Fix bug causing entire LiveView to be re-rendering with only a component changed Fix issue where rejoins would not trigger phx:page-loading-stop Enhancements Support deep change tracking so @foo.bar only executes and diffs when bar changes Add @myself assign, to allow components to target themselves instead of relying on a DOM ID, for example: phx-target=&quot;&lt;%= @myself %&gt;&quot; Optimize various client rendering scenarios for faster DOM patching of components and append/prepended content Add enableProfiling() and disableProfiling() to LiveSocket for client performance profiling to aid the development process Allow LiveViews to be rendered inside LiveComponents Add support for clearing flash inside components","ref":"changelog.html#0-11-0-2020-04-06","title":"Changelog - 0.11.0 (2020-04-06)","type":"extras"},{"doc":"Backwards incompatible changes Rename socket assign @live_view_module to @live_module Rename socket assign @live_view_action to @live_action LiveView no longer uses the default app layout and put_live_layout is no longer supported. Instead, use put_root_layout . Note, however, that the layout given to put_root_layout must use @inner_content instead of &lt;%= render(@view_module, @view_template, assigns) %&gt; and that the root layout will also be used by regular views. Check the Live Layouts section of the docs. Bug fixes Fix loading states causing nested LiveViews to be removed during live navigation Only trigger phx-update=&quot;ignore&quot; hook if data attributes have changed Fix LiveEEx fingerprint bug causing no diff to be sent in certain cases Enhancements Support collocated templates where an .html.leex template of the same basename of the LiveView will be automatically used for render/1 Add live_title_tag/2 helper for automatic prefix/suffix on @page_title updates","ref":"changelog.html#0-10-0-2020-03-18","title":"Changelog - 0.10.0 (2020-03-18)","type":"extras"},{"doc":"Bug fixes Do not set ignored inputs and buttons as readonly Only decode paths in URIs Only destroy main descendents when replacing main Fix sibling component patches when siblings at same root DOM tree Do not pick the layout from :use on child LiveViews Respect when the layout is set to false in the router and on mount Fix sibling component patch when component siblings lack a container Make flash optional (i.e. LiveView will still work if you don't fetch_flash before) Enhancements Raise if :flash is given as an assign Support user-defined metadata in router Allow the router to be accessed as socket.router Allow MFArgs as the :session option in the live router macro Trigger page loading event when main LV errors Automatically clear the flash on live navigation examples - only the newly assigned flash is persisted","ref":"changelog.html#0-9-0-2020-03-08","title":"Changelog - 0.9.0 (2020-03-08)","type":"extras"},{"doc":"Enhancements Support phx-disable-with on live redirect and live patch links Bug Fixes Fix focus issue on date and time inputs Fix LiveViews failing to mount when page restored from back/forward cache following a redirect on the server Fix IE coercing undefined to string when issuing pushState Fix IE error when focused element is null Fix client error when using components and live navigation where a dynamic template is rendered Fix latent component diff causing error when component removed from DOM before patch arrives Fix race condition where a component event received on the server for a component already removed by the server raised a match error","ref":"changelog.html#0-8-1-2020-02-27","title":"Changelog - 0.8.1 (2020-02-27)","type":"extras"},{"doc":"Backwards incompatible changes Remove Phoenix.LiveView.Flash in favor of :fetch_live_flash imported by Phoenix.LiveView.Router Live layout must now access the child contents with @inner_content instead of invoking the LiveView directly Returning :stop tuples from LiveView mount or handle_[params|call|cast|info|event] is no longer supported. LiveViews are stopped when issuing a redirect or push_redirect Enhancements Add put_live_layout plug to put the root layout used for live routes Allow redirect and push_redirect from mount Use acknowledgement tracking to avoid patching inputs until the server has processed the form event Add css loading states to all phx bound elements with event specific css classes Dispatch phx:page-loading-start and phx:page-loading-stop on window for live navigation, initial page loads, and form submits, for user controlled page loading integration Allow any phx bound element to specify phx-page-loading to dispatch loading events above when the event is pushed Add client side latency simulator with new enableLatencySim(milliseconds) and disableLatencySim() Add enableDebug() and disableDebug() to LiveSocket for ondemand browser debugging from the web console Do not connect LiveSocket WebSocket or bind client events unless a LiveView is found on the page Add transport_pid/1 to return the websocket transport pid when the socket is connected Bug Fixes Fix issue where a failed mount from a live_redirect would reload the current URL instead of the attempted new URL","ref":"changelog.html#0-8-0-2020-02-22","title":"Changelog - 0.8.0 (2020-02-22)","type":"extras"},{"doc":"Bug Fixes Fix checkbox bug failing to send phx-change event to the server in certain cases Fix checkbox bug failing to maintain checked state when a checkbox is programmatically updated by the server Fix select bug in Firefox causing the highlighted index to jump when a patch is applied during hover state","ref":"changelog.html#0-7-1-2020-02-13","title":"Changelog - 0.7.1 (2020-02-13)","type":"extras"},{"doc":"Backwards incompatible changes live_redirect was removed in favor of push_patch (for updating the URL and params of the current LiveView) and push_redirect (for updating the URL to another LiveView) live_link was removed in favor of live_patch (for updating the URL and params of the current LiveView) and live_redirect (for updating the URL to another LiveView) Phoenix.LiveViewTest.assert_redirect no longer accepts an anonymous function in favor of executing the code prior to asserting the redirects, just like assert_receive . Enhancements Support @live_view_action in LiveViews to simplify tracking of URL state Recovery form input data automatically on disconnects or crash recovery Add phx-auto-recover form binding for specialized recovery Scroll to top of page while respecting anchor hash targets on live_patch and live_redirect Add phx-capture-click to use event capturing to bind a click event as it propagates inwards from the target Revamp flash support so it works between static views, live views, and components Add phx-key binding to scope phx-window-keydown and phx-window-keyup events Bug Fixes Send phx-value-* on key events Trigger updated hook callbacks on phx-update=&quot;ignore&quot; container when the container's attributes have changed Fix nested phx-update=&quot;append&quot; raising ArgumentError in LiveViewTest Fix updates not being applied in rare cases where an leex template is wrapped in an if expression","ref":"changelog.html#0-7-0-2020-02-12","title":"Changelog - 0.7.0 (2020-02-12)","type":"extras"},{"doc":"Deprecations LiveView mount/2 has been deprecated in favor of mount/3 . The params are now passed as the first argument to mount/3 , followed by the session and socket. Backwards incompatible changes The socket session now accepts only string keys Enhancements Allow window beforeunload to be cancelled without losing websocket connection Bug Fixes Fix handle_params not decoding URL path parameters properly Fix LiveViewTest error when routing at root path Fix URI encoded params failing to be decoded in handle_params Fix component target failing to locate correct component when the target is on an input tag","ref":"changelog.html#0-6-0-2020-01-22","title":"Changelog - 0.6.0 (2020-01-22)","type":"extras"},{"doc":"Bug Fixes Fix optimization bug causing some DOM nodes to be removed on updates","ref":"changelog.html#0-5-2-2020-01-17","title":"Changelog - 0.5.2 (2020-01-17)","type":"extras"},{"doc":"Bug Fixes Fix phx-change bug causing phx-target to not be used","ref":"changelog.html#0-5-1-2020-01-15","title":"Changelog - 0.5.1 (2020-01-15)","type":"extras"},{"doc":"LiveView now makes the connection session automatically available in LiveViews. However, to do so, you need to configure your endpoint accordingly, otherwise LiveView will fail to connect . The steps are: Find plug Plug.Session, ... in your endpoint.ex and move the options ... to a module attribute: @session_options [ ... ] Change the plug Plug.Session to use said attribute: plug Plug.Session , @session_options Also pass the @session_options to your LiveView socket: socket &quot;/live&quot; , Phoenix.LiveView.Socket , websocket : [ connect_info : [ session : @session_options ] ] You should define the CSRF meta tag inside &lt;head&gt; in your layout, before app.js is included: &lt; meta name = &quot;csrf-token&quot; content = { Plug.CSRFProtection . get_csrf_token ( ) } /&gt; &lt; script type = &quot;text/javascript&quot; src = &quot; &lt;%= Routes . static_path ( @conn , &quot;/js/app.js&quot; ) %&gt; &quot; &gt; &lt;/ script &gt; Then in your app.js: let csrfToken = document . querySelector ( &quot;meta[name=&#39;csrf-token&#39;]&quot; ) . getAttribute ( &quot;content&quot; ) ; let liveSocket = new LiveSocket ( &quot;/live&quot; , Socket , { params : { _csrf_token : csrfToken } } ) ; Also note that the session from now on will have string keys . LiveView will warn if atom keys are used. Enhancements Respect new tab behavior in live_link Add beforeUpdate and beforeDestroy JS hooks Make all assigns defined on the socket mount available on the layout on first render Provide support for live layouts with new :layout option Detect duplicate IDs on the front-end when DEBUG mode is enabled Automatically forward the session to LiveView Support &quot;live_socket_id&quot; session key for identifying (and disconnecting) LiveView sockets Add support for hibernate_after on LiveView processes Support redirecting to full URLs on live_redirect and redirect Add offsetX and offsetY to click event metadata Allow live_link and live_redirect to exist anywhere in the page and it will always target the main LiveView (the one defined at the router) Backwards incompatible changes phx-target=&quot;window&quot; has been removed in favor of phx-window-keydown , phx-window-focus , etc, and the phx-target binding has been repurposed for targeting LiveView and LiveComponent events from the client Phoenix.LiveView no longer defined live_render and live_link . These functions have been moved to Phoenix.LiveView.Helpers which can now be fully imported in your views. In other words, replace import Phoenix.LiveView, only: [live_render: ..., live_link: ...] by import Phoenix.LiveView.Helpers","ref":"changelog.html#0-5-0-2020-01-15","title":"Changelog - 0.5.0 (2020-01-15)","type":"extras"},{"doc":"Bug Fixes Fix bug causing blurred inputs","ref":"changelog.html#0-4-1-2019-11-07","title":"Changelog - 0.4.1 (2019-11-07)","type":"extras"},{"doc":"Enhancements Add Phoenix.LiveComponent to compartmentalize state, markup, and events in LiveView Handle outdated clients by refreshing the page with jitter when a valid, but outdated session is detected Only dispatch live link clicks to router LiveView Refresh the page for graceful error recovery on failed mount when the socket is in a connected state Bug Fixes Fix phx-hook destroyed callback failing to be called in certain cases Fix back/forward bug causing LiveView to fail to remount","ref":"changelog.html#0-4-0-2019-11-07","title":"Changelog - 0.4.0 (2019-11-07)","type":"extras"},{"doc":"Backwards incompatible changes live_isolated in tests no longer requires a router and a pipeline (it now expects only 3 arguments) Raise if handle_params is used on a non-router LiveView Bug Fixes [LiveViewTest] Fix function clause errors caused by HTML comments","ref":"changelog.html#0-3-1-2019-09-23","title":"Changelog - 0.3.1 (2019-09-23)","type":"extras"},{"doc":"Enhancements Add phx-debounce and phx-throttle bindings to rate limit events Backwards incompatible changes IE11 support now requires two additional polyfills, mdn-polyfills/CustomEvent and mdn-polyfills/String.prototype.startsWith Bug Fixes Fix IE11 support caused by unsupported getAttributeNames lookup Fix Floki dependency compilation warnings","ref":"changelog.html#0-3-0-2019-09-19","title":"Changelog - 0.3.0 (2019-09-19)","type":"extras"},{"doc":"Bug Fixes [LiveView.Router] Fix module concat failing to build correct layout module when using custom namespace [LiveViewTest] Fix phx-update append/prepend containers not building proper DOM content [LiveViewTest] Fix phx-update append/prepend containers not updating existing child containers with matching IDs","ref":"changelog.html#0-2-1-2019-09-17","title":"Changelog - 0.2.1 (2019-09-17)","type":"extras"},{"doc":"Enhancements [LiveView] Add new :container option to use Phoenix.LiveView [LiveViewTest] Add live_isolated test helper for testing LiveViews which are not routable Backwards incompatible changes Replace configure_temporary_assigns/2 with 3-tuple mount return, supporting a :temporary_assigns key Do not allow redirect / live_redirect on mount nor in child live views All phx-update containers now require a unique ID LiveSocket JavaScript constructor now requires explicit dependency injection of Phoenix Socket constructor. For example: import { Socket } from &quot;phoenix&quot; import LiveSocket from &quot;phoenix_live_view&quot; let liveSocket = new LiveSocket ( &quot;/live&quot; , Socket , { ... } ) Bug Fixes Fix phx-update=append/prepend failing to join new nested live views or wire up new phx-hooks Fix number input handling causing some browsers to reset form fields on invalid inputs Fix multi-select decoding causing server error Fix multi-select change tracking failing to submit an event when a value is deselected Fix live redirect loop triggered under certain scenarios Fix params failing to update on re-mounts after live_redirect Fix blur event metadata being sent with type of &quot;focus&quot;","ref":"changelog.html#0-2-0-2019-09-12","title":"Changelog - 0.2.0 (2019-09-12)","type":"extras"},{"doc":"Backwards incompatible changes phx-value has no effect, use phx-value-* instead The :path_params key in session has no effect (use handle_params in LiveView instead)","ref":"changelog.html#0-1-2-2019-08-28","title":"Changelog - 0.1.2 (2019-08-28)","type":"extras"},{"doc":"Enhancements Use optimized insertAdjacentHTML for faster append/prepend and proper css animation handling Allow for replacing previously appended/prepended elements by replacing duplicate IDs during append/prepend instead of adding new DOM nodes Bug Fixes Fix duplicate append/prepend updates when parent content is updated Fix JS hooks not being applied for appending and prepended content","ref":"changelog.html#0-1-1-2019-08-27","title":"Changelog - 0.1.1 (2019-08-27)","type":"extras"},{"doc":"Enhancements The LiveView handle_in/3 callback now receives a map of metadata about the client event For phx-change events, handle_in/3 now receives a &quot;_target&quot; param representing the keyspace of the form input name which triggered the change Multiple values may be provided for any phx binding by using the phx-value- prefix, such as phx-value-myval1 , phx-value-myval2 , etc Add control over the DOM patching via phx-update , which can be set to &quot;replace&quot; , &quot;append&quot; , &quot;prepend&quot; or &quot;ignore&quot; Backwards incompatible changes phx-ignore was renamed to phx-update=&quot;ignore&quot;","ref":"changelog.html#0-1-0-2019-08-25","title":"Changelog - 0.1.0 (2019-08-25)","type":"extras"},{"doc":"","ref":"installation.html","title":"Installation","type":"extras"},{"doc":"Phoenix v1.5+ comes with built-in support for LiveView apps. Just create your application with mix phx.new my_app --live . The --live flag has become the default on Phoenix v1.6. Once you've created a LiveView project, refer to LiveView documentation for further information on how to use it.","ref":"installation.html#new-projects","title":"Installation - New projects","type":"extras"},{"doc":"If you are using a Phoenix version earlier than v1.5 or your app already exists, continue with the following steps. The instructions below will serve if you are installing the latest stable version from Hex. To start using LiveView, add one of the following dependencies to your mix.exs and run mix deps.get . If installing from Hex, use the latest version from there: def deps do [ { :phoenix_live_view , &quot;~&gt; 0.18&quot; } , { :floki , &quot;&gt;= 0.30.0&quot; , only : :test } ] end If you want the latest features, install from GitHub: def deps do [ { :phoenix_live_view , github : &quot;phoenixframework/phoenix_live_view&quot; } , { :floki , &quot;&gt;= 0.30.0&quot; , only : :test } ] Once installed, update your endpoint's configuration to include a signing salt. You can generate a signing salt by running mix phx.gen.secret 32 : # config/config.exs config :my_app , MyAppWeb.Endpoint , live_view : [ signing_salt : &quot;SECRET_SALT&quot; ] Next, add the following imports to your web file in lib/my_app_web.ex : # lib/my_app_web.ex def view do quote do # ... import Phoenix.Component end end def router do quote do # ... import Phoenix.LiveView.Router end end In that same file, update your live_view layout configuration: # lib/my_app_web.ex def live_view do use Phoenix.LiveView, - layout: {MyAppWeb.LayoutView, &quot;live.html&quot;} + layout: {MyAppWeb.LayoutView, :live} unquote(view_helpers()) end Then add the Phoenix.LiveView.Router.fetch_live_flash/2 plug to your browser pipeline, in place of :fetch_flash : # lib/my_app_web/router.ex pipeline :browser do # ... plug :fetch_session - plug :fetch_flash + plug :fetch_live_flash end Next, expose a new socket for LiveView updates in your app's endpoint module. # lib/my_app_web/endpoint.ex defmodule MyAppWeb.Endpoint do use Phoenix.Endpoint # ... socket &quot;/live&quot; , Phoenix.LiveView.Socket , websocket : [ connect_info : [ session : @session_options ] ] # ... end Where @session_options are the options given to plug Plug.Session by using a module attribute. If you don't have a @session_options in your endpoint yet, here is how to create one: Find plug Plug.Session in your endpoint.ex plug Plug.Session store : :cookie , key : &quot;_my_app_key&quot; , signing_salt : &quot;somesigningsalt&quot; Move the options to a module attribute at the top of your file: @session_options [ store : :cookie , key : &quot;_my_app_key&quot; , signing_salt : &quot;somesigningsalt&quot; ] Change the plug Plug.Session to use that attribute: plug Plug.Session , @session_options Finally, ensure you have placed a CSRF meta tag inside the &lt;head&gt; tag in your layout ( lib/my_app_web/templates/layout/app.html.* ) before app.js is included, like so: &lt; meta name = &quot;csrf-token&quot; content = { Plug.CSRFProtection . get_csrf_token ( ) } /&gt; &lt; script defer type = &quot;text/javascript&quot; src = &quot; &lt;%= Routes . static_path ( @conn , &quot;/js/app.js&quot; ) %&gt; &quot; &gt; &lt;/ script &gt; and enable connecting to a LiveView socket in your app.js file. // assets/js/app.js import {Socket} from &quot;phoenix&quot; import {LiveSocket} from &quot;phoenix_live_view&quot; let csrfToken = document.querySelector(&quot;meta[name='csrf-token']&quot;).getAttribute(&quot;content&quot;) let liveSocket = new LiveSocket(&quot;/live&quot;, Socket, {params: {_csrf_token: csrfToken}}) // Connect if there are any LiveViews on the page liveSocket.connect() // Expose liveSocket on window for web console debug logs and latency simulation: // &gt;&gt; liveSocket.enableDebug() // &gt;&gt; liveSocket.enableLatencySim(1000) // The latency simulator is enabled for the duration of the browser session. // Call disableLatencySim() to disable: // &gt;&gt; liveSocket.disableLatencySim() window.liveSocket = liveSocket The JavaScript above expects phoenix_live_view to be available as a JavaScript dependency. Let's do that.","ref":"installation.html#existing-projects","title":"Installation - Existing projects","type":"extras"},{"doc":"If using npm , you need to add LiveView to your assets/package.json . For a regular project, do: { &quot;dependencies&quot;: { &quot;phoenix&quot;: &quot;file:../deps/phoenix&quot;, &quot;phoenix_html&quot;: &quot;file:../deps/phoenix_html&quot;, &quot;phoenix_live_view&quot;: &quot;file:../deps/phoenix_live_view&quot; } } However, if you're adding phoenix_live_view to an umbrella project, the dependency paths should be modified appropriately: { &quot;dependencies&quot;: { &quot;phoenix&quot;: &quot;file:../../../deps/phoenix&quot;, &quot;phoenix_html&quot;: &quot;file:../../../deps/phoenix_html&quot;, &quot;phoenix_live_view&quot;: &quot;file:../../../deps/phoenix_live_view&quot; } } Now run the next commands from the root of your web app project: npm install --prefix assets If you had previously installed phoenix_live_view and want to get the latest javascript, then force an install with: npm install --force phoenix_live_view --prefix assets","ref":"installation.html#npm-dependencies","title":"Installation - npm dependencies","type":"extras"},{"doc":"LiveView does not use the default app layout. Instead, you typically call put_root_layout in your router to specify a layout that is used by both &quot;regular&quot; views and live views. In your router, do: # lib/my_app_web/router.ex pipeline :browser do # ... plug :put_root_layout , { MyAppWeb.LayoutView , :root } # ... end The layout given to put_root_layout is typically very barebones, with mostly &lt;head&gt; and &lt;body&gt; tags. For example: &lt;!DOCTYPE html&gt; &lt; html lang = &quot;en&quot; &gt; &lt; head &gt; &lt; meta name = &quot;csrf-token&quot; content = { Plug.CSRFProtection . get_csrf_token ( ) } /&gt; &lt; Phoenix.Component.live_title &gt; &lt;%= assigns [ :page_title ] || &quot;MyApp&quot; %&gt; &lt;/ Phoenix.Component.live_title &gt; &lt; link rel = &quot;stylesheet&quot; href = &quot; &lt;%= Routes . static_path ( @conn , &quot;/css/app.css&quot; ) %&gt; &quot; /&gt; &lt; script defer type = &quot;text/javascript&quot; src = &quot; &lt;%= Routes . static_path ( @conn , &quot;/js/app.js&quot; ) %&gt; &quot; &gt; &lt;/ script &gt; &lt;/ head &gt; &lt; body &gt; &lt;%= @inner_content %&gt; &lt;/ body &gt; &lt;/ html &gt; Once you have specified a root layout, app.html.heex will be rendered within your root layout for all non-LiveViews. You may also optionally define a live.html.heex layout to be used across all LiveViews, as we will describe in the next section. Optionally, you can add a phx-track-static to all script and link elements in your layout that use src and href . This way you can detect when new assets have been deployed by calling static_changed? . &lt; link phx - track - static rel = &quot;stylesheet&quot; href = { Routes . static_path ( @conn , &quot;/css/app.css&quot; ) } / &gt; &lt; script phx - track - static defer type = &quot;text/javascript&quot; src = { Routes . static_path ( @conn , &quot;/js/app.js&quot; ) } &gt; &lt; / script &gt;","ref":"installation.html#layouts","title":"Installation - Layouts","type":"extras"},{"doc":"If you want to show a progress bar as users perform live actions, we recommend using topbar . You can either add a copy of topbar to assets/vendor/topbar.js or add it as a npm dependency by calling: $ npm install --prefix assets --save topbar Then customize LiveView to use it in your assets/js/app.js , right before the liveSocket.connect() call: // Show progress bar on live navigation and form submits import topbar from &quot;topbar&quot; topbar.config({barColors: {0: &quot;#29d&quot;}, shadowColor: &quot;rgba(0, 0, 0, .3)&quot;}) window.addEventListener(&quot;phx:page-loading-start&quot;, info =&gt; topbar.show()) window.addEventListener(&quot;phx:page-loading-stop&quot;, info =&gt; topbar.hide()) Alternatively, you can also delay showing the topbar and wait if the results do not appear within 200ms: // Show progress bar on live navigation and form submits import topbar from &quot;topbar&quot; topbar.config({barColors: {0: &quot;#29d&quot;}, shadowColor: &quot;rgba(0, 0, 0, .3)&quot;}) let topBarScheduled = undefined window.addEventListener(&quot;phx:page-loading-start&quot;, () =&gt; { if(!topBarScheduled) { topBarScheduled = setTimeout(() =&gt; topbar.show(), 200) } }) window.addEventListener(&quot;phx:page-loading-stop&quot;, () =&gt; { clearTimeout(topBarScheduled) topBarScheduled = undefined topbar.hide() })","ref":"installation.html#progress-animation","title":"Installation - Progress animation","type":"extras"},{"doc":"By convention your LiveView modules and heex templates should be placed in lib/my_app_web/live/ directory.","ref":"installation.html#location-for-liveview-modules","title":"Installation - Location for LiveView modules","type":"extras"},{"doc":"All of the data in a LiveView is stored in the socket as assigns, which is a server side struct in Phoenix.LiveView.Socket . Socket state is never shared with the client beyond what your template renders. The Phoenix.Component.assign/2 and Phoenix.Component.assign/3 functions help store those values. Those values can be accessed in the LiveView as socket.assigns.name but they are accessed inside LiveView templates as @name . Phoenix template language is called HEEx (HTML+EEx). Those templates are either files with the .heex extension or they are created directly in source files via the ~H sigil. You can learn more about the HEEx syntax by checking the docs for the ~H sigil . In this section, we are going to cover how LiveView minimizes the payload over the wire by understanding the interplay between assigns and templates.","ref":"assigns-eex.html","title":"Assigns and HEEx templates","type":"extras"},{"doc":"When you first render a .heex template, it will send all of the static and dynamic parts of the template to the client. Imagine the following template: &lt; h1 &gt; &lt;%= expand_title ( @title ) %&gt; &lt;/ h1 &gt; It has two static parts, &lt;h1&gt; and &lt;/h1&gt; and one dynamic part made of expand_title(@title) . Further rendering of this template won't resend the static parts and it will only resend the dynamic part if it changes. The tracking of changes is done via assigns. If the @title assign changes, then LiveView will execute expand_title(@title) and send the new content. If @title is the same, nothing is executed and nothing is sent. Change tracking also works when accessing map/struct fields. Take this template: &lt; div id = { &quot;user_ \#{ @user . id } &quot; } &gt; &lt;%= @user . name %&gt; &lt;/ div &gt; If the @user.name changes but @user.id doesn't, then LiveView will re-render only @user.name and it will not execute or resend @user.id at all. The change tracking also works when rendering other templates as long as they are also .heex templates: &lt;%= render &quot;child_template.html&quot; , assigns %&gt; Or when using function components: &lt; .show_name name = { @user . name } /&gt; The assign tracking feature also implies that you MUST avoid performing direct operations in the template. For example, if you perform a database query in your template: &lt;%= for user &lt;- Repo . all ( User ) do %&gt; &lt;%= user . name %&gt; &lt;% end %&gt; Then Phoenix will never re-render the section above, even if the number of users in the database changes. Instead, you need to store the users as assigns in your LiveView before it renders the template: assign ( socket , :users , Repo . all ( User ) ) Generally speaking, data loading should never happen inside the template , regardless if you are using LiveView or not. The difference is that LiveView enforces this best practice.","ref":"assigns-eex.html#change-tracking","title":"Assigns and HEEx templates - Change tracking","type":"extras"},{"doc":"There are two common pitfalls to keep in mind when using the ~H sigil or .heex templates inside LiveViews. When it comes to do/end blocks, change tracking is supported only on blocks given to Elixir's basic constructs, such as if , case , for , and similar. If the do/end block is given to a library function or user function, such as content_tag , change tracking won't work. For example, imagine the following template that renders a div : &lt;%= content_tag :div , id : &quot;user_ \#{ @id } &quot; do %&gt; &lt;%= @name %&gt; &lt;%= @description %&gt; &lt;% end %&gt; LiveView knows nothing about content_tag , which means the whole div will be sent whenever any of the assigns change. Luckily, HEEx templates provide a nice syntax for building tags, so there is rarely a need to use content_tag inside .heex templates: &lt; div id = { &quot;user_ \#{ @id } &quot; } &gt; &lt;%= @name %&gt; &lt;%= @description %&gt; &lt;/ div &gt; The next pitfall is related to variables. Due to the scope of variables, LiveView has to disable change tracking whenever variables are used in the template, with the exception of variables introduced by Elixir basic case , for , and other block constructs. Therefore, you must avoid code like this in your LiveView templates: &lt;% some_var = @x + @y %&gt; &lt;%= some_var %&gt; Instead, use a function: &lt;%= sum ( @x , @y ) %&gt; Similarly, do not define variables at the top of your render function: def render ( assigns ) do sum = assigns . x + assigns . y ~H&quot;&quot;&quot; &lt;%= sum %&gt; &quot;&quot;&quot; end Instead explicitly precompute the assign in your LiveView, outside of render: assign ( socket , sum : socket . assigns . x + socket . assigns . y ) Generally speaking, avoid accessing variables inside LiveViews, as code that access variables is always executed on every render. This also applies to the assigns variable. The exception are variables introduced by Elixir's block constructs. For example, accessing the post variable defined by the comprehension below works as expected: &lt;%= for post &lt;- @posts do %&gt; ... &lt;% end %&gt; To sum up: Avoid passing block expressions to library and custom functions, instead prefer to use the conveniences in HEEx templates Avoid defining local variables, except within Elixir's constructs","ref":"assigns-eex.html#pitfalls","title":"Assigns and HEEx templates - Pitfalls","type":"extras"},{"doc":"As with any other Elixir code, exceptions may happen during the LiveView life-cycle. In this section we will describe how LiveView reacts to errors at different stages.","ref":"error-handling.html","title":"Error and exception handling","type":"extras"},{"doc":"In this section, we will talk about error cases that you expect to happen within your application. For example, a user filling in a form with invalid data is expected. In a LiveView, we typically handle those cases by storing a change in the LiveView state, which causes the LiveView to be re-rendered with the error message. We may also use flash messages for this. For example, imagine you have a page to manage all &quot;Team members&quot; in an organization. However, if there is only one member left in the organization, they should not be allowed to leave. You may want to handle this by using flash messages: if MyApp.Org . leave ( socket . assigns . current_org , member ) do { :noreply , socket } else { :noreply , put_flash ( socket , :error , &quot;last member cannot leave organization&quot; ) } end However, one may argue that, if the last member of an organization cannot leave it, it may be better to not even show the &quot;Leave&quot; button in the UI when the organization has only one member. Given the button does not appear in the UI, triggering the &quot;leave&quot; when the organization has now only one member is an unexpected scenario. This means we can probably rewrite the code above to: true = MyApp.Org . leave ( socket . assigns . current_org , member ) { :noreply , socket } If leave returns false by any chance, it will just raise. Or you can even provide a leave! function that raises a specific exception: MyApp.Org . leave! ( socket . assigns . current_org , member ) { :noreply , socket } However, what will happen with a LiveView in case of exceptions? Let's talk about unexpected scenarios.","ref":"error-handling.html#expected-scenarios","title":"Error and exception handling - Expected scenarios","type":"extras"},{"doc":"Elixir developers tend to write assertive code. This means that, if we expect leave to always return true, we can explicitly match on its result, as we did above: true = MyApp.Org . leave ( socket . assigns . current_org , member ) { :noreply , socket } If leave fails and returns false , an exception is raised. It is common for Elixir developers to use exceptions for unexpected scenarios in their Phoenix applications. For example, if you are building an application where a user may belong to one or more organizations, when accessing the organization page, you may want to check that the user has access to it like this: organizations_query = Ecto . assoc ( socket . assigns . current_user , :organizations ) Repo . get! ( organizations_query , params [ &quot;org_id&quot; ] ) The code above builds a query that returns all organizations that belongs to the current user and then validates that the given &quot;org_id&quot; belongs to the user. If there is no such &quot;org_id&quot; or if the user has no access to it, an Ecto.NoResultsError exception is raised. During a regular controller request, this exception will be converted to a 404 exception and rendered as a custom error page, as detailed here . To understand how a LiveView reacts to exceptions, we need to consider two scenarios: exceptions on mount and during any event.","ref":"error-handling.html#unexpected-scenarios","title":"Error and exception handling - Unexpected scenarios","type":"extras"},{"doc":"Given the code on mount runs both on the initial disconnected render and the connected render, an exception on mount will trigger the following events: Exceptions during disconnected render: An exception on mount is caught and converted to an exception page by Phoenix error views - pretty much like the way it works with controllers Exceptions during connected render: An exception on mount will crash the LiveView process - which will be logged Once the client has noticed the crash during mount , it will fully reload the page Reloading the page will start a disconnected render, that will cause mount to be invoked again and most likely raise the same exception. Except this time it will be caught and converted to an exception page by Phoenix error views In other words, LiveView will reload the page in case of errors, making it fail as if LiveView was not involved in the rendering in the first place.","ref":"error-handling.html#exceptions-on-mount","title":"Error and exception handling - Exceptions on mount","type":"extras"},{"doc":"If the error happens during an event, the LiveView process will crash. The client will notice the error and remount the LiveView - without reloading the page. This is enough to update the page and show the user the latest information. For example, let's say two users try to leave the organization at the same time. In this case, both of them see the &quot;Leave&quot; button, but our leave function call will succeed only for one of them: true = MyApp.Org . leave ( socket . assigns . current_org , member ) { :noreply , socket } When the exception raises, the client will remount the LiveView. Once you remount, your code will now notice that there is only one user in the organization and therefore no longer show the &quot;Leave&quot; button. In other words, by remounting, we often update the state of the page, allowing exceptions to be automatically handled. Note that the choice between conditionally checking on the result of the leave function with an if , or simply asserting it returns true , is completely up to you. If the likelihood of everyone leaving the organization at the same time is low, then you may as well treat it as an unexpected scenario. Although other developers will be more comfortable by explicitly handling those cases. In both scenarios, LiveView has you covered.","ref":"error-handling.html#exceptions-on-events-handle_info-handle_event-etc","title":"Error and exception handling - Exceptions on events (handle_info, handle_event, etc)","type":"extras"},{"doc":"NOTE: Make sure you've read the Assigns and HEEx templates guide before moving forward. From Phoenix v1.7, your application is made of two layouts: the root layout - this is a layout used by both LiveView and regular views. This layout typically contains the &lt;html&gt; definition alongside the head and body tags. Any content defined in the root layout will remain the same, even as you live navigate across LiveViews. The root layout is typically declared on the router with put_root_layout and defined as &quot;root.html.heex&quot; in your layouts folder. It may also be given via the :root_layout option to a live_session macro in the router. the app layout - this is the default application layout which is rendered on both regular HTTP requests and LiveViews. It defaults to &quot;app.html.heex&quot; Overall, those layouts are found in components/layouts and are embedded within MyAppWeb.Layouts . All layouts must call &lt;%= @inner_content %&gt; to inject the content rendered by the layout. The &quot;root&quot; layout is rendered only on the initial request and therefore it has access to the @conn assign. The root layout is typically defined in your router: plug :put_root_layout , { MyAppWeb.LayoutView , :root } The &quot;app.html.heex&quot; layout is rendered with either @conn or @socket . See the def controller and def live_view definitions in your MyAppWeb to learn how it is included. Note : The live layout is always wrapped by the LiveView's :container tag.","ref":"live-layouts.html","title":"Live layouts","type":"extras"},{"doc":"Because the root layout from the Plug pipeline is rendered outside of LiveView, the contents cannot be dynamically changed. The one exception is the &lt;title&gt; of the HTML document. Phoenix LiveView special cases the @page_title assign to allow dynamically updating the title of the page, which is useful when using live navigation, or annotating the browser tab with a notification. For example, to update the user's notification count in the browser's title bar, first set the page_title assign on mount: def mount ( _params , _session , socket ) do socket = assign ( socket , page_title : &quot;Latest Posts&quot; ) { :ok , socket } end Then access @page_title in the root layout: &lt; title &gt; &lt;%= @page_title %&gt; &lt;/ title &gt; You can also use the Phoenix.Component.live_title/1 component to support adding automatic prefix and suffix to the page title when rendered and on subsequent updates: &lt; Phoenix.Component.live_title prefix = &quot;MyApp – &quot; &gt; &lt;%= assigns [ :page_title ] || &quot;Welcome&quot; %&gt; &lt;/ Phoenix.Component.live_title &gt; Although the root layout is not updated by LiveView, by simply assigning to page_title , LiveView knows you want the title to be updated: def handle_info ( { :new_messages , count } , socket ) do { :noreply , assign ( socket , page_title : &quot;Latest Posts ( \#{ count } new)&quot; ) } end Note : If you find yourself needing to dynamically patch other parts of the base layout, such as injecting new scripts or styles into the &lt;head&gt; during live navigation, then a regular, non-live, page navigation should be used instead . Assigning the @page_title updates the document.title directly, and therefore cannot be used to update any other part of the base layout.","ref":"live-layouts.html#updating-the-html-document-title","title":"Live layouts - Updating the HTML document title","type":"extras"},{"doc":"LiveView provides functionality to allow page navigation using the browser's pushState API . With live navigation, the page is updated without a full page reload. You can trigger live navigation in two ways: From the client - this is done by passing either patch={url} or navigate={url} to the Phoenix.Component.link/1 component From the server - this is done by by Phoenix.LiveView.push_patch/2 or Phoenix.LiveView.push_navigate/2 . For example, instead of writing the following in a template: &lt; .link href = { ~p&quot;/pages/ \#{ @page + 1 } &quot; } &gt; Next &lt;/ .link &gt; You would write: &lt; .link patch = { ~p&quot;/pages/ \#{ @page + 1 } &quot; } &gt; Next &lt;/ .link &gt; Or in a LiveView: { :noreply , push_patch ( socket , to : ~p&quot;/pages/ \#{ @page + 1 } &quot; ) } The &quot;patch&quot; operations must be used when you want to navigate to the current LiveView, simply updating the URL and the current parameters, without mounting a new LiveView. When patch is used, the handle_params/3 callback is invoked and the minimal set of changes are sent to the client. See the next section for more information. The &quot;navigate&quot; operations must be used when you want to dismount the current LiveView and mount a new one. You can only &quot;navigate&quot; between LiveViews in the same session. While redirecting, a phx-loading class is added to the LiveView, which can be used to indicate to the user a new page is being loaded. If you attempt to patch to another LiveView or navigate across live sessions, a full page reload is triggered. This means your application continues to work, in case your application structure changes and that's not reflected in the navigation. Here is a quick breakdown: &lt;.link href={...}&gt; and redirect/2 are HTTP-based, work everywhere, and perform full page reloads &lt;.link navigate={...}&gt; and push_navigate/2 work across LiveViews in the same session. They mount a new LiveView while keeping the current layout &lt;.link patch={...}&gt; and push_patch/2 updates the current LiveView and sends only the minimal diff while also maintaining the scroll position","ref":"live-navigation.html","title":"Live navigation","type":"extras"},{"doc":"The handle_params/3 callback is invoked after mount/3 and before the initial render. It is also invoked every time &lt;.link patch={...}&gt; or push_patch/2 are used. It receives the request parameters as first argument, the url as second, and the socket as third. For example, imagine you have a UserTable LiveView to show all users in the system and you define it in the router as: live &quot;/users&quot; , UserTable Now to add live sorting, you could do: &lt; .link patch = { path ( ~p&quot;/users&quot; , sort_by : &quot;name&quot; ) } &gt; Sort by name &lt;/ .link &gt; When clicked, since we are navigating to the current LiveView, handle_params/3 will be invoked. Remember you should never trust the received params, so you must use the callback to validate the user input and change the state accordingly: def handle_params ( params , _uri , socket ) do socket = case params [ &quot;sort_by&quot; ] do sort_by when sort_by in ~w(name company) -&gt; assign ( socket , sort_by : sort_by ) _ -&gt; socket end { :noreply , load_users ( socket ) } end Note we returned {:noreply, socket} , where :noreply means no additional information is sent to the client. As with other handle_* callbacks, changes to the state inside handle_params/3 will trigger a new server render. Note the parameters given to handle_params/3 are the same as the ones given to mount/3 . So how do you decide which callback to use to load data? Generally speaking, data should always be loaded on mount/3 , since mount/3 is invoked once per LiveView life-cycle. Only the params you expect to be changed via &lt;.link patch={...}&gt; or push_patch/2 must be loaded on handle_params/3 . For example, imagine you have a blog. The URL for a single post is: &quot;/blog/posts/:post_id&quot;. In the post page, you have comments and they are paginated. You use &lt;.link patch={...}&gt; to update the shown comments every time the user paginates, updating the URL to &quot;/blog/posts/:post_id?page=X&quot;. In this example, you will access &quot;post_id&quot; on mount/3 and the page of comments on handle_params/3 .","ref":"live-navigation.html#handle_params-3","title":"Live navigation - handle_params/3","type":"extras"},{"doc":"LiveView also allows the current browser URL to be replaced. This is useful when you want certain events to change the URL but without polluting the browser's history. This can be done by passing the &lt;.link replace&gt; option to any of the navigation helpers.","ref":"live-navigation.html#replace-page-address","title":"Live navigation - Replace page address","type":"extras"},{"doc":"LiveView allows you to have multiple LiveViews in the same page by calling Phoenix.Component.live_render/3 in your templates. However, only the LiveViews defined directly in your router can use the &quot;Live Navigation&quot; functionality described here. This is important because LiveViews work closely with your router, guaranteeing you can only navigate to known routes.","ref":"live-navigation.html#multiple-liveviews-in-the-same-page","title":"Live navigation - Multiple LiveViews in the same page","type":"extras"},{"doc":"LiveView begins its life-cycle as a regular HTTP request. Then a stateful connection is established. Both the HTTP request and the stateful connection receive the client data via parameters and session. This means that any session validation must happen both in the HTTP request (plug pipeline) and the stateful connection (LiveView mount).","ref":"security-model.html","title":"Security considerations of the LiveView model","type":"extras"},{"doc":"When speaking about security, there are two terms commonly used: authentication and authorization. Authentication is about identifying a user. Authorization is about telling if a user has access to a certain resource or feature in the system. In a regular web application, once a user is authenticated, for example by entering their email and password, or by using a third-party service such as Google, Twitter, or Facebook, a token identifying the user is stored in the session, which is a cookie (a key-value pair) stored in the user's browser. Every time there is a request, we read the value from the session, and, if valid, we fetch the user stored in the session from the database. The session is automatically validated by Phoenix and tools like mix phx.gen.auth can generate the building blocks of an authentication system for you. Once the user is authenticated, they may perform many actions on the page, and some of those actions require specific permissions. This is called authorization and the specific rules often change per application. In a regular web application, we perform authentication and authorization checks on every request. In LiveView, we should also run those exact same checks, always. Once the user is authenticated, we typically validate the sessions on the mount callback. Authorization rules generally happen on mount (for instance, is the user allowed to see this page?) and also on handle_event (is the user allowed to delete this item?).","ref":"security-model.html#authentication-vs-authorization","title":"Security considerations of the LiveView model - Authentication vs authorization","type":"extras"},{"doc":"The mount/3 callback is invoked both on the initial HTTP mount and when LiveView is connected. Therefore, any authentication performed during mount will cover all scenarios. If you perform user authentication and confirmation exclusively on HTTP requests via Plugs, such as this: plug :ensure_user_authenticated plug :ensure_user_confirmed Then the mount/3 callback of your LiveView should execute those same verifications: def mount ( _params , %{ &quot;user_id&quot; =&gt; user_id } = _session , socket ) do socket = assign ( socket , current_user : Accounts . get_user! ( user_id ) ) socket = if socket . assigns . current_user . confirmed_at do socket else redirect ( socket , to : &quot;/login&quot; ) end { :ok , socket } end LiveView v0.17 includes the on_mount ( Phoenix.LiveView.on_mount/1 ) hook, which allows you to encapsulate this logic and execute it on every mount, as you would with plug: defmodule MyAppWeb.UserLiveAuth do import Phoenix.Component import Phoenix.LiveView alias MyAppWeb.Accounts # from `mix phx.gen.auth` def on_mount ( :default , _params , %{ &quot;user_token&quot; =&gt; user_token } = _session , socket ) do socket = assign_new ( socket , :current_user , fn -&gt; Accounts . get_user_by_session_token ( user_token ) end ) if socket . assigns . current_user . confirmed_at do { :cont , socket } else { :halt , redirect ( socket , to : &quot;/login&quot; ) } end end end We use assign_new/3 . This is a convenience to avoid fetching the current_user multiple times across LiveViews. Now we can use the hook whenever relevant: defmodule MyAppWeb.PageLive do use MyAppWeb , :live_view on_mount MyAppWeb.UserLiveAuth ... end If you prefer, you can add the hook to def live_view under MyAppWeb , to run it on all LiveViews by default: def live_view do quote do use Phoenix.LiveView , layout : { MyAppWeb.LayoutView , :live } on_mount MyAppWeb.UserLiveAuth unquote ( html_helpers ( ) ) end end","ref":"security-model.html#mounting-considerations","title":"Security considerations of the LiveView model - Mounting considerations","type":"extras"},{"doc":"Every time the user performs an action on your system, you should verify if the user is authorized to do so, regardless if you are using LiveViews or not. For example, imagine a user can see all projects in a web application, but they may not have permission to delete any of them. At the UI level, you handle this accordingly by not showing the delete button in the projects listing, but a savvy user can directly talk to the server and request a deletion anyway. For this reason, you must always verify permissions on the server . In LiveView, most actions are handled by the handle_event callback. Therefore, you typically authorize the user within those callbacks. In the scenario just described, one might implement this: on_mount MyAppWeb.UserLiveAuth def mount ( _params , _session , socket ) do { :ok , load_projects ( socket ) } end def handle_event ( &quot;delete_project&quot; , %{ &quot;project_id&quot; =&gt; project_id } , socket ) do Project . delete! ( socket . assigns . current_user , project_id ) { :noreply , update ( socket , :projects , &amp; Enum . reject ( &amp;1 , fn p -&gt; p . id == project_id end ) } end defp load_projects ( socket ) do projects = Project . all_projects ( socket . assigns . current_user ) assign ( socket , projects : projects ) end First, we used on_mount to authenticate the user based on the data stored in the session. Then we load all projects based on the authenticated user. Now, whenever there is a request to delete a project, we still pass the current user as argument to the Project context, so it verifies if the user is allowed to delete it or not. In case it cannot delete, it is fine to just raise an exception. After all, users are not meant to trigger this code path anyway (unless they are fiddling with something they are not supposed to!).","ref":"security-model.html#events-considerations","title":"Security considerations of the LiveView model - Events considerations","type":"extras"},{"doc":"So far, the security model between LiveView and regular web applications have been remarkably similar. After all, we must always authenticate and authorize every user. The main difference between them happens on logout or when revoking access. Because LiveView is a permanent connection between client and server, if a user is logged out, or removed from the system, this change won't reflect on the LiveView part unless the user reloads the page. Luckily, it is possible to address this by setting a live_socket_id in the session. For example, when logging in a user, you could do: conn |&gt; put_session ( :current_user_id , user . id ) |&gt; put_session ( :live_socket_id , &quot;users_socket: \#{ user . id } &quot; ) Now all LiveView sockets will be identified and listen to the given live_socket_id . You can then disconnect all live users identified by said ID by broadcasting on the topic: MyAppWeb.Endpoint . broadcast ( &quot;users_socket: \#{ user . id } &quot; , &quot;disconnect&quot; , %{ } ) Note: If you use mix phx.gen.auth to generate your authentication system, lines to that effect are already present in the generated code. The generated code uses a user_token instead of referring to the user_id . Once a LiveView is disconnected, the client will attempt to reestablish the connection and re-execute the mount/3 callback. In this case, if the user is no longer logged in or it no longer has access to the current resource, mount/3 will fail and the user will be redirected. This is the same mechanism provided by Phoenix.Channel s. Therefore, if your application uses both channels and LiveViews, you can use the same technique to disconnect any stateful connection.","ref":"security-model.html#disconnecting-all-instances-of-a-live-user","title":"Security considerations of the LiveView model - Disconnecting all instances of a live user","type":"extras"},{"doc":"LiveView supports live redirect, which allows users to navigate between pages over the LiveView connection. Whenever there is a live_redirect , a new LiveView will be mounted, skipping the regular HTTP requests and without going through the plug pipeline. However, if you want to draw stronger boundaries between parts of your application, you can also use Phoenix.LiveView.Router.live_session/2 to group your live routes. This can be handy because you can only live_redirect between LiveViews in the same live_session . For example, imagine you need to authenticate two distinct types of users. Your regular users login via email and password, and you have an admin dashboard that uses http auth. You can specify different live_session s for each authentication flow: live_session :default do scope &quot;/&quot; do pipe_through [ :authenticate_user ] get ... live ... end end live_session :admin do scope &quot;/admin&quot; do pipe_through [ :http_auth_admin ] get ... live ... end end Now every time you try to navigate to an admin panel, and out of it, a regular page navigation will happen and a brand new live connection will be established. Once again, it is worth remembering that LiveViews require their own security checks, so we use pipe_through above to protect the regular routes (get, post, etc.) and the LiveViews should run their own checks using on_mount hooks. live_session can also be used to enforce each LiveView group has a different root layout, since layouts are not updated between live redirects: live_session :default , root_layout : { LayoutView , &quot;app.html&quot; } do ... end live_session :admin , root_layout : { LayoutView , &quot;admin.html&quot; } do ... end Finally, you can even combine live_session with on_mount . Instead of declaring on_mount on every LiveView, you can declare it at the router level and it will enforce it on all LiveViews under it: live_session :default , on_mount : MyAppWeb.UserLiveAuth do scope &quot;/&quot; do pipe_through [ :authenticate_user ] live ... end end live_session :admin , on_mount : MyAppWeb.AdminLiveAuth do scope &quot;/admin&quot; do pipe_through [ :authenticate_admin ] live ... end end Each live route under the :default live_session will invoke the MyAppWeb.UserLiveAuth hook on mount. This module was defined earlier in this guide. We will also pipe regular web requests through :authenticate_user , which must execute the same checks as MyAppWeb.UserLiveAuth , but tailored to plug. Similarly, the :admin live_session has its own authentication flow, powered by MyAppWeb.AdminLiveAuth . It also defines a plug equivalent named :authenticate_admin , which will be used by any regular request. If there are no regular web requests defined under a live session, then the pipe_through checks are not necessary. Declaring the on_mount on live_session is exactly the same as declaring it in each LiveView inside the live_session . It will be executed every time a LiveView is mounted, even after live_redirect s. The important concepts to keep in mind are: If you have both LiveViews and regular web requests, then you must always authorize and authenticate your LiveViews (using on mount hooks) and your web requests (using plugs) All actions (events) must also be explicitly authorized by checking permissions. Those permissions are often domain/business specific, and typically happen in your context modules live_session can be used to draw boundaries between groups of LiveViews. While you could use live_session to draw lines between different authorization rules, doing so would lead to frequent page reloads. For this reason, we typically use live_session to enforce different authentication requirements or whenever you need to change root layouts","ref":"security-model.html#live_session-and-live_redirect","title":"Security considerations of the LiveView model - live_session and live_redirect","type":"extras"},{"doc":"LiveView currently exposes the following telemetry events: [:phoenix, :live_view, :mount, :start] - Dispatched by a Phoenix.LiveView immediately before mount/3 is invoked. Measurement: %{ system_time : System . monotonic_time } Metadata: %{ socket : Phoenix.LiveView.Socket . t , params : unsigned_params | :not_mounted_at_router , session : map , uri : String . t ( ) | nil } [:phoenix, :live_view, :mount, :stop] - Dispatched by a Phoenix.LiveView when the mount/3 callback completes successfully. Measurement: %{ duration : native_time } Metadata: %{ socket : Phoenix.LiveView.Socket . t , params : unsigned_params | :not_mounted_at_router , session : map , uri : String . t ( ) | nil } [:phoenix, :live_view, :mount, :exception] - Dispatched by a Phoenix.LiveView when an exception is raised in the mount/3 callback. Measurement: %{duration: native_time} Metadata: %{ socket : Phoenix.LiveView.Socket . t , kind : atom , reason : term , params : unsigned_params | :not_mounted_at_router , session : map , uri : String . t ( ) | nil } [:phoenix, :live_view, :handle_params, :start] - Dispatched by a Phoenix.LiveView immediately before handle_params/3 is invoked. Measurement: %{ system_time : System . monotonic_time } Metadata: %{ socket : Phoenix.LiveView.Socket . t , params : unsigned_params , uri : String . t ( ) } [:phoenix, :live_view, :handle_params, :stop] - Dispatched by a Phoenix.LiveView when the handle_params/3 callback completes successfully. Measurement: %{ duration : native_time } Metadata: %{ socket : Phoenix.LiveView.Socket . t , params : unsigned_params , uri : String . t ( ) } [:phoenix, :live_view, :handle_params, :exception] - Dispatched by a Phoenix.LiveView when an exception is raised in the handle_params/3 callback. Measurement: %{ duration : native_time } Metadata: %{ socket : Phoenix.LiveView.Socket . t , kind : atom , reason : term , params : unsigned_params , uri : String . t ( ) } [:phoenix, :live_view, :handle_event, :start] - Dispatched by a Phoenix.LiveView immediately before handle_event/3 is invoked. Measurement: %{ system_time : System . monotonic_time } Metadata: %{ socket : Phoenix.LiveView.Socket . t , event : String . t ( ) , params : unsigned_params } [:phoenix, :live_view, :handle_event, :stop] - Dispatched by a Phoenix.LiveView when the handle_event/3 callback completes successfully. Measurement: %{ duration : native_time } Metadata: %{ socket : Phoenix.LiveView.Socket . t , event : String . t ( ) , params : unsigned_params } [:phoenix, :live_view, :handle_event, :exception] - Dispatched by a Phoenix.LiveView when an exception is raised in the handle_event/3 callback. Measurement: %{ duration : native_time } Metadata: %{ socket : Phoenix.LiveView.Socket . t , kind : atom , reason : term , event : String . t ( ) , params : unsigned_params } [:phoenix, :live_component, :handle_event, :start] - Dispatched by a Phoenix.LiveComponent immediately before handle_event/3 is invoked. Measurement: %{ system_time : System . monotonic_time } Metadata: %{ socket : Phoenix.LiveView.Socket . t , component : atom , event : String . t ( ) , params : unsigned_params } [:phoenix, :live_component, :handle_event, :stop] - Dispatched by a Phoenix.LiveComponent when the handle_event/3 callback completes successfully. Measurement: %{ duration : native_time } Metadata: %{ socket : Phoenix.LiveView.Socket . t , component : atom , event : String . t ( ) , params : unsigned_params } [:phoenix, :live_component, :handle_event, :exception] - Dispatched by a Phoenix.LiveComponent when an exception is raised in the handle_event/3 callback. Measurement: %{ duration : native_time } Metadata: %{ socket : Phoenix.LiveView.Socket . t , kind : atom , reason : term , component : atom , event : String . t ( ) , params : unsigned_params }","ref":"telemetry.html","title":"Telemetry","type":"extras"},{"doc":"LiveView supports interactive file uploads with progress for both direct to server uploads as well as direct-to-cloud external uploads on the client.","ref":"uploads.html","title":"Uploads","type":"extras"},{"doc":"Accept specification - Define accepted file types, max number of entries, max file size, etc. When the client selects file(s), the file metadata is automatically validated against the specification. See Phoenix.LiveView.allow_upload/3 . Reactive entries - Uploads are populated in an @uploads assign in the socket. Entries automatically respond to progress, errors, cancellation, etc. Drag and drop - Use the phx-drop-target attribute to enable. See Phoenix.Component.live_file_input/1 .","ref":"uploads.html#built-in-features","title":"Uploads - Built-in Features","type":"extras"},{"doc":"You enable an upload, typically on mount, via allow_upload/3 : @impl Phoenix.LiveView def mount ( _params , _session , socket ) do { :ok , socket |&gt; assign ( :uploaded_files , [ ] ) |&gt; allow_upload ( :avatar , accept : ~w(.jpg .jpeg) , max_entries : 2 ) } end That's it for now! We will come back to the LiveView to implement some form- and upload-related callbacks later, but most of the functionality around uploads takes place in the template.","ref":"uploads.html#allow-uploads","title":"Uploads - Allow uploads","type":"extras"},{"doc":"Use the Phoenix.Component.live_file_input/1 component to render a file input for the upload: &lt;% ! -- lib / my_app_web / live / upload_live . html . heex -- %&gt; &lt; form id = &quot;upload-form&quot; phx-submit = &quot;save&quot; phx-change = &quot;validate&quot; &gt; &lt; .live_file_input upload = { @uploads . avatar } /&gt; &lt; button type = &quot;submit&quot; &gt; Upload &lt;/ button &gt; &lt;/ form &gt; Important: You must bind phx-submit and phx-change on the form. Note that while live_file_input/1 allows you to set additional attributes on the file input, many attributes such as id , accept , and multiple will be set automatically based on the allow_upload/3 spec. Reactive updates to the template will occur as the end-user interacts with the file input. Upload entries Uploads are populated in an @uploads assign in the socket. Each allowed upload contains a list of entries, irrespective of the :max_entries value in the allow_upload/3 spec. These entry structs contain all the information about an upload, including progress, client file info, errors, etc. Let's look at an annotated example: &lt;% ! -- lib / my_app_web / live / upload_live . html . heex -- %&gt; &lt;% ! -- use phx - drop - target with the upload ref to enable file drag and drop -- %&gt; &lt; section phx-drop-target = { @uploads . avatar . ref } &gt; &lt;% ! -- render each avatar entry -- %&gt; &lt;%= for entry &lt;- @uploads . avatar . entries do %&gt; &lt; article class = &quot;upload-entry&quot; &gt; &lt; figure &gt; &lt; .live_img_preview entry = { entry } /&gt; &lt; figcaption &gt; &lt;%= entry . client_name %&gt; &lt;/ figcaption &gt; &lt;/ figure &gt; &lt;% ! -- entry . progress will update automatically for in - flight entries -- %&gt; &lt; progress value = { entry . progress } max = &quot;100&quot; &gt; &lt;%= entry . progress %&gt; % &lt;/ progress &gt; &lt;% ! -- a regular click event whose handler will invoke Phoenix.LiveView . cancel_upload / 3 -- %&gt; &lt; button type = &quot;button&quot; phx-click = &quot;cancel-upload&quot; phx-value-ref = { entry . ref } aria-label = &quot;cancel&quot; &gt; &amp;times; &lt;/ button &gt; &lt;% ! -- Phoenix.Component . upload_errors / 2 returns a list of error atoms -- %&gt; &lt;%= for err &lt;- upload_errors ( @uploads . avatar , entry ) do %&gt; &lt; p class = &quot;alert alert-danger&quot; &gt; &lt;%= error_to_string ( err ) %&gt; &lt;/ p &gt; &lt;% end %&gt; &lt;/ article &gt; &lt;% end %&gt; &lt;% ! -- Phoenix.Component . upload_errors / 1 returns a list of error atoms -- %&gt; &lt;%= for err &lt;- upload_errors ( @uploads . avatar ) do %&gt; &lt; p class = &quot;alert alert-danger&quot; &gt; &lt;%= error_to_string ( err ) %&gt; &lt;/ p &gt; &lt;% end %&gt; &lt;/ section &gt; The section element in the example acts as the phx-drop-target for the :avatar upload. Users can interact with the file input or they can drop files over the element to add new entries. Upload entries are created when a file is added to the form input and each will exist until it has been consumed, following a successfully completed upload. Entry validation Validation occurs automatically based on any conditions that were specified in allow_upload/3 however, as mentioned previously you are required to bind phx-change on the form in order for the validation to be performed. Therefore you must implement at least a minimal callback: @impl Phoenix.LiveView def handle_event ( &quot;validate&quot; , _params , socket ) do { :noreply , socket } end Entries for files that do not match the allow_upload/3 spec will contain errors. Use Phoenix.Component.upload_errors/2 and your own helper function to render a friendly error message: def error_to_string ( :too_large ) , do : &quot;Too large&quot; def error_to_string ( :not_accepted ) , do : &quot;You have selected an unacceptable file type&quot; For error messages that affect all entries, use Phoenix.Component.upload_errors/1 , and your own helper function to render a friendly error message: def error_to_string ( :too_many_files ) , do : &quot;You have selected too many files&quot; Cancel an entry Upload entries may also be canceled, either programmatically or as a result of a user action. For instance, to handle the click event in the template above, you could do the following: @impl Phoenix.LiveView def handle_event ( &quot;cancel-upload&quot; , %{ &quot;ref&quot; =&gt; ref } , socket ) do { :noreply , cancel_upload ( socket , :avatar , ref ) } end","ref":"uploads.html#render-reactive-elements","title":"Uploads - Render reactive elements","type":"extras"},{"doc":"When the end-user submits a form containing a live_file_input/1 , the JavaScript client first uploads the file(s) before invoking the callback for the form's phx-submit event. Within the callback for the phx-submit event, you invoke the Phoenix.LiveView.consume_uploaded_entries/3 function to process the completed uploads, persisting the relevant upload data alongside the form data: @impl Phoenix.LiveView def handle_event ( &quot;save&quot; , _params , socket ) do uploaded_files = consume_uploaded_entries ( socket , :avatar , fn %{ path : path } , _entry -&gt; dest = Path . join ( [ :code . priv_dir ( :my_app ) , &quot;static&quot; , &quot;uploads&quot; , Path . basename ( path ) ] ) # The `static/uploads` directory must exist for `File.cp!/2` # and MyAppWeb.static_paths/0 should contain uploads to work,. File . cp! ( path , dest ) { :ok , ~p&quot;/uploads/ \#{ Path . basename ( dest ) } &quot; } end ) { :noreply , update ( socket , :uploaded_files , &amp; ( &amp;1 ++ uploaded_files ) ) } end Note : While client metadata cannot be trusted, max file size validations are enforced as each chunk is received when performing direct to server uploads. For more information on implementing client-side, direct-to-cloud uploads, see the External Uploads guide .","ref":"uploads.html#consume-uploaded-entries","title":"Uploads - Consume uploaded entries","type":"extras"},{"doc":"A complete example of the LiveView from this guide: # lib/my_app_web/live/upload_live.ex defmodule MyAppWeb.UploadLive do use MyAppWeb , :live_view @impl Phoenix.LiveView def mount ( _params , _session , socket ) do { :ok , socket |&gt; assign ( :uploaded_files , [ ] ) |&gt; allow_upload ( :avatar , accept : ~w(.jpg .jpeg) , max_entries : 2 ) } end @impl Phoenix.LiveView def handle_event ( &quot;validate&quot; , _params , socket ) do { :noreply , socket } end @impl Phoenix.LiveView def handle_event ( &quot;cancel-upload&quot; , %{ &quot;ref&quot; =&gt; ref } , socket ) do { :noreply , cancel_upload ( socket , :avatar , ref ) } end @impl Phoenix.LiveView def handle_event ( &quot;save&quot; , _params , socket ) do uploaded_files = consume_uploaded_entries ( socket , :avatar , fn %{ path : path } , _entry -&gt; dest = Path . join ( [ :code . priv_dir ( :my_app ) , &quot;static&quot; , &quot;uploads&quot; , Path . basename ( path ) ] ) File . cp! ( path , dest ) { :ok , ~p&quot;/uploads/ \#{ Path . basename ( dest ) } &quot; } end ) { :noreply , update ( socket , :uploaded_files , &amp; ( &amp;1 ++ uploaded_files ) ) } end defp error_to_string ( :too_large ) , do : &quot;Too large&quot; defp error_to_string ( :too_many_files ) , do : &quot;You have selected too many files&quot; defp error_to_string ( :not_accepted ) , do : &quot;You have selected an unacceptable file type&quot; end","ref":"uploads.html#appendix-a-uploadlive","title":"Uploads - Appendix A: UploadLive","type":"extras"},{"doc":"For internationalization with gettext , you must call Gettext.put_locale/2 on the LiveView mount callback to instruct the LiveView which locale should be used for rendering the page. However, one question that has to be answered is how to retrieve the locale in the first place. There are many approaches to solve this problem: The locale could be stored in the URL as a parameter The locale could be stored in the session The locale could be stored in the database We will briefly cover these approaches to provide some direction.","ref":"using-gettext.html","title":"Using Gettext for internationalization","type":"extras"},{"doc":"You can say all URLs have a locale parameter. In your router: scope &quot;/:locale&quot; do live ... get ... end Accessing a page without a locale should automatically redirect to a URL with locale (the best locale could be fetched from HTTP headers, which is outside of the scope of this guide). Then, assuming all URLs have a locale, you can set the Gettext locale accordingly: def mount ( %{ &quot;locale&quot; =&gt; locale } , _session , socket ) do Gettext . put_locale ( MyApp.Gettext , locale ) { :ok , socket } end You can also use the on_mount hook to automatically restore the locale for every LiveView in your application: defmodule MyAppWeb.RestoreLocale do def on_mount ( :default , %{ &quot;locale&quot; =&gt; locale } , _session , socket ) do Gettext . put_locale ( MyApp.Gettext , locale ) { :cont , socket } end # catch-all case def on_mount ( :default , _params , _session , socket ) , do : { :cont , socket } end Then, add this hook to def live_view under MyAppWeb , to run it on all LiveViews by default: def live_view do quote do use Phoenix.LiveView , layout : { MyAppWeb.LayoutView , :live } on_mount MyAppWeb.RestoreLocale unquote ( view_helpers ( ) ) end end Note that, because the Gettext locale is not stored in the assigns, if you want to change the locale, you must use &lt;.link navigate={...} /&gt; , instead of simply patching the page.","ref":"using-gettext.html#locale-from-parameters","title":"Using Gettext for internationalization - Locale from parameters","type":"extras"},{"doc":"You may also store the locale in the Plug session. For example, in a controller you might do: def put_user_session ( conn , current_user ) do Gettext . put_locale ( MyApp.Gettext , locale ) conn |&gt; put_session ( :user_id , current_user . id ) |&gt; put_session ( :locale , current_user . locale ) end and then restore the locale from session within your LiveView mount: def mount ( _params , %{ &quot;locale&quot; =&gt; locale } , socket ) do Gettext . put_locale ( MyApp.Gettext , locale ) { :ok , socket } end You can also encapsulate this in a hook, as done in the previous section. However, if the locale is stored in the session, you can only change it by using regular controller requests. Therefore you should always use &lt;.link to={...} /&gt; to point to a controller that change the session accordingly, reloading any LiveView.","ref":"using-gettext.html#locale-from-session","title":"Using Gettext for internationalization - Locale from session","type":"extras"},{"doc":"You may also allow users to store their locale configuration in the database. Then, on mount/3 , you can retrieve the user id from the session and load the locale: def mount ( _params , %{ &quot;user_id&quot; =&gt; user_id } , socket ) do user = Users . get_user! ( user_id ) Gettext . put_locale ( MyApp.Gettext , user . locale ) { :ok , socket } end In practice, you may end-up mixing more than one approach listed here. For example, reading from the database is great once the user is logged in but, before that happens, you may need to store the locale in the session or in the URL. Similarly, you can keep the locale in the URL, but change the URL accordingly to the user preferred locale once they sign in. Hopefully this guide gives some suggestions on how to move forward and explore the best approach for your application.","ref":"using-gettext.html#locale-from-database","title":"Using Gettext for internationalization - Locale from database","type":"extras"},{"doc":"Phoenix supports DOM element bindings for client-server interaction. For example, to react to a click on a button, you would render the element: &lt; button phx - click = &quot;inc_temperature&quot; &gt; + &lt; / button &gt; Then on the server, all LiveView bindings are handled with the handle_event callback, for example: def handle_event ( &quot;inc_temperature&quot; , _value , socket ) do { :ok , new_temp } = Thermostat . inc_temperature ( socket . assigns . id ) { :noreply , assign ( socket , :temperature , new_temp ) } end Binding Attributes Params phx-value-* Click Events phx-click , phx-click-away Form Events phx-change , phx-submit , phx-feedback-for , phx-disable-with , phx-trigger-action , phx-auto-recover Focus Events phx-blur , phx-focus , phx-window-blur , phx-window-focus Key Events phx-keydown , phx-keyup , phx-window-keydown , phx-window-keyup , phx-key DOM Patching phx-mounted , phx-update , phx-remove JS Interop phx-hook Lifecycle Events phx-mounted , phx-disconnected , phx-connected Rate Limiting phx-debounce , phx-throttle Static tracking phx-track-static","ref":"bindings.html","title":"Bindings","type":"extras"},{"doc":"The phx-click binding is used to send click events to the server. When any client event, such as a phx-click click is pushed, the value sent to the server will be chosen with the following priority: The :value specified in Phoenix.LiveView.JS.push/3 , such as: &lt; div phx-click = { JS . push ( &quot;inc&quot; , value : %{ myvar1 : @val1 } ) } &gt; Any number of optional phx-value- prefixed attributes, such as: &lt; div phx - click = &quot;inc&quot; phx - value - myvar1 = &quot;val1&quot; phx - value - myvar2 = &quot;val2&quot; &gt; will send the following map of params to the server: def handle_event ( &quot;inc&quot; , %{ &quot;myvar1&quot; =&gt; &quot;val1&quot; , &quot;myvar2&quot; =&gt; &quot;val2&quot; } , socket ) do If the phx-value- prefix is used, the server payload will also contain a &quot;value&quot; if the element's value attribute exists. The payload will also include any additional user defined metadata of the client event. For example, the following LiveSocket client option would send the coordinates and altKey information for all clicks: let liveSocket = new LiveSocket ( &quot;/live&quot; , Socket , { params : { _csrf_token : csrfToken } , metadata : { click : ( e , el ) =&gt; { return { altKey : e . altKey , clientX : e . clientX , clientY : e . clientY } } } } ) The phx-click-away event is fired when a click event happens outside of the element. This is useful for hiding toggled containers like drop-downs.","ref":"bindings.html#click-events","title":"Bindings - Click Events","type":"extras"},{"doc":"Focus and blur events may be bound to DOM elements that emit such events, using the phx-blur , and phx-focus bindings, for example: &lt; input name = &quot;email&quot; phx-focus = &quot;myfocus&quot; phx-blur = &quot;myblur&quot; /&gt; To detect when the page itself has received focus or blur, phx-window-focus and phx-window-blur may be specified. These window level events may also be necessary if the element in consideration (most often a div with no tabindex) cannot receive focus. Like other bindings, phx-value-* can be provided on the bound element, and those values will be sent as part of the payload. For example: &lt; div class = &quot;container&quot; phx-window-focus = &quot;page-active&quot; phx-window-blur = &quot;page-inactive&quot; phx-value-page = &quot;123&quot; &gt; ... &lt;/ div &gt;","ref":"bindings.html#focus-and-blur-events","title":"Bindings - Focus and Blur Events","type":"extras"},{"doc":"The onkeydown , and onkeyup events are supported via the phx-keydown , and phx-keyup bindings. Each binding supports a phx-key attribute, which triggers the event for the specific key press. If no phx-key is provided, the event is triggered for any key press. When pushed, the value sent to the server will contain the &quot;key&quot; that was pressed, plus any user-defined metadata. For example, pressing the Escape key looks like this: %{ &quot;key&quot; =&gt; &quot;Escape&quot; } To capture additional user-defined metadata, the metadata option for keydown events may be provided to the LiveSocket constructor. For example: let liveSocket = new LiveSocket ( &quot;/live&quot; , Socket , { params : { _csrf_token : csrfToken } , metadata : { keydown : ( e , el ) =&gt; { return { key : e . key , metaKey : e . metaKey , repeat : e . repeat } } } } ) To determine which key has been pressed you should use key value. The available options can be found on MDN or via the Key Event Viewer . Note : it is possible for certain browser features like autofill to trigger key events with no &quot;key&quot; field present in the value map sent to the server. For this reason, we recommend always having a fallback catch-all event handler for LiveView key bindings. By default, the bound element will be the event listener, but a window-level binding may be provided via phx-window-keydown or phx-window-keyup , for example: def render ( assigns ) do ~H&quot;&quot;&quot; &lt;div id=&quot;thermostat&quot; phx-window-keyup=&quot;update_temp&quot;&gt; Current temperature: &lt;%= @temperature %&gt; &lt;/div&gt; &quot;&quot;&quot; end def handle_event ( &quot;update_temp&quot; , %{ &quot;key&quot; =&gt; &quot;ArrowUp&quot; } , socket ) do { :ok , new_temp } = Thermostat . inc_temperature ( socket . assigns . id ) { :noreply , assign ( socket , :temperature , new_temp ) } end def handle_event ( &quot;update_temp&quot; , %{ &quot;key&quot; =&gt; &quot;ArrowDown&quot; } , socket ) do { :ok , new_temp } = Thermostat . dec_temperature ( socket . assigns . id ) { :noreply , assign ( socket , :temperature , new_temp ) } end def handle_event ( &quot;update_temp&quot; , _ , socket ) do { :noreply , socket } end","ref":"bindings.html#key-events","title":"Bindings - Key Events","type":"extras"},{"doc":"All events can be rate-limited on the client by using the phx-debounce and phx-throttle bindings, with the exception of the phx-blur binding, which is fired immediately. Rate limited and debounced events have the following behavior: phx-debounce - Accepts either an integer timeout value (in milliseconds), or &quot;blur&quot; . When an integer is provided, emitting the event is delayed by the specified milliseconds. When &quot;blur&quot; is provided, emitting the event is delayed until the field is blurred by the user. Debouncing is typically used for input elements. phx-throttle - Accepts an integer timeout value to throttle the event in milliseconds. Unlike debounce, throttle will immediately emit the event, then rate limit it at once per provided timeout. Throttling is typically used to rate limit clicks, mouse and keyboard actions. For example, to avoid validating an email until the field is blurred, while validating the username at most every 2 seconds after a user changes the field: &lt; form phx-change = &quot;validate&quot; phx-submit = &quot;save&quot; &gt; &lt; input type = &quot;text&quot; name = &quot;user[email]&quot; phx-debounce = &quot;blur&quot; /&gt; &lt; input type = &quot;text&quot; name = &quot;user[username]&quot; phx-debounce = &quot;2000&quot; /&gt; &lt;/ form &gt; And to rate limit a volume up click to once every second: &lt; button phx - click = &quot;volume_up&quot; phx - throttle = &quot;1000&quot; &gt; + &lt; / button &gt; Likewise, you may throttle held-down keydown: &lt; div phx-window-keydown = &quot;keydown&quot; phx-throttle = &quot;500&quot; &gt; ... &lt;/ div &gt; Unless held-down keys are required, a better approach is generally to use phx-keyup bindings which only trigger on key up, thereby being self-limiting. However, phx-keydown is useful for games and other use cases where a constant press on a key is desired. In such cases, throttle should always be used. Debounce and Throttle special behavior The following specialized behavior is performed for forms and keydown bindings: When a phx-submit , or a phx-change for a different input is triggered, any current debounce or throttle timers are reset for existing inputs. A phx-keydown binding is only throttled for key repeats. Unique keypresses back-to-back will dispatch the pressed key events.","ref":"bindings.html#rate-limiting-events-with-debounce-and-throttle","title":"Bindings - Rate limiting events with Debounce and Throttle","type":"extras"},{"doc":"LiveView bindings support a JavaScript command interface via the Phoenix.LiveView.JS module, which allows you to specify utility operations that execute on the client when firing phx- binding events, such as phx-click , phx-change , etc. Commands compose together to allow you to push events, add classes to elements, transition elements in and out, and more. See the Phoenix.LiveView.JS documentation for full usage. For a small example of what's possible, imagine you want to show and hide a modal on the page without needing to make the round trip to the server to render the content: &lt; div id = &quot;modal&quot; class = &quot;modal&quot; &gt; My Modal &lt;/ div &gt; &lt; button phx-click = { JS . show ( to : &quot;#modal&quot; , transition : &quot;fade-in&quot; ) } &gt; show modal &lt;/ button &gt; &lt; button phx-click = { JS . hide ( to : &quot;#modal&quot; , transition : &quot;fade-out&quot; ) } &gt; hide modal &lt;/ button &gt; &lt; button phx-click = { JS . toggle ( to : &quot;#modal&quot; , in : &quot;fade-in&quot; , out : &quot;fade-out&quot; ) } &gt; toggle modal &lt;/ button &gt; Or if your UI library relies on classes to perform the showing or hiding: &lt; div id = &quot;modal&quot; class = &quot;modal&quot; &gt; My Modal &lt;/ div &gt; &lt; button phx-click = { JS . add_class ( &quot;show&quot; , to : &quot;#modal&quot; , transition : &quot;fade-in&quot; ) } &gt; show modal &lt;/ button &gt; &lt; button phx-click = { JS . remove_class ( &quot;show&quot; , to : &quot;#modal&quot; , transition : &quot;fade-out&quot; ) } &gt; hide modal &lt;/ button &gt; Commands compose together. For example, you can push an event to the server and immediately hide the modal on the client: &lt; div id = &quot;modal&quot; class = &quot;modal&quot; &gt; My Modal &lt;/ div &gt; &lt; button phx-click = { JS . push ( &quot;modal-closed&quot; ) |&gt; JS . remove_class ( &quot;show&quot; , to : &quot;#modal&quot; , transition : &quot;fade-out&quot; ) } &gt; hide modal &lt;/ button &gt; It is also useful to extract commands into their own functions: alias Phoenix.LiveView.JS def hide_modal ( js \\\\ % JS { } , selector ) do js |&gt; JS . push ( &quot;modal-closed&quot; ) |&gt; JS . remove_class ( &quot;show&quot; , to : selector , transition : &quot;fade-out&quot; ) end &lt; button phx-click = { hide_modal ( &quot;#modal&quot; ) } &gt; hide modal &lt;/ button &gt; The Phoenix.LiveView.JS.push/3 command is particularly powerful in allowing you to customize the event being pushed to the server. For example, imagine you start with a familiar phx-click which pushes a message to the server when clicked: &lt; button phx-click = &quot;clicked&quot; &gt; click &lt;/ button &gt; Now imagine you want to customize what happens when the &quot;clicked&quot; event is pushed, such as which component should be targeted, which element should receive css loading state classes, etc. This can be accomplished with options on the JS push command. For example: &lt; button phx-click = { JS . push ( &quot;clicked&quot; , target : @myself , loading : &quot;.container&quot; ) } &gt; click &lt;/ button &gt; See Phoenix.LiveView.JS.push/3 for all supported options.","ref":"bindings.html#js-commands","title":"Bindings - JS Commands","type":"extras"},{"doc":"LiveView supports the phx-mounted , phx-connected , and phx-disconnected events to react to different lifecycle events with JS commands. To execute commands when an element first appears on the page, you can leverage phx-mounted , such as to animate a notice into view: &lt; div id = &quot;flash&quot; class = &quot;hidden&quot; phx-mounted = { JS . show ( transition : ... ) } &gt; Welcome back! &lt;/ div &gt; If phx-mounted is used on the initial page render, it will be invoked only after the initial WebSocket connection is established. To manage the connection lifecycle, you can combine phx-disconnected and phx-connected to show an element when the LiveView has lost its connection, and hide it when the connection recovers: &lt; div id = &quot;status&quot; class = &quot;hidden&quot; phx-disconnected = { JS . show ( ) } phx-connected = { JS . hide ( ) } &gt; Attempting to reconnect... &lt;/ div &gt; LiveView vs static view phx-connected and phx-disconnected are only executed when operating inside a LiveView container. For static templates, they will have no effect. For LiveView, the phx-mounted binding is executed as soon as the LiveView is mounted with a connection. When using phx-mounted in static views, it is executed as soon as the DOM is ready.","ref":"bindings.html#lifecycle-events","title":"Bindings - Lifecycle Events","type":"extras"},{"doc":"The lv: event prefix supports LiveView specific features that are handled by LiveView without calling the user's handle_event/3 callbacks. Today, the following events are supported: lv:clear-flash – clears the flash when sent to the server. If a phx-value-key is provided, the specific key will be removed from the flash. For example: &lt; p class = &quot;alert&quot; phx-click = &quot;lv:clear-flash&quot; phx-value-key = &quot;info&quot; &gt; &lt;%= live_flash ( @flash , :info ) %&gt; &lt;/ p &gt;","ref":"bindings.html#liveview-specific-events","title":"Bindings - LiveView Specific Events","type":"extras"},{"doc":"All phx- event bindings apply their own css classes when pushed. For example the following markup: &lt; button phx-click = &quot;clicked&quot; phx-window-keydown = &quot;key&quot; &gt; ... &lt;/ button &gt; On click, would receive the phx-click-loading class, and on keydown would receive the phx-keydown-loading class. The css loading classes are maintained until an acknowledgement is received on the client for the pushed event. In the case of forms, when a phx-change is sent to the server, the input element which emitted the change receives the phx-change-loading class, along with the parent form tag. The following events receive css loading classes: phx-click - phx-click-loading phx-change - phx-change-loading phx-submit - phx-submit-loading phx-focus - phx-focus-loading phx-blur - phx-blur-loading phx-window-keydown - phx-keydown-loading phx-window-keyup - phx-keyup-loading Additionally, the following classes are applied to the LiveView's parent container: &quot;phx-connected&quot; - applied when the view has connected to the server &quot;phx-loading&quot; - applied when the view is not connected to the server &quot;phx-error&quot; - applied when an error occurs on the server. Note, this class will be applied in conjunction with &quot;phx-loading&quot; if connection to the server is lost. For navigation related loading states (both automatic and manual), see phx-page-loading as described in JavaScript interoperability: Live navigation events .","ref":"bindings.html#loading-states-and-errors","title":"Bindings - Loading states and errors","type":"extras"},{"doc":"","ref":"form-bindings.html","title":"Form bindings","type":"extras"},{"doc":"To handle form changes and submissions, use the phx-change and phx-submit events. In general, it is preferred to handle input changes at the form level, where all form fields are passed to the LiveView's callback given any single input change. For example, to handle real-time form validation and saving, your form would use both phx-change and phx-submit bindings. Let's get started with an example: &lt; .form for = { @form } phx-change = &quot;validate&quot; phx-submit = &quot;save&quot; &gt; &lt; .input type = &quot;text&quot; field = { @form [ :username ] } /&gt; &lt; .input type = &quot;email&quot; field = { @form [ :email ] } /&gt; &lt; button &gt; Save &lt;/ button &gt; &lt;/ .form &gt; .form is the function component defined in Phoenix.Component.form/1 , we recommend reading its documentation for more details on how it works and all supported options. .form expects a @form assign, which can be created from a changeset or user parameters via Phoenix.Component.to_form/1 . input/1 is a function component for rendering inputs, most often defined in your own application, often encapsulating labelling, error handling, and more. Here is a simple version to get started with: attr :field , Phoenix.HTML.FormField attr :rest , include : ~w(type) def input ( assigns ) do ~H&quot;&quot;&quot; &lt;input id={@field.id} name={@field.name} value={@field.value} {@rest} /&gt; &quot;&quot;&quot; end The CoreComponents module {.info} If your application was generated with Phoenix v1.7, then mix phx.new automatically imports many ready-to-use function components, such as .input component with built-in features and styles. With the form rendered, your LiveView picks up the events in handle_event callbacks, to validate and attempt to save the parameter accordingly: def render ( assigns ) ... def mount ( _params , _session , socket ) do { :ok , assign ( socket , form : to_form ( Accounts . change_user ( % User { } ) ) } end def handle_event ( &quot;validate&quot; , %{ &quot;user&quot; =&gt; params } , socket ) do form = % User { } |&gt; Accounts . change_user ( params ) |&gt; Map . put ( :action , :insert ) |&gt; to_form ( ) { :noreply , assign ( socket , form : form ) } end def handle_event ( &quot;save&quot; , %{ &quot;user&quot; =&gt; user_params } , socket ) do case Accounts . create_user ( user_params ) do { :ok , user } -&gt; { :noreply , socket |&gt; put_flash ( :info , &quot;user created&quot; ) |&gt; redirect ( to : ~p&quot;/users/ \#{ user } &quot; ) } { :error , % Ecto.Changeset { } = changeset } -&gt; { :noreply , assign ( socket , form : to_form ( changeset ) ) } end end The validate callback simply updates the changeset based on all form input values, then convert the changeset to a form and assign it to the socket. If the form changes, such as generating new errors, render/1 is invoked and the form is re-rendered. Likewise for phx-submit bindings, the same callback is invoked and persistence is attempted. On success, a :noreply tuple is returned and the socket is annotated for redirect with Phoenix.LiveView.redirect/2 to the new user page, otherwise the socket assigns are updated with the errored changeset to be re-rendered for the client. You may wish for an individual input to use its own change event or to target a different component. This can be accomplished by annotating the input itself with phx-change , for example: &lt; . form for = { @form } phx - change = &quot;validate&quot; phx - submit = &quot;save&quot; &gt; ... &lt; . input field = { @form [ :email ] } phx - change = &quot;email_changed&quot; phx - target = { @myself } / &gt; &lt; / . form &gt; Then your LiveView or LiveComponent would handle the event: def handle_event ( &quot;email_changed&quot; , %{ &quot;user&quot; =&gt; %{ &quot;email&quot; =&gt; email } } , socket ) do ... end Note : only the individual input is sent as params for an input marked with phx-change .","ref":"form-bindings.html#form-events","title":"Form bindings - Form Events","type":"extras"},{"doc":"For proper form error tag updates, the error tag must specify which input it belongs to. This is accomplished with the phx-feedback-for attribute, which specifies the name (or id, for backwards compatibility) of the input it belongs to. Failing to add the phx-feedback-for attribute will result in displaying error messages for form fields that the user has not changed yet (e.g. required fields further down on the page). For example, your MyAppWeb.CoreComponents may use this function: def input ( assigns ) do ~H&quot;&quot;&quot; &lt;div phx-feedback-for={@name}&gt; &lt;input type={@type} name={@name} id={@id || @name} value={Phoenix.HTML.Form.normalize_value(@type, @value)} class={[ &quot;phx-no-feedback:border-zinc-300 phx-no-feedback:focus:border-zinc-400&quot;, &quot;border-zinc-300 focus:border-zinc-400 focus:ring-zinc-800/5&quot;, ]} {@rest} /&gt; &lt;.error :for={msg &lt;- @errors}&gt;&lt;%%= msg %&gt;&lt;/.error&gt; &lt;/div&gt; &quot;&quot;&quot; end def error ( assigns ) do ~H&quot;&quot;&quot; &lt;p class=&quot;phx-no-feedback:hidden&quot;&gt; &lt;Heroicons.exclamation_circle mini class=&quot;mt-0.5 h-5 w-5 flex-none fill-rose-500&quot; /&gt; &lt;%= render_slot(@inner_block) %&gt; &lt;/p&gt; &quot;&quot;&quot; end Now, any DOM container with the phx-feedback-for attribute will receive a phx-no-feedback class in cases where the form fields has yet to receive user input/focus. Using new CSS rules or tailwindcss variants allows you errors to be shown, hidden, and styled as feedback changes.","ref":"form-bindings.html#phx-feedback-for","title":"Form bindings - phx-feedback-for","type":"extras"},{"doc":"Number inputs are a special case in LiveView forms. On programmatic updates, some browsers will clear invalid inputs. So LiveView will not send change events from the client when an input is invalid, instead allowing the browser's native validation UI to drive user interaction. Once the input becomes valid, change and submit events will be sent normally. &lt; input type = &quot;number&quot; &gt; This is known to have a plethora of problems including accessibility, large numbers are converted to exponential notation, and scrolling can accidentally increase or decrease the number. One alternative is the inputmode attribute, which may serve your application's needs and users much better. According to Can I Use? , the following is supported by 86% of the global market (as of Sep 2021): &lt; input type = &quot;text&quot; inputmode = &quot;numeric&quot; pattern = &quot;[0-9]*&quot; &gt;","ref":"form-bindings.html#number-inputs","title":"Form bindings - Number inputs","type":"extras"},{"doc":"Password inputs are also special cased in Phoenix.HTML . For security reasons, password field values are not reused when rendering a password input tag. This requires explicitly setting the :value in your markup, for example: &lt; .input field = { f [ :password ] } value = { input_value ( f [ :password ] . value ) } /&gt; &lt; .input field = { f [ :password_confirmation ] } value = { input_value ( f [ :password_confirmation ] . value ) } /&gt;","ref":"form-bindings.html#password-inputs","title":"Form bindings - Password inputs","type":"extras"},{"doc":"Nested inputs are handled using .inputs_for function component. By default it will add the necessary hidden input fields for tracking ids of Ecto associations. &lt; .inputs_for :let = { fp } field = { f [ :friends ] } &gt; &lt; .input field = { fp [ :name ] } type = &quot;text&quot; &gt; &lt;/ .inputs_for &gt;","ref":"form-bindings.html#nested-inputs","title":"Form bindings - Nested inputs","type":"extras"},{"doc":"LiveView forms support reactive file inputs , including drag and drop support via the phx-drop-target attribute: &lt; div class = &quot;container&quot; phx-drop-target = { @uploads . avatar . ref } &gt; ... &lt; .live_file_input upload = { @uploads . avatar } /&gt; &lt;/ div &gt; See Phoenix.Component.live_file_input/1 for more.","ref":"form-bindings.html#file-inputs","title":"Form bindings - File inputs","type":"extras"},{"doc":"The phx-trigger-action attribute can be added to a form to trigger a standard form submit on DOM patch to the URL specified in the form's standard action attribute. This is useful to perform pre-final validation of a LiveView form submit before posting to a controller route for operations that require Plug session mutation. For example, in your LiveView template you can annotate the phx-trigger-action with a boolean assign: &lt; .form :let = { f } for = { @changeset } action = { Routes . reset_password_path ( @socket , :create ) } phx-submit = &quot;save&quot; phx-trigger-action = { @trigger_submit } &gt; Then in your LiveView, you can toggle the assign to trigger the form with the current fields on next render: def handle_event ( &quot;save&quot; , params , socket ) do case validate_change_password ( socket . assigns . user , params ) do { :ok , changeset } -&gt; { :noreply , assign ( socket , changeset : changeset , trigger_submit : true ) } { :error , changeset } -&gt; { :noreply , assign ( socket , changeset : changeset ) } end end Once phx-trigger-action is true, LiveView disconnects and then submits the form.","ref":"form-bindings.html#submitting-the-form-action-over-http","title":"Form bindings - Submitting the form action over HTTP","type":"extras"},{"doc":"By default, all forms marked with phx-change and having id attribute will recover input values automatically after the user has reconnected or the LiveView has remounted after a crash. This is achieved by the client triggering the same phx-change to the server as soon as the mount has been completed. Note: if you want to see form recovery working in development, please make sure to disable live reloading in development by commenting out the LiveReload plug in your endpoint.ex file or by setting code_reloader: false in your config/dev.exs . Otherwise live reloading may cause the current page to be reloaded whenever you restart the server, which will discard all form state. For most use cases, this is all you need and form recovery will happen without consideration. In some cases, where forms are built step-by-step in a stateful fashion, it may require extra recovery handling on the server outside of your existing phx-change callback code. To enable specialized recovery, provide a phx-auto-recover binding on the form to specify a different event to trigger for recovery, which will receive the form params as usual. For example, imagine a LiveView wizard form where the form is stateful and built based on what step the user is on and by prior selections: &lt; form id = &quot;wizard&quot; phx - change = &quot;validate_wizard_step&quot; phx - auto - recover = &quot;recover_wizard&quot; &gt; On the server, the &quot;validate_wizard_step&quot; event is only concerned with the current client form data, but the server maintains the entire state of the wizard. To recover in this scenario, you can specify a recovery event, such as &quot;recover_wizard&quot; above, which would wire up to the following server callbacks in your LiveView: def handle_event ( &quot;validate_wizard_step&quot; , params , socket ) do # regular validations for current step { :noreply , socket } end def handle_event ( &quot;recover_wizard&quot; , params , socket ) do # rebuild state based on client input data up to the current step { :noreply , socket } end To forgo automatic form recovery, set phx-auto-recover=&quot;ignore&quot; .","ref":"form-bindings.html#recovery-following-crashes-or-disconnects","title":"Form bindings - Recovery following crashes or disconnects","type":"extras"},{"doc":"The JavaScript client is always the source of truth for current input values. For any given input with focus, LiveView will never overwrite the input's current value, even if it deviates from the server's rendered updates. This works well for updates where major side effects are not expected, such as form validation errors, or additive UX around the user's input values as they fill out a form. For these use cases, the phx-change input does not concern itself with disabling input editing while an event to the server is in flight. When a phx-change event is sent to the server, the input tag and parent form tag receive the phx-change-loading CSS class, then the payload is pushed to the server with a &quot;_target&quot; param in the root payload containing the keyspace of the input name which triggered the change event. For example, if the following input triggered a change event: &lt; input name = &quot;user[username]&quot; /&gt; The server's handle_event/3 would receive a payload: %{ &quot;_target&quot; =&gt; [ &quot;user&quot; , &quot;username&quot; ] , &quot;user&quot; =&gt; %{ &quot;username&quot; =&gt; &quot;Name&quot; } } The phx-submit event is used for form submissions where major side effects typically happen, such as rendering new containers, calling an external service, or redirecting to a new page. On submission of a form bound with a phx-submit event: The form's inputs are set to readonly Any submit button on the form is disabled The form receives the &quot;phx-submit-loading&quot; class On completion of server processing of the phx-submit event: The submitted form is reactivated and loses the &quot;phx-submit-loading&quot; class The last input with focus is restored (unless another input has received focus) Updates are patched to the DOM as usual To handle latent events, the &lt;button&gt; tag of a form can be annotated with phx-disable-with , which swaps the element's innerText with the provided value during event submission. For example, the following code would change the &quot;Save&quot; button to &quot;Saving...&quot;, and restore it to &quot;Save&quot; on acknowledgment: &lt; button type = &quot;submit&quot; phx-disable-with = &quot;Saving...&quot; &gt; Save &lt;/ button &gt; You may also take advantage of LiveView's CSS loading state classes to swap out your form content while the form is submitting. For example, with the following rules in your app.css : . while - submitting { display : none ; } . inputs { display : block ; } . phx - submit - loading . while - submitting { display : block ; } . phx - submit - loading . inputs { display : none ; } You can show and hide content with the following markup: &lt; form phx-change = &quot;update&quot; &gt; &lt; div class = &quot;while-submitting&quot; &gt; Please wait while we save our content... &lt;/ div &gt; &lt; div class = &quot;inputs&quot; &gt; &lt; input type = &quot;text&quot; name = &quot;text&quot; value = { @text } &gt; &lt;/ div &gt; &lt;/ form &gt; Additionally, we strongly recommend including a unique HTML &quot;id&quot; attribute on the form. When DOM siblings change, elements without an ID will be replaced rather than moved, which can cause issues such as form fields losing focus.","ref":"form-bindings.html#javascript-client-specifics","title":"Form bindings - JavaScript client specifics","type":"extras"},{"doc":"Often it is desirable to trigger an event on a DOM element without explicit user interaction on the element. For example, a custom form element such as a date picker or custom select input which utilizes a hidden input element to store the selected state. In these cases, the event functions on the DOM API can be used, for example to trigger a phx-change event: document . getElementById ( &quot;my-select&quot; ) . dispatchEvent ( new Event ( &quot;input&quot; , { bubbles : true } ) ) When using a client hook, this.el can be used to determine the element as outlined in the &quot;Client hooks&quot; documentation. It is also possible to trigger a phx-submit using a &quot;submit&quot; event: document . getElementById ( &quot;my-form&quot; ) . dispatchEvent ( new Event ( &quot;submit&quot; , { bubbles : true , cancelable : true } ) )","ref":"form-bindings.html#triggering-phx-form-events-with-javascript","title":"Form bindings - Triggering phx- form events with JavaScript","type":"extras"},{"doc":"A container can be marked with phx-update , allowing the DOM patch operations to avoid updating or removing portions of the LiveView, or to append or prepend the updates rather than replacing the existing contents. This is useful for client-side interop with existing libraries that do their own DOM operations. The following phx-update values are supported: replace - the default operation. Replaces the element with the contents stream - supports stream operations. Streams are used to manage large collections in the UI without having to store the collection on the server ignore - ignores updates to the DOM regardless of new content changes When using phx-update , a unique DOM ID must always be set in the container. If using &quot;stream&quot;, a DOM ID must also be set for each child. When inserting stream elements containing an ID already present in the container, LiveView will replace the existing element with the new content. See Phoenix.LiveView.stream/3 for more information. The &quot;ignore&quot; behaviour is frequently used when you need to integrate with another JS library. Note only the element contents are ignored, its attributes can still be updated. To react to elements being mounted to the DOM, the phx-mounted binding can be used. For example, to animate an element on mount: &lt; div phx - mounted = { JS . transition ( &quot;animate-ping&quot; , time : 500 ) } &gt; If phx-mounted is used on the initial page render, it will be invoked only after the initial WebSocket connection is established. To react to elements being removed from the DOM, the phx-remove binding may be specified, which can contain a Phoenix.LiveView.JS command to execute. Note : The phx-remove command is only executed for the removed parent element. It does not cascade to children.","ref":"dom-patching.html","title":"DOM patching & temporary assigns","type":"extras"},{"doc":"By default, all LiveView assigns are stateful, which enables change tracking and stateful interactions. In some cases, it's useful to mark assigns as temporary, meaning they will be reset to a default value after each update. This allows otherwise large but infrequently updated values to be discarded after the client has been patched. Imagine you want to implement a chat application with LiveView. You could render each message like this: &lt;%= for message &lt;- @messages do %&gt; &lt; p &gt; &lt; span &gt; &lt;%= message . username %&gt; : &lt;/ span &gt; &lt;%= message . text %&gt; &lt;/ p &gt; &lt;% end %&gt; Every time there is a new message, you would append it to the @messages assign and re-render all messages. As you may suspect, keeping the whole chat conversation in memory and resending it on every update would be too expensive, even with LiveView smart change tracking. By using temporary assigns and phx-update, we don't need to keep any messages in memory, and send messages to be appended to the UI only when there are new ones. To do so, the first step is to mark which assigns are temporary and what values they should be reset to on mount: def mount ( _params , _session , socket ) do socket = assign ( socket , :messages , load_last_20_messages ( ) ) { :ok , socket , temporary_assigns : [ messages : [ ] ] } end On mount we also load the initial number of messages we want to send. After the initial render, the initial batch of messages will be reset back to an empty list. Now, whenever there are one or more new messages, we will assign only the new messages to @messages : socket = assign ( socket , :messages , new_messages ) In the template, we want to wrap all of the messages in a container and tag this content with phx-update . Remember, we must add an ID to the container as well as to each child: &lt; div id = &quot;chat-messages&quot; phx-update = &quot;append&quot; &gt; &lt;%= for message &lt;- @messages do %&gt; &lt; p id = { message . id } &gt; &lt; span &gt; &lt;%= message . username %&gt; : &lt;/ span &gt; &lt;%= message . text %&gt; &lt;/ p &gt; &lt;% end %&gt; &lt;/ div &gt; When the client receives new messages, it now knows to append to the old content rather than replace it. You can also update the direction of messages. Suppose there is an edit to a message that is being sent to your LiveView like this: def handle_info ( { :update_message , message } , socket ) do { :noreply , update ( socket , :messages , fn messages -&gt; [ message | messages ] end ) } end You can add it to the list like you do with new messages. LiveView is aware that this message was rendered on the client, even though the message itself is discarded on the server after it is rendered. LiveView uses DOM ids to check if a message is rendered before or not. If an id is rendered before, the DOM element is updated rather than appending or prepending a new node. Also, the order of elements is not changed. You can use it to show edited messages, show likes, or anything that would require an update to a rendered message.","ref":"dom-patching.html#temporary-assigns","title":"DOM patching & temporary assigns - Temporary assigns","type":"extras"},{"doc":"To enable LiveView client/server interaction, we instantiate a LiveSocket. For example: import { Socket } from &quot;phoenix&quot; import { LiveSocket } from &quot;phoenix_live_view&quot; let csrfToken = document . querySelector ( &quot;meta[name=&#39;csrf-token&#39;]&quot; ) . getAttribute ( &quot;content&quot; ) let liveSocket = new LiveSocket ( &quot;/live&quot; , Socket , { params : { _csrf_token : csrfToken } } ) liveSocket . connect ( ) All options are passed directly to the Phoenix.Socket constructor, except for the following LiveView specific options: bindingPrefix - the prefix to use for phoenix bindings. Defaults &quot;phx-&quot; params - the connect_params to pass to the view's mount callback. May be a literal object or closure returning an object. When a closure is provided, the function receives the view's element. hooks – a reference to a user-defined hooks namespace, containing client callbacks for server/client interop. See the Client hooks section below for details. uploaders – a reference to a user-defined uploaders namespace, containing client callbacks for client-side direct-to-cloud uploads. See the External Uploads guide for details.","ref":"js-interop.html","title":"JavaScript interoperability","type":"extras"},{"doc":"To aid debugging on the client when troubleshooting issues, the enableDebug() and disableDebug() functions are exposed on the LiveSocket JavaScript instance. Calling enableDebug() turns on debug logging which includes LiveView life-cycle and payload events as they come and go from client to server. In practice, you can expose your instance on window for quick access in the browser's web console, for example: // app . js let liveSocket = new LiveSocket ( ... ) liveSocket . connect ( ) window . liveSocket = liveSocket // in the browser &#39; s web console &gt;&gt; liveSocket . enableDebug ( ) The debug state uses the browser's built-in sessionStorage , so it will remain in effect for as long as your browser session lasts.","ref":"js-interop.html#debugging-client-events","title":"JavaScript interoperability - Debugging Client Events","type":"extras"},{"doc":"Proper handling of latency is critical for good UX. LiveView's CSS loading states allow the client to provide user feedback while awaiting a server response. In development, near zero latency on localhost does not allow latency to be easily represented or tested, so LiveView includes a latency simulator with the JavaScript client to ensure your application provides a pleasant experience. Like the enableDebug() function above, the LiveSocket instance includes enableLatencySim(milliseconds) and disableLatencySim() functions which apply throughout the current browser session. The enableLatencySim function accepts an integer in milliseconds for the round-trip-time to the server. For example: // app . js let liveSocket = new LiveSocket ( ... ) liveSocket . connect ( ) window . liveSocket = liveSocket // in the browser &#39; s web console &gt;&gt; liveSocket . enableLatencySim ( 1000 ) [ Log ] latency simulator enabled for the duration of this browser session . Call disableLatencySim ( ) to disable","ref":"js-interop.html#simulating-latency","title":"JavaScript interoperability - Simulating Latency","type":"extras"},{"doc":"LiveView emits several events to the browsers and allows developers to submit their own events too. Live navigation events For live page navigation via &lt;.link navigate={...}&gt; and &lt;.link patch={...}&gt; , their server-side equivalents push_redirect and push_patch , as well as form submits via phx-submit , the JavaScript events &quot;phx:page-loading-start&quot; and &quot;phx:page-loading-stop&quot; are dispatched on window. Additionally, any phx- event may dispatch page loading events by annotating the DOM element with phx-page-loading . This is useful for showing main page loading status, for example: // app . js import topbar from &quot;topbar&quot; window . addEventListener ( &quot;phx:page-loading-start&quot; , info =&gt; topbar . show ( ) ) window . addEventListener ( &quot;phx:page-loading-stop&quot; , info =&gt; topbar . hide ( ) ) Within the callback, info.detail will be an object that contains a kind key, with a value that depends on the triggering event: &quot;redirect&quot; - the event was triggered by a redirect &quot;patch&quot; - the event was triggered by a patch &quot;initial&quot; - the event was triggered by initial page load &quot;element&quot; - the event was triggered by a phx- bound element, such as phx-click For all kinds of page loading events, all but &quot;element&quot; will receive an additional to key in the info metadata pointing to the href associated with the page load. In the case of an &quot;element&quot; page loading event, the info will contain a &quot;target&quot; key containing the DOM element which triggered the page loading state. Handling server-pushed events When the server uses Phoenix.LiveView.push_event/3 , the event name will be dispatched in the browser with the phx: prefix. For example, imagine the following template where you want to highlight an existing element from the server to draw the user's attention: &lt; div id = { &quot;item- \#{ item . id } &quot; } class = &quot;item&quot; &gt; &lt;%= item . title %&gt; &lt;/ div &gt; Next, the server can issue a highlight using the standard push_event : def handle_info ( { :item_updated , item } , socket ) do { :noreply , push_event ( socket , &quot;highlight&quot; , %{ id : &quot;item- \#{ item . id } &quot; } ) } end Finally, a window event listener can listen for the event and conditionally execute the highlight command if the element matches: let liveSocket = new LiveSocket ( ... ) window . addEventListener ( ` phx :highlight ` , ( e ) =&gt; { let el = document . getElementById ( e . detail . id ) if ( el ) { // logic for highlighting } } ) If you desire, you can also integrate this functionality with Phoenix' JS commands, executing JS commands for the given element whenever highlight is triggered. First, update the element to embed the JS command into a data attribute: &lt; div id = { &quot;item- \#{ item . id } &quot; } class = &quot;item&quot; data-highlight = { JS . transition ( &quot;highlight&quot; ) } &gt; &lt;%= item . title %&gt; &lt;/ div &gt; Now, in the event listener, use LiveSocket.execJS to trigger all JS commands in the new attribute: let liveSocket = new LiveSocket ( ... ) window . addEventListener ( ` phx :highlight ` , ( e ) =&gt; { document . querySelectorAll ( ` [ data - highlight ] ` ) . forEach ( el =&gt; { if ( el . id == e . detail . id ) { liveSocket . execJS ( el , el . getAttribute ( &quot;data-highlight&quot; ) ) } } ) } )","ref":"js-interop.html#event-listeners","title":"JavaScript interoperability - Event listeners","type":"extras"},{"doc":"To handle custom client-side JavaScript when an element is added, updated, or removed by the server, a hook object may be provided via phx-hook . phx-hook must point to an object with the following life-cycle callbacks: mounted - the element has been added to the DOM and its server LiveView has finished mounting beforeUpdate - the element is about to be updated in the DOM. Note : any call here must be synchronous as the operation cannot be deferred or cancelled. updated - the element has been updated in the DOM by the server destroyed - the element has been removed from the page, either by a parent update, or by the parent being removed entirely disconnected - the element's parent LiveView has disconnected from the server reconnected - the element's parent LiveView has reconnected to the server Note: When using hooks outside the context of a LiveView, mounted is the only callback invoked, and only those elements on the page at DOM ready will be tracked. For dynamic tracking of the DOM as elements are added, removed, and updated, a LiveView should be used. The above life-cycle callbacks have in-scope access to the following attributes: el - attribute referencing the bound DOM node liveSocket - the reference to the underlying LiveSocket instance pushEvent(event, payload, (reply, ref) =&gt; ...) - method to push an event from the client to the LiveView server pushEventTo(selectorOrTarget, event, payload, (reply, ref) =&gt; ...) - method to push targeted events from the client to LiveViews and LiveComponents. It sends the event to the LiveComponent or LiveView the selectorOrTarget is defined in, where its value can be either a query selector or an actual DOM element. If the query selector returns more than one element it will send the event to all of them, even if all the elements are in the same LiveComponent or LiveView. handleEvent(event, (payload) =&gt; ...) - method to handle an event pushed from the server upload(name, files) - method to inject a list of file-like objects into an uploader. uploadTo(selectorOrTarget, name, files) - method to inject a list of file-like objects into an uploader. The hook will send the files to the uploader with name defined by allow_upload/3 on the server-side. Dispatching new uploads triggers an input change event which will be sent to the LiveComponent or LiveView the selectorOrTarget is defined in, where its value can be either a query selector or an actual DOM element. If the query selector returns more than one live file input, an error will be logged. For example, the markup for a controlled input for phone-number formatting could be written like this: &lt; input type = &quot;text&quot; name = &quot;user[phone_number]&quot; id = &quot;user-phone-number&quot; phx - hook = &quot;PhoneNumber&quot; / &gt; Then a hook callback object could be defined and passed to the socket: let Hooks = { } Hooks.PhoneNumber = { mounted ( ) { this . el . addEventListener ( &quot;input&quot; , e =&gt; { let match = this . el . value . replace ( / \\ D / g , &quot;&quot; ) . match ( / ^ ( \\ d { 3 } ) ( \\ d { 3 } ) ( \\ d { 4 } ) $ / ) if ( match ) { this . el . value = ` $ { match [ 1 ] } - $ { match [ 2 ] } - $ { match [ 3 ] } ` } } ) } } let liveSocket = new LiveSocket ( &quot;/live&quot; , Socket , { hooks : Hooks , ... } ) ... Note : when using phx-hook , a unique DOM ID must always be set. For integration with client-side libraries which require a broader access to full DOM management, the LiveSocket constructor accepts a dom option with an onBeforeElUpdated callback. The fromEl and toEl DOM nodes are passed to the function just before the DOM patch operations occurs in LiveView. This allows external libraries to (re)initialize DOM elements or copy attributes as necessary as LiveView performs its own patch operations. The update operation cannot be cancelled or deferred, and the return value is ignored. For example, the following option could be used to add Alpine.js support to your project: let liveSocket = new LiveSocket ( &quot;/live&quot; , Socket , { ... , dom : { onBeforeElUpdated ( from , to ) { if ( from . _x_dataStack ) { window . Alpine . clone ( from , to ) } } } , } ) You could also use the same approach to guarantee that some attributes set on the client-side are kept intact. In the following example, all attributes starting with data-js- won't be replaced when the DOM is patched by LiveView: onBeforeElUpdated ( from , to ) { for ( const attr of from . attributes ) { if ( attr . name . startsWith ( &quot;data-js-&quot; ) ) { to . setAttribute ( attr . name , attr . value ) ; } } } Client-server communication A hook can push events to the LiveView by using the pushEvent function and receive a reply from the server via a {:reply, map, socket} return value. The reply payload will be passed to the optional pushEvent response callback. Communication with the hook from the server can be done by reading data attributes on the hook element or by using Phoenix.LiveView.push_event/3 on the server and handleEvent on the client. For example, to implement infinite scrolling, one can pass the current page using data attributes: &lt; div id = &quot;infinite-scroll&quot; phx - hook = &quot;InfiniteScroll&quot; data - page = { @page } &gt; And then in the client: Hooks.InfiniteScroll = { page ( ) { return this . el . dataset . page } , mounted ( ) { this . pending = this . page ( ) window . addEventListener ( &quot;scroll&quot; , e =&gt; { if ( this . pending == this . page ( ) &amp;&amp; scrollAt ( ) &gt; 90 ) { this . pending = this . page ( ) + 1 this . pushEvent ( &quot;load-more&quot; , { } ) } } ) } , updated ( ) { this . pending = this . page ( ) } } However, the data attribute approach is not a good approach if you need to frequently push data to the client. To push out-of-band events to the client, for example to render charting points, one could do: &lt; div id = &quot;chart&quot; phx - hook = &quot;Chart&quot; &gt; { :noreply , push_event ( socket , &quot;points&quot; , %{ points : new_points } ) } And then on the client: Hooks.Chart = { mounted ( ) { this . handleEvent ( &quot;points&quot; , ( { points } ) =&gt; MyChartLib . addPoints ( points ) ) } } Note : remember events pushed from the server via push_event are global and will be dispatched to all active hooks on the client who are handling that event. Note : In case a LiveView pushes events and renders content, handleEvent callbacks are invoked after the page is updated. Therefore, if the LiveView redirects at the same time it pushes events, callbacks won't be invoked on the old page's elements. Callbacks would be invoked on the redirected page's newly mounted hook elements.","ref":"js-interop.html#client-hooks-via-phx-hook","title":"JavaScript interoperability - Client hooks via phx-hook","type":"extras"},{"doc":"This guide continues from the configuration started in the server Uploads guide . Uploads to external cloud providers, such as Amazon S3, Google Cloud, etc., can be achieved by using the :external option in allow_upload/3 . You provide a 2-arity function to allow the server to generate metadata for each upload entry, which is passed to a user-specified JavaScript function on the client. Typically when your function is invoked, you will generate a pre-signed URL, specific to your cloud storage provider, that will provide temporary access for the end-user to upload data directly to your cloud storage.","ref":"uploads-external.html","title":"External Uploads","type":"extras"},{"doc":"For any service that supports large file uploads via chunked HTTP requests with Content-Range headers, you can use the UpChunk JS library by Mux to do all the hard work of uploading the file. You only need to wire the UpChunk instance to the LiveView UploadEntry callbacks, and LiveView will take care of the rest. Install UpChunk by saving its contents to assets/vendor/upchunk.js or by installing it with npm : $ npm install --prefix assets --save @mux/upchunk Configure your uploader on Phoenix.LiveView.mount/3 : def mount ( _params , _session , socket ) do { :ok , socket |&gt; assign ( :uploaded_files , [ ] ) |&gt; allow_upload ( :avatar , accept : :any , max_entries : 3 , external : &amp; presign_upload / 2 ) } end Supply the :external option to Phoenix.LiveView.allow_upload/3 . It requires a 2-arity function that generates a signed URL where the client will push the bytes for the upload entry. For example, if you were using a context that provided a start_session function, you might write something like this: defp presign_upload ( entry , socket ) do { :ok , %{ &quot;Location&quot; =&gt; link } } = SomeTube . start_session ( %{ &quot;uploadType&quot; =&gt; &quot;resumable&quot; , &quot;x-upload-content-length&quot; =&gt; entry . client_size } ) { :ok , %{ uploader : &quot;UpChunk&quot; , entrypoint : link } , socket } end Finally, on the client-side, we use UpChunk to create an upload from the temporary URL generated on the server and attach listeners for its events to the entry's callbacks: import * as UpChunk from &quot;@mux/upchunk&quot; let Uploaders = {} Uploaders.UpChunk = function(entries, onViewError){ entries.forEach(entry =&gt; { // create the upload session with UpChunk let { file, meta: { entrypoint } } = entry let upload = UpChunk.createUpload({ endpoint: entrypoint, file }) // stop uploading in the event of a view error onViewError(() =&gt; upload.pause()) // upload error triggers LiveView error upload.on(&quot;error&quot;, (e) =&gt; entry.error(e.detail.message)) // notify progress events to LiveView upload.on(&quot;progress&quot;, (e) =&gt; { if(e.detail &lt; 100){ entry.progress(e.detail) } }) // success completes the UploadEntry upload.on(&quot;success&quot;, () =&gt; entry.progress(100)) }) } // Don't forget to assign Uploaders to the liveSocket let liveSocket = new LiveSocket(&quot;/live&quot;, Socket, { uploaders: Uploaders, params: {_csrf_token: csrfToken} })","ref":"uploads-external.html#chunked-http-uploads","title":"External Uploads - Chunked HTTP Uploads","type":"extras"},{"doc":"In order to enforce all of your file constraints when uploading to S3, it is necessary to perform a multipart form POST with your file data. This guide assumes an existing S3 bucket with the correct CORS configuration which allows uploading directly to the bucket. An example CORS config is: [ { &quot;AllowedHeaders&quot;: [ &quot;*&quot; ], &quot;AllowedMethods&quot;: [ &quot;PUT&quot;, &quot;POST&quot; ], &quot;AllowedOrigins&quot;: [ your_domain_or_*_here ], &quot;ExposeHeaders&quot;: [] } ] More information on configuring CORS for S3 buckets is available at: https://docs.aws.amazon.com/AmazonS3/latest/userguide/ManageCorsUsing.html The following example uses a zero-dependency module called SimpleS3Upload written by Chris McCord to generate pre-signed URLs for S3. def mount ( _params , _session , socket ) do { :ok , socket |&gt; assign ( :uploaded_files , [ ] ) |&gt; allow_upload ( :avatar , accept : :any , max_entries : 3 , external : &amp; presign_upload / 2 ) } end defp presign_upload ( entry , socket ) do uploads = socket . assigns . uploads bucket = &quot;phx-upload-example&quot; key = &quot;public/ \#{ entry . client_name } &quot; config = %{ region : &quot;us-east-1&quot; , access_key_id : System . fetch_env! ( &quot;AWS_ACCESS_KEY_ID&quot; ) , secret_access_key : System . fetch_env! ( &quot;AWS_SECRET_ACCESS_KEY&quot; ) } { :ok , fields } = SimpleS3Upload . sign_form_upload ( config , bucket , key : key , content_type : entry . client_type , max_file_size : uploads [ entry . upload_config ] . max_file_size , expires_in : :timer . hours ( 1 ) ) meta = %{ uploader : &quot;S3&quot; , key : key , url : &quot;http:// \#{ bucket } .s3- \#{ config . region } .amazonaws.com&quot; , fields : fields } { :ok , meta , socket } end Here, we implemented a presign_upload/2 function, which we passed as a captured anonymous function to :external . Next, we generate a pre-signed URL for the upload. Lastly, we return our :ok result, with a payload of metadata for the client, along with our unchanged socket. The metadata must contain the :uploader key, specifying the name of the JavaScript client-side uploader, in this case &quot;S3&quot; . To complete the flow, we can implement our S3 client uploader and tell the LiveSocket where to find it: let Uploaders = {} Uploaders.S3 = function(entries, onViewError){ entries.forEach(entry =&gt; { let formData = new FormData() let {url, fields} = entry.meta Object.entries(fields).forEach(([key, val]) =&gt; formData.append(key, val)) formData.append(&quot;file&quot;, entry.file) let xhr = new XMLHttpRequest() onViewError(() =&gt; xhr.abort()) xhr.onload = () =&gt; xhr.status === 204 ? entry.progress(100) : entry.error() xhr.onerror = () =&gt; entry.error() xhr.upload.addEventListener(&quot;progress&quot;, (event) =&gt; { if(event.lengthComputable){ let percent = Math.round((event.loaded / event.total) * 100) if(percent &lt; 100){ entry.progress(percent) } } }) xhr.open(&quot;POST&quot;, url, true) xhr.send(formData) }) } let liveSocket = new LiveSocket(&quot;/live&quot;, Socket, { uploaders: Uploaders, params: {_csrf_token: csrfToken} }) We define an Uploaders.S3 function, which receives our entries. It then performs an AJAX request for each entry, using the entry.progress() and entry.error() . functions to report upload events back to the LiveView. Lastly, we pass the uploaders namespace to the LiveSocket constructor to tell phoenix where to find the uploaders returned within the external metadata.","ref":"uploads-external.html#direct-to-s3","title":"External Uploads - Direct to S3","type":"extras"}]